/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		Isearch.cxx
Version:	1.00
Description:	Command-line search utility
Author:		Nassib Nassar, nrn@cnidr.org
@@@*/

#include <stdio.h>
#include <string.h>
#include "db.hxx"
#include "string.hxx"
#include "common.hxx"
#include "squery.hxx"
#include "rset.hxx"

int main(int argc, char** argv) {
	if (argc < 2) {
		printf("\nIsearch, Version 1.00, Copyright (c) 1994 MCNC/CNIDR\n\n");
		printf("Isearch [-d (X)]  // Search index with base name (X).\n");
		printf("        [-q]  // Print results and exit immediately.\n");
		printf("        (X) (Y) (...)  // Search for words (X), (Y), etc.\n\n");
		printf("Example:  Isearch -d BLAKE night \"Tyger*\" \"Lamb*\"\n\n");
		return 0;
	}
	STRING Flag;
	STRING DBName;
	INT QuitFlag = 0;
	INT x = 0;
	INT LastUsed = 0;
	while (x < argc) {
		if (argv[x][0] == '-') {
			Flag = argv[x];
			if (Flag.Equals("-d")) {
				if (++x >= argc) {
					printf("ERROR: No database name specified after -d.\n\n");
					return 0;
				}
				DBName = argv[x];
				LastUsed = x;
			}
			if (Flag.Equals("-q")) {
				QuitFlag = 1;
				LastUsed = x;
			}
		}
		x++;
	}

	if (DBName.Equals("")) {
		printf("ERROR: No database name specified!\n");
		return 0;
	}

	// this should be in the engine
	PFILE fp;
	STRING temp;
	temp = DBName;
	temp.Cat(".inx");
	fp = fopen(temp, "r");
	if (fp) {
		fclose(fp);
	} else {
		printf("ERROR: Database not found.\n");
		return 0;
	}
	temp = DBName;
	temp.Cat(".mdt");
	fp = fopen(temp, "r");
	if (fp) {
		fclose(fp);
	} else {
		printf("ERROR: Database not found.\n");
		return 0;
	}

	x = LastUsed + 1;
	if (x >= argc) {
		return 0;
	}

	INT NumWords = argc - x;
	INT z = x;
	STRING WordList[NumWords];
	for (z=0; z<NumWords; z++) {
		WordList[z] = argv[z+x];
	}

	PDB pdb;
	STRING DBPathName, DBFileName;
	STRING PathName, FileName;
	SQUERY squery;
	PRSET prset;
	RESULT result;
	INT t, n;

	DBPathName = DBName;
	DBFileName = DBName;
	RemovePath(&DBFileName);
	RemoveFileName(&DBPathName);
	pdb = new DB(DBPathName, DBFileName);

	printf("Searching database ");
	DBName.Print();
	printf(":\n");
	STRING QueryString;
	for (z=0; z<NumWords; z++) {
		if (z != 0) {
			QueryString.Cat(' ');
		}
		QueryString.Cat(WordList[z]);
	}
	squery.SetTerm(QueryString);
	prset = pdb->Search(squery);
	n = prset->GetTotalEntries();
	printf("\n%d record(s) matched your query.\n\n", n);

	CHR Selection[80];
//	CHR Buffer[256];
	CHR s[256];
	INT FileNum;
	do {

		if (n != 0) {
			printf("       Score   File\n");
		}
		for (t=1; t<=n; t++) {
			prset->GetEntry(t, &result);
			printf("%4d. ", t);
			printf(" %5i   ", (INT)(result.GetScore()*1000));
			result.GetPathName(&PathName);
			result.GetFileName(&FileName);
			PathName.Print();
			FileName.Print();
			printf("\n");
		}
		if ( (QuitFlag) || (n == 0) ) {
			FileNum = 0;
		} else {
			printf("\nSelect file #: ");
			gets(Selection);
			FileNum = atoi(Selection);
		}
		if ( (FileNum > n) || (FileNum < 0) ) {
			printf("\nSelect a number between 1 and %d.\n", n);
		}
		if ( (FileNum != 0) && (FileNum <= n) && (FileNum >= 1) ) {
			prset->GetEntry(FileNum, &result);

			LONG RecordSize = result.GetRecordSize();
			CHR buf[ RecordSize + 1 ];
			buf[ result.GetRecordData(buf, RecordSize) ] = '\0';
			printf("%s", buf);

/*
			result.GetFullFileName(&FileName);
			FileName.GetCString(s, 255);
			sprintf(Buffer, "more %s", s);
//			system(Buffer);

			PFILE hitfp = fopen(s, "r");
			GPTYPE RecSize = result.GetRecordEnd()-result.GetRecordStart()+1;
			CHR buf[RecSize+1];
			fseek(hitfp, result.GetRecordStart(), 0);
			buf[ fread(buf, 1, RecSize, hitfp) ] = '\0';
			printf("%s", buf);
			fclose(hitfp);
*/

			printf("Press <Return> to select another file: ");
			gets(s);
			printf("\n");
		}
	} while (FileNum != 0);

	delete prset;
	delete pdb;
	return 0;
}
