/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		db.cxx
Version:	1.00
Description:	Class DB
Author:		Nassib Nassar, nrn@cnidr.org
@@@*/

#include <stdio.h>
#include "db.hxx"
#include "common.hxx"

DB::DB(const STRING& NewPathName, const STRING& NewFileName) {
	DbPathName = NewPathName;
	AddTrailingSlash(&DbPathName);
	ExpandFileSpec(&DbPathName);
	DbFileName = NewFileName;
	RemovePath(&DbFileName);
	STRING IndexFN;
	ComposeFN(&IndexFN, ".inx");
	MainIndex = new INDEX(this, IndexFN);
	STRING MDTFN;
	ComposeFN(&MDTFN, ".mdt");
	MainMdt = new MDT();
	MainMdt->LoadTable(MDTFN);
/*
	STRING DFDTFN;
	ComposeFN(&DFDTFN, ".dfm");
	MainDfdt = new DFDT();
	MainDfdt->LoadTable(DFDTFN);
*/
/*
	STRING ChangeLogFN;
	ComposeFN(&ChangeLogFN, ".clg");
	MainChlog = new CHLOG();
*/
	IndexingMemory = 2 * 1024 * 1024;
}

void DB::ComposeFN(PSTRING StringBuffer, const PCHR Suffix) {
	*StringBuffer = DbPathName;
	StringBuffer->Cat(DbFileName);
	StringBuffer->Cat(Suffix);
}

/*

void DB::SetTitle(const STRING& NewTitle) {
	Title = NewTitle;
}

void DB::GetTitle(PSTRING StringBuffer) {
	*StringBuffer = Title;
}

void DB::SetComments(const STRING& NewComments) {
	Comments = NewComments;
}

void DB::GetComments(PSTRING StringBuffer) {
	*StringBuffer = Comments;
}

*/

/*
void DB::AddFieldDef(const DFD& NewDfd) {
	FieldDefs->AddEntry(NewDfd);
}
*/

void DB::AddRecordList(const RECLIST& NewRecordList) {
	MainIndex->AddRecordList(NewRecordList);
}

void DB::IndexingStatus(const INT StatusMessage, const PSTRING FileName,
		const INT WordCount) const {
}

INT DB::GetTotalRecords() const {
	return MainMdt->GetTotalEntries();
}

void DB::SetIndexingMemory(const UINT4 MemorySize) {
	if (MemorySize < (1 * 1024 * 1024)) {
		IndexingMemory = 1 * 1024 * 1024;
		// ER
		return;
	}
	IndexingMemory = MemorySize;
}

UINT4 DB::GetIndexingMemory() const {
	return IndexingMemory;
}

PRSET DB::Search(const SQUERY& SearchQuery) {
	return MainIndex->Search(SearchQuery);
}

PMDT DB::GetMainMdt() {
	return MainMdt;
}

DB::~DB() {
//MainIndex->DumpIndex();
//MainMdt->Dump();
	if (MainIndex)
		delete MainIndex;
	STRING MDTFN;
	ComposeFN(&MDTFN, ".mdt");
	MainMdt->SaveTable(MDTFN);
	if (MainMdt)
		delete MainMdt;
/*
	STRING DFDTFN;
	ComposeFN(&DFDTFN, ".dfm");
//	MainDfdt->SaveTable(DFDTFN);
	if (MainDfdt)
		delete MainDfdt;
*/
/*
	if (MainChlog)
		delete MainChlog;
*/
}



