/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		index.cxx
Version:	1.00
Description:	Class INDEX
Author:		Nassib Nassar, nrn@cnidr.org
@@@*/

// This code is *extremely* unoptimized.  I wanted to stabilize the external
// class interface at the outset, and then fine-tune the internal structure.
// The most obvious optimizations are getting rid of linear searches and
// keeping files open whenever possible.  This internal code will be changing
// a lot in the next few weeks, so please don't rely on anything below the 
// top-level classes.  -Nassib

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "index.hxx"
#include "common.hxx"
#include "irset.hxx"
#include "sw.cxx"

const StopWordSize = 390;

static PCHR MemoryData;
static INT MemoryDataLength;

INDEX::INDEX(const PDBOBJ DbParent, const STRING& NewFileName) {
	Parent = DbParent;
	IndexFileName = NewFileName;
}

int MemIndexCompare(const void* x, const void* y) {
	INT result;
	INT MaxCompare;
	INT t = MemoryDataLength - ((*((PGPTYPE)y)));
	MaxCompare = MemoryDataLength - ((*((PGPTYPE)x)));
	if (t < MaxCompare) {
		MaxCompare = t;
	}
	if (StringCompLength < MaxCompare) {
		MaxCompare = StringCompLength;
	}
	result = strncasecmp(MemoryData + (*((PGPTYPE)x)),
			MemoryData + (*((PGPTYPE)y)), MaxCompare);
	return result;
}

void INDEX::AddRecordList(const RECLIST& NewRecordList) {
	if (NewRecordList.GetTotalEntries() == 0) {
		// ER
		return;
	}
	DataMemorySize = (UINT4)(Parent->GetIndexingMemory() / 1.3);
	IndexMemorySize = (UINT4)(DataMemorySize * 0.3);
	MemoryData = new CHR[DataMemorySize];
	MemoryIndex = new GPTYPE[IndexMemorySize];
	INT FirstRecord = 1;
	INT CurrentRecord;
	INT MemoryIndexLength;
	PFILE DataFp;
	RECORD record;
	STRING s, DataFileName, OldDataFileName;
	MDTREC mdtrec;
	INT DataFileSize;
	INT GpListSize;
	INT x, y;
	INT Error;
	GPTYPE TrueGlobalStart = 0;
	GPTYPE TrueGlobalEnd = 0;
	GPTYPE GlobalStart;
	do {
		CurrentRecord = FirstRecord;
		Error = 0;
		MemoryMdt = new MDT;
		MemoryDataLength = 0;
		MemoryIndexLength = 0;
		do {
			NewRecordList.GetEntry(CurrentRecord, &record);
			record.GetFullFileName(&DataFileName);
			DataFp = fopen(DataFileName, "r");
			if (!DataFp) {
				// ER
				Error = 1;
				break;	// maybe return
			}
			if (record.GetRecordEnd() == 0) {
				fseek(DataFp, 0, 2);
				DataFileSize = ftell(DataFp);
			} else {
				DataFileSize = record.GetRecordEnd() -
						record.GetRecordStart() + 1;
			}
			if (!DataFileName.Equals(OldDataFileName)) {
				TrueGlobalStart = MemoryDataLength;
				fseek(DataFp, 0, 2);
				TrueGlobalEnd = MemoryDataLength +
						ftell(DataFp);
			}
/*
OldDataFileName.Print();
printf("\t");
DataFileName.Print();
printf("\t%d\t%d\n", TrueGlobalStart, TrueGlobalEnd);
*/
			if ( DataFileSize > DataMemorySize ) {
				printf("This version does not support indexing of files larger than allocated\n");
				printf("memory.  Try increasing allocated memory with the -m option.  The next\n");
				printf("version (sometime next week) will fix this problem.\n");
				exit(1);
			}
			if ( (DataFileSize + MemoryDataLength) >
					DataMemorySize ) {
				break;
			}
			fseek(DataFp, record.GetRecordStart(), 0);
			fread(MemoryData + MemoryDataLength, 1, DataFileSize,
					DataFp);
			fclose(DataFp);
			GpListSize = BuildGpList(MemoryDataLength,
					MemoryData,
					MemoryDataLength + DataFileSize,
					MemoryIndex + MemoryIndexLength,
					IndexMemorySize - MemoryIndexLength);
			if (GpListSize == -1) {
				// ??
				break;
			}
			if (!DataFileName.Equals(OldDataFileName)) {
				Parent->IndexingStatus(IndexingStatusReading,
						&DataFileName, 0);
			}
			record.GetKey(&s);
			mdtrec.SetKey(s);
			record.GetPathName(&s);
			mdtrec.SetPathName(s);
			record.GetFileName(&s);
			mdtrec.SetFileName(s);
			mdtrec.SetLocalRecordStart(record.GetRecordStart());
			if ( (record.GetRecordStart() == 0) &&
					(record.GetRecordEnd() == 0) ) {
				mdtrec.SetLocalRecordEnd(DataFileSize);
			} else {
				mdtrec.SetLocalRecordEnd(record.GetRecordEnd());
			}
/*
			mdtrec.SetGlobalFileStart(MemoryDataLength);
			mdtrec.SetGlobalFileEnd(MemoryDataLength +
					DataFileSize - 1);
*/
			mdtrec.SetGlobalFileStart(TrueGlobalStart);
			mdtrec.SetGlobalFileEnd(TrueGlobalEnd);
			MemoryDataLength += DataFileSize;
			MemoryIndexLength += GpListSize;
			MemoryMdt->AddEntry(mdtrec);
			OldDataFileName = DataFileName;
			CurrentRecord++;
		} while (CurrentRecord <= NewRecordList.GetTotalEntries());
		if (Error == 0) {
//printf("   Indexing %d words ...\n", MemoryIndexLength);
			Parent->IndexingStatus(IndexingStatusIndexing, NULL,
					MemoryIndexLength);
			qsort(MemoryIndex, MemoryIndexLength, sizeof(GPTYPE),
					MemIndexCompare);
			GlobalStart = Parent->GetMainMdt()->GetNextGlobal();
//printf("   Merging index ...\n");
			Parent->IndexingStatus(IndexingStatusMerging, NULL, 0);
			MergeIndex(MemoryData, MemoryDataLength, MemoryIndex,
					MemoryIndexLength, GlobalStart);
			y = 0;
			for (x=1; x<=(MemoryMdt->GetTotalEntries()); x++) {
				MemoryMdt->GetEntry(x, &mdtrec);
/*
				mdtrec.SetGlobalFileStart(y + GlobalStart);
				mdtrec.SetGlobalFileEnd(y + GlobalStart + 
						(mdtrec.GetLocalRecordEnd() -
						mdtrec.GetLocalRecordStart()) 
						- 1);
*/
				Parent->GetMainMdt()->AddEntry(mdtrec);
				y += (mdtrec.GetLocalRecordEnd() -
						mdtrec.GetLocalRecordStart());
			}
		}
		FirstRecord = CurrentRecord;
		delete MemoryMdt;
	} while (FirstRecord <= NewRecordList.GetTotalEntries());
	delete [] MemoryData;
	delete [] MemoryIndex;
}

void INDEX::MergeIndex(PCHR MemoryData, INT MemoryDataLength,
		PGPTYPE MemoryIndex, INT MemoryIndexLength,
		GPTYPE GlobalStart) {
	// Open index file
	PFILE fp = fopen(IndexFileName, "r");
	if (fp) {
//		GPTYPE GlobalStart = Parent->GetMainMdt()->GetGlobalEnd() + 1;
//		mdtrec.SetGlobalFileStart(GlobalStart);
//		mdtrec.SetGlobalFileEnd(GlobalStart+MemoryDataLength-1);
		// Merge index
		STRING NewIndexFN;
		Parent->ComposeFN(&NewIndexFN, ".tmp");
		PFILE fpNew = fopen(NewIndexFN, "w");
		GPTYPE x, y;
		INT w;
		GPTYPE c;
		INT b, g;
		GPTYPE DiskGP;
		PFILE FileFP;
		CHR DiskData[StringCompLength+1];

		INT firsttime = 1;
		x = 0;
		g = 1;
		b = 0;
		do {

			if (!firsttime) {
				b = 0;
				c = 0;
				do {
					if (c < StringCompLength) {
						b = ( toupper(
							MemoryData[MemoryIndex[x]+c]) -
							toupper(DiskData[c]) );
					}
					c++;
				} while ( (c < StringCompLength) && (b == 0) );
				if (b >= 0) {
					fwrite(&DiskGP, 1, sizeof(GPTYPE),
							fpNew);
				} else {
					y = GlobalStart + MemoryIndex[x];
					fwrite(&y, 1, sizeof(GPTYPE), fpNew);
				}
			}
			if ( (b >= 0) || (firsttime) ) {
				g = fread(&DiskGP, 1, sizeof(GPTYPE), fp);
				if (g) {
					FileFP = Parent->GetMainMdt()->GetFilePointer(DiskGP);
					w = fread(DiskData, 1, StringCompLength,
							FileFP);
					DiskData[w] = '\0';
					fclose(FileFP);
				}
			}
			if (b < 0)
				x++;

			firsttime = 0;
		} while ( (g) && (x<MemoryIndexLength) );

		while (x<MemoryIndexLength) {
			y = GlobalStart + MemoryIndex[x];
			fwrite(&y, 1, sizeof(GPTYPE), fpNew);
			x++;
		}
		while ( fread(&DiskGP, 1, sizeof(GPTYPE), fp) > 0 ) {
			fwrite(&DiskGP, 1, sizeof(GPTYPE), fpNew);
		}

		fclose(fpNew);
		fclose(fp);
		PCHR TempFN, Temp2FN;
		TempFN = IndexFileName.NewCString();
		Temp2FN = NewIndexFN.NewCString();
		rename(Temp2FN, TempFN);
		delete [] TempFN;
		delete [] Temp2FN;
	} else {
//		mdtrec.SetGlobalFileStart(0);
//		mdtrec.SetGlobalFileEnd(MemoryDataLength-1);
		fp = fopen(IndexFileName, "w");
		// Dump out index
		fwrite(MemoryIndex, 1, MemoryIndexLength*sizeof(GPTYPE), fp);
		fclose(fp);
	}
//	Parent->GetMainMdt()->AddEntry(mdtrec);
}

INT INDEX::IsStopWord(PCHR WordStart, INT WordMaximum) {
	INT x = 0;
	INT WordLength = 0;
	while ( (WordLength < WordMaximum) &&
				(isalnum(WordStart[WordLength])) ) {
		WordLength++;
	}
	INT High = StopWordSize;
	INT Low = 0;
	INT Middle = 0;
	INT Old;
	do {
		Old = Middle;
		Middle = (Low + High) / 2;
		x = strncasecmp(WordStart, stoplist[Middle], WordLength);
		if (x == 0) {
			return 1;
		}
		if (x < 0) {
			High = Middle;
		}
		if (x > 0) {
			Low = Middle;
		}
	} while (Middle != Old);
	return 0;
}

GPTYPE INDEX::BuildGpList(INT StartingPosition, PCHR MemoryData,
		INT MemoryDataLength, PGPTYPE MemoryIndex,
		INT MemoryIndexLength) {
	INT GpListSize = 0;
	INT Position = StartingPosition;
	while (Position < MemoryDataLength) {
		while ( (Position < MemoryDataLength) &&
				(!isalnum(MemoryData[Position])) ) {
			Position++;
		}
		if ( (Position < MemoryDataLength) &&
				(!IsStopWord(MemoryData + Position,
						MemoryDataLength)) ) {
			MemoryIndex[GpListSize++] = Position;
		}
		while ( (Position < MemoryDataLength) &&
				(isalnum(MemoryData[Position])) ) {
			Position++;
		}
	}
	return GpListSize;
}

PRSET INDEX::Search(const SQUERY& SearchQuery) {
	INT x;
	STRING QueryTerm;
	STRING SingleWord;
	PIRSET MainIrset = new IRSET(Parent);
	PIRSET NewIrset;
	SearchQuery.GetTerm(&QueryTerm);
	while ( !QueryTerm.Equals("") ) {
		x = QueryTerm.SearchReverse(' ');
		SingleWord = QueryTerm;
		SingleWord.EraseBefore(x+1);
		if (x == 0) {
			QueryTerm = "";
		} else {
			QueryTerm.EraseAfter(x-1);
		}
		NewIrset = TermSearch(SingleWord);
		NewIrset->ComputeScores();;
		MainIrset->Or(*NewIrset);
		delete NewIrset;
	}
	MainIrset->SortByScore();
	PRSET prset = MainIrset->GetRset();
	delete MainIrset;
	return prset;
}

//private
void INDEX::GetIndirectBuffer(const GPTYPE Gp, PCHR Buffer) {
	MDTREC Mdtrec;
	STRING FileName;
	PFILE Fp;
	INT x;
	Parent->GetMainMdt()->GetMdtRecord(Gp, &Mdtrec);
	Mdtrec.GetFullFileName(&FileName);
	Fp = fopen(FileName, "r");
	fseek(Fp, Gp - Mdtrec.GetGlobalFileStart(), 0);
	x = fread(Buffer, 1, StringCompLength, Fp);
	fclose(Fp);
	Buffer[x] = '\0';
}

PIRSET INDEX::TermSearch(const STRING& QueryTerm) {
	// binary search
	PFILE fpi = fopen(IndexFileName, "r");
	GPTYPE gp;
	INT ip, oip, maxip, low, high;
	INT x, z;
	CHR Buffer[StringCompLength+1];
	CHR Term[StringCompLength+1];
	INT done = 0;
	fseek(fpi, 0, 2);
	maxip = (ftell(fpi) / sizeof(GPTYPE)) - 1;
	high = maxip;
	ip = high / 2;
	low = 0;
	INT hit;
	do {
		hit = 0;
		oip = ip;
		fseek(fpi, ip * sizeof(GPTYPE), 0);
		x = fread(&gp, 1, sizeof(GPTYPE), fpi);
		if (x) {
/*
			Parent->GetMainMdt()->GetMdtRecord(gp, &mdtrec);
			mdtrec.GetFullFileName(&FileName);
			fpd = fopen(FileName, "r");
			fseek(fpd, gp - mdtrec.GetGlobalFileStart(), 0);
			y = fread(Buffer, 1, StringCompLength, fpd);
			fclose(fpd);
			Buffer[y] = '\0';
*/
			GetIndirectBuffer(gp, Buffer);

/*
		INT w = 0;
		w = 0;
		printf("%d: ", ip);
		while (isalnum(Buffer[w]))
			printf("%c", Buffer[w++]);
		printf("\t");
//		FileName.Print();
*/

			QueryTerm.GetCString(Term, sizeof(Term));
			if (Term[strlen(Term)-1] == '*') {
				Term[strlen(Term)-1] = '\0';
				z = strncasecmp(Term, Buffer, strlen(Term));
			} else {
				z = strncasecmp(Term, Buffer, strlen(Term));
				if (z == 0) {
					if (isalnum(Buffer[strlen(Term)])) {
						z = -1;
					}
				}
			}
//cout<<"("<<z<<")\n";
			if (z == 0) {
				done = 1;
				hit = 1;
			}
			if (z < 0) {
				high = ip;
			}
			if (z > 0) {
				low = ip;
			}
			ip = (low + high) / 2;
			if (ip < 0) {
				ip = 0;
			}
			if (ip > maxip) {
				ip = maxip;
			}
		} else {
			ip = 0;
			done = 1;
		}
	} while ( (!done) && (ip != oip) );

	if (!hit) {
		PIRSET pirset = new IRSET(Parent);
		return pirset;
	}

	// bracket hits (linear!)
	INT first, last;
	first = last = ip;
	// find first
	do {
		first--;
		fseek(fpi, first * sizeof(GPTYPE), 0);
		x = fread(&gp, 1, sizeof(GPTYPE), fpi);
		if (x) {
/*
			Parent->GetMainMdt()->GetMdtRecord(gp, &mdtrec);
			mdtrec.GetFullFileName(&FileName);
			fpd = fopen(FileName, "r");
			fseek(fpd, gp - mdtrec.GetGlobalFileStart(), 0);
			y = fread(Buffer, 1, StringCompLength, fpd);
			fclose(fpd);
			Buffer[y] = '\0';
*/
			GetIndirectBuffer(gp, Buffer);
			QueryTerm.GetCString(Term, sizeof(Term));
			if (Term[strlen(Term)-1] == '*') {
				Term[strlen(Term)-1] = '\0';
				z = strncasecmp(Term, Buffer, strlen(Term));
			} else {
				z = strncasecmp(Term, Buffer, strlen(Term));
				if (z == 0) {
					if (isalnum(Buffer[strlen(Term)])) {
						z = -1;
					}
				}
			}
		}
	} while ( (z == 0) && (first > 0) );
	// find last
	do {
		last++;
		fseek(fpi, last * sizeof(GPTYPE), 0);
		x = fread(&gp, 1, sizeof(GPTYPE), fpi);
		if (x) {
/*
			Parent->GetMainMdt()->GetMdtRecord(gp, &mdtrec);
			mdtrec.GetFullFileName(&FileName);
			fpd = fopen(FileName, "r");
			fseek(fpd, gp - mdtrec.GetGlobalFileStart(), 0);
			y = fread(Buffer, 1, StringCompLength, fpd);
			fclose(fpd);
			Buffer[y] = '\0';
*/
			GetIndirectBuffer(gp, Buffer);
			QueryTerm.GetCString(Term, sizeof(Term));
			if (Term[strlen(Term)-1] == '*') {
				Term[strlen(Term)-1] = '\0';
				z = strncasecmp(Term, Buffer, strlen(Term));
			} else {
				z = strncasecmp(Term, Buffer, strlen(Term));
				if (z == 0) {
					if (isalnum(Buffer[strlen(Term)])) {
						z = -1;
					}
				}
			}
		}
	} while ( (z == 0) && (last < maxip) );

	first++;
	last--;

	// Build result set
	IRESULT iresult;
	PIRSET pirset = new IRSET(Parent);
	INT w;
	for (ip=first; ip<=last; ip++) {
		fseek(fpi, ip * sizeof(GPTYPE), 0);
		x = fread(&gp, 1, sizeof(GPTYPE), fpi);
		if (x) {
			w = Parent->GetMainMdt()->GetMdtIndex(gp);
			iresult.SetMdtIndex(w);
			iresult.SetHitCount(1);
			iresult.SetScore(0);
			pirset->AddEntry(iresult, 1);
		}
	}
	fclose(fpi);
	return pirset;
}

void INDEX::DumpIndex() {
	PFILE fpi = fopen(IndexFileName, "r");
	PFILE fpd;
	GPTYPE gp;
	MDTREC mdtrec;
	INT x;
	CHR Buffer[17];
	STRING FileName;
	while (fread(&gp, 1, sizeof(GPTYPE), fpi) > 0) {
		printf("%d\t", gp);
		Parent->GetMainMdt()->GetMdtRecord(gp, &mdtrec);
		mdtrec.GetFullFileName(&FileName);
		fpd = fopen(FileName, "r");
		fseek(fpd, gp - mdtrec.GetGlobalFileStart() +
				mdtrec.GetLocalRecordStart(), 0);
		x = fread(Buffer, 1, 16, fpd);
		fclose(fpd);
		Buffer[x] = '\0';
		x = 0;
		while (isalnum(Buffer[x]))
			printf("%c", Buffer[x++]);
		printf("\t");
		FileName.Print();
		printf("\n");
	}
	fclose(fpi);
}

INDEX::~INDEX() {
}
