/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		string.cxx
Version:	1.00
Description:	Class STRING
Author:		Nassib Nassar, nrn@cnidr.org
@@@*/

#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <unistd.h>
#include "string.hxx"

STRING::STRING() {
	Length = 0;
	Buffer = 0;
}

STRING::STRING(const PCHR CString) {
	Assign(CString);
}

STRING::STRING(const PCHR NewBuffer, const STRINGINDEX BufferLength) {
	if ( (NewBuffer) && (BufferLength != 0) ) {
		Length = BufferLength;
		Buffer = new CHR[Length];
		memcpy(Buffer, NewBuffer, Length);
	} else {
		Length = 0;
		Buffer = 0;
	}
}

STRING& STRING::operator=(const PCHR CString) {
	if (Buffer)
		delete [] Buffer;
	Assign(CString);
	return *this;
}

STRING& STRING::operator=(const STRING& OtherString) {
	if (Buffer)
		delete [] Buffer;
	if (OtherString.Buffer) {
		Length = OtherString.Length;
		Buffer = new CHR[Length];
		memcpy(Buffer, OtherString.Buffer, Length);
	} else {
		Length = 0;
		Buffer = 0;
	}
	return *this;
}

void STRING::Assign(const PCHR CString) {
	if (*CString != '\0') {
		Length = strlen(CString);
		Buffer = new CHR[Length];
		memcpy(Buffer, CString, Length);
	} else {
		Length = 0;
		Buffer = 0;
	}
}

INT STRING::Equals(const STRING& OtherString) {
	if (Length != OtherString.Length) {
		return 0;
	}
	return ( memcmp(Buffer, OtherString.Buffer, Length) == 0 );
}

INT STRING::Equals(const PCHR CString) {
	if (Length != strlen(CString)) {
		return 0;
	}
	return ( memcmp(Buffer, CString, Length) == 0 );
}

void STRING::Print() const {
	STRINGINDEX x;
	for (x=0; x<Length; x++)
		printf("%c", Buffer[x]);
}

STRINGINDEX STRING::GetLength() const {
	return Length;
}

CHR STRING::GetChr(STRINGINDEX Index) const {
	if ( (Index > 0) && (Index <= Length) )
		return Buffer[Index-1];
	else
		return 0;	// generate ER
}

void STRING::Cat(const CHR Character) {
	PCHR Temp = new CHR[Length+1];
	if (Buffer) {
		memcpy(Temp, Buffer, Length);
		delete [] Buffer;
	}
	Temp[Length] = Character;
	Buffer = Temp;
	Length = Length + 1;
}

void STRING::Cat(const PCHR CString) {
	STRINGINDEX StringLength = strlen(CString);
	if (StringLength == 0) {
		return;
	}
	PCHR Temp = new CHR[Length+StringLength];
	if (Buffer) {
		memcpy(Temp, Buffer, Length);
		delete [] Buffer;
	}
	memcpy(Temp+Length, CString, StringLength);
	Buffer = Temp;
	Length = Length + StringLength;
}

void STRING::Cat(const STRING& OtherString) {
	STRINGINDEX StringLength = OtherString.Length;
	if (StringLength == 0) {
		return;
	}
	PCHR Temp = new CHR[Length+StringLength];
	if (Buffer) {
		memcpy(Temp, Buffer, Length);
		delete [] Buffer;
	}
	memcpy(Temp+Length, OtherString.Buffer, StringLength);
	Buffer = Temp;
	Length = Length + StringLength;
}

STRINGINDEX STRING::SearchReverse(PCHR CString) const {
	STRINGINDEX x;
	INT n = strlen(CString);
	if (n > Length)
		return 0;
	if (n == 0)
		return 0;	// Generate warning ER?
	x = Length - n + 1;
	if (x > 0)
		do {
			x--;
			if (strncmp(CString, Buffer+x, n) == 0)
				return (x + 1);
		} while (x > 0);
	return 0;
}

STRINGINDEX STRING::SearchReverse(CHR Character) const {
	STRINGINDEX x;
	if (Length == 0)
		return 0;
	x = Length;
	if (x > 0)
		do {
			x--;
			if (Buffer[x] == Character)
				return (x+1);
		} while (x > 0);
	return 0;
}

STRINGINDEX STRING::Search(CHR Character) const {
	STRINGINDEX x;
	if (Length == 0)
		return 0;
	x = 0;
	do {
		if (Buffer[x] == Character)
			return (x+1);
		x++;
	} while (x < Length);
	return 0;
}

void STRING::EraseBefore(STRINGINDEX Index) {
	PCHR Temp;
	if ( (Index <= 1) || (Length == 0) ) {
		return;
	}
	if (Index > Length) {
		if (Buffer)
			delete [] Buffer;
		Buffer = 0;
		Length = 0;
		return;
	}
	Length = Length - Index + 1;
	Temp = new CHR[Length];
	memcpy(Temp, (Buffer+Index-1), Length);
	delete [] Buffer;
	Buffer = Temp;
	return;
}

void STRING::EraseAfter(STRINGINDEX Index) {
	if (Length == 0) {
		return;
	}
	if (Index < 1) {
		if (Buffer) {
			delete [] Buffer;
		}
		Buffer = 0;
		Length = 0;
		return;
	}
	if (Length > Index) {
		PCHR Temp = new CHR[Index];
		memcpy(Temp, Buffer, Index);
		delete [] Buffer;
		Buffer = Temp;
		Length = Index;
	}
}

void STRING::GetCString(PCHR CStringBuffer, const INT BufferSize) const {
	STRINGINDEX ShortLength = BufferSize - 1;
	if (Length < ShortLength)
		ShortLength = Length;
	if (Buffer)
		memcpy(CStringBuffer, Buffer, ShortLength);
	CStringBuffer[ShortLength] = '\0';
}

PCHR STRING::NewCString() const {
	PCHR p = new CHR[Length+1];
	if (Buffer) {
		memcpy(p, Buffer, Length);
	}
	p[Length] = '\0';
	return p;
}

void STRING::ReadFile(const STRING& FileName) {
	PFILE fp;
	if (Buffer)
		delete [] Buffer;
	fp = fopen(FileName, "r");
	fseek(fp, 0, 2);
	Length = ftell(fp);
	if (Length) {
		fseek(fp, 0, 0);
		Buffer = new CHR[Length];
		fread(Buffer, 1, Length, fp);
	} else {
		Buffer = 0;
	}
	fclose(fp);
}

STRING::~STRING() {
	if (Buffer)
		delete [] Buffer;
}

PFILE fopen(const STRING& FileName, const PCHR Type) {
	PFILE fp;
	PCHR fn = FileName.NewCString();
//	CHR fn[256];
//	FileName.GetCString(fn, 255);
	fp = fopen(fn, Type);
	delete [] fn;
	return fp;
}

INT unlink(const STRING& FileName) {
	INT ReturnCode;
	PCHR fn = FileName.NewCString();
	ReturnCode = unlink(fn);
	delete [] fn;
	return ReturnCode;
}
