/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:		string.hxx
Version:	1.00
Description:	Class STRING
Author:		Nassib Nassar, nrn@cnidr.org
@@@*/

#ifndef STRING_HXX
#define STRING_HXX

#include "gdt.h"

typedef UINT4 STRINGINDEX;
typedef STRINGINDEX* PSTRINGINDEX;

class STRING {
public:
	STRING();
	STRING(const PCHR CString);
	STRING(const PCHR NewBuffer, const STRINGINDEX BufferLength);
	STRING& operator=(const PCHR CString);
	STRING& operator=(const STRING& OtherString);
	INT Equals(const STRING& OtherString);
	INT Equals(const PCHR CString);
	void Print() const;
	STRINGINDEX GetLength() const;
	CHR GetChr(STRINGINDEX Index) const;
	void Cat(const CHR Character);
	void Cat(const PCHR CString);
	void Cat(const STRING& OtherString);
	STRINGINDEX SearchReverse(PCHR CString) const;
	STRINGINDEX SearchReverse(CHR Character) const;
	STRINGINDEX Search(CHR Character) const;
	void EraseBefore(STRINGINDEX Index);
	void EraseAfter(STRINGINDEX Index);
	void GetCString(PCHR CStringBuffer, const INT BufferSize) const;
	PCHR NewCString() const;	// Remember to delete [] !!
	void ReadFile(const STRING& FileName);
	~STRING();
private:
	void Assign(const PCHR CString);
	STRINGINDEX Length;
	PCHR Buffer;
//	static EREG ereg;
};

typedef STRING* PSTRING;

PFILE fopen(const STRING& FileName, const PCHR Type);
INT unlink(const STRING& FileName);

#endif /* STRING_HXX */
