/*@@@
File:           netbox.h
Version:        1.01.01
Description:    Socket-interfacing network functions
Author:         Nassib Nassar; Kevin Gamiel
@@@*/

/*************************************************************************

Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

*************************************************************************/

#ifndef NETBOX_H
#define NETBOX_H

#include "platform.h"

/* Global definitions */

#ifdef PLATFORM_WINDOWS
#include "winsock.h"
#define MAXINT 32767
#else
#define MAXINT 2147483647
#define SOCKET int
#define SOCKADDR_IN struct sockaddr_in
#define LPHOSTENT struct hostent*
#define LPSERVENT struct servent*
#define INVALID_SOCKET (SOCKET)(-1)
#define SOCKET_ERROR (-1)
#define closesocket(s) close(s)
#endif

#ifndef FAR
#define FAR
#endif

/* Public typedefs & vars */
#define NB_INETD 0
#define NB_COMMANDLINE	1

typedef struct tagNETBOXPROFILE {
	SOCKET Socket;		/* Resulting value */
	long TimeOutSec;	/* Must be set by user */
	long TimeOutUSec;	/* Must be set by user */
	char HostName[256];	/* Must be set by user */
	int ServerOwner;	/* Resulting value. 0=INETD, 1=Command Line */
	char PeerName[256];	/* Resulting value */
	int Port;		/* Must be set by user */
} NETBOXPROFILE;

typedef struct tagNETBOXPROFILE FAR *PNETBOXPROFILE;

/* Public function prototypes */
void netbox_Destroy(PNETBOXPROFILE P);
char FAR *netbox_SetHostName(PNETBOXPROFILE P, char FAR* Name);
int netbox_SetPort(PNETBOXPROFILE P, int Port);
char FAR *netbox_GetHostName(PNETBOXPROFILE P);
int netbox_GetPort(PNETBOXPROFILE P);
long netbox_SetTimeOutSec(PNETBOXPROFILE P, long Seconds);
long netbox_SetTimeOutUSec(PNETBOXPROFILE P, long USeconds);
int netbox_WSAStartup(void);
void netbox_WSACleanup(void);
int netbox_SetBufferSize(PNETBOXPROFILE profile, int BufferSize);
int netbox_Open(PNETBOXPROFILE profile);
int netbox_Close(PNETBOXPROFILE profile);
int netbox_SendBuffer(PNETBOXPROFILE profile, char FAR* Buffer,
		int BufferSize);
int netbox_iGetBuffer(PNETBOXPROFILE profile, char FAR* Buffer,
		int BufferSize);
long netbox_GetBuffer(PNETBOXPROFILE profile, char FAR* Buffer,
		long BufferSize);
int netbox_WaitForData(PNETBOXPROFILE profile);
int netbox_StartServer(PNETBOXPROFILE Profile);
PNETBOXPROFILE netbox_AcceptClient(PNETBOXPROFILE Profile);
int netbox_CurrentStatus(PNETBOXPROFILE Profile);
int netbox_GetLine(PNETBOXPROFILE Profile, char *Buffer, int MaxLen);

#endif
