/*@@@
File:           sapitest.c
Version:        1.00
Description:    Example code showing libsapi usage
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/*
This module provides a working example of using the CNIDR Search API.

This application generates a sample database configuration file, then 
reads values from that in order to initialize various database systems.

See online docs for more info:
	<URL:http://vinca.cnidr.org/software/sapi/sapi.html>

Make sure you change the INSTALL_DIR path below before compiling.
*/
#define INSTALL_DIR "/local/kgamiel/pub/sapi1.00/API"

#include <stdio.h>
#include "sapi.h"

void sapitest_BuildSampleConfigFile(PCHR File);

int main(int argc, char **argv)
{
	PSAPI sapi;

	/*
	First, let's build an example SAPI Configuration File (which of
	course would normally be maintained by the server administrator)
	*/
	sapitest_BuildSampleConfigFile("sapitest.ini");

	/* 
	Create an instance of a SAPI object based on the SAPI Configuration
	File we just created.  This function does all the dirty work of
	loading the contents of the configuration file into memory and
	setting up all the needed data structures.
	*/
	if((sapi = sapi_CreateInitFile("sapitest.ini","Default"))==NULL) {
		printf("Failed to initialized the search api\n");
		exit(1);
	}

	/* 
	OK, for starters, let's list some general information about the
	current status of the SAPI.
	*/
	printf("# Databases Available:\t%d\n", sapi_GetDatabaseCount(sapi));

	/* We're done with the SAPI, so clean up. */
	sapi_Destroy(sapi);

	exit(0);
}

/* 
Builds an example SAPI configuration file
*/
void sapitest_BuildSampleConfigFile(PCHR File)
{
	FILE *fp;

	if((fp=fopen(File,"w"))==NULL) {
		perror("sapitest_BuildSampleConfigFile");
		return;
	}

	fprintf(fp, "[Default]\n");
	fprintf(fp, "DBList=ManPages,DB2\n");
	fprintf(fp, "[ManPages]\n");
	fprintf(fp, "Type=SCRIPT\n");
	fprintf(fp, "Location=%s/ManPages.sh\n", INSTALL_DIR);
	
	fprintf(fp, "[DB2]\n");
	fprintf(fp, "FieldCount=2\n");
	fprintf(fp, "Field1=URN	yes	yes\n");
	fprintf(fp, "Field2=Title	yes	yes\n");
	fclose(fp);
}

