/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           sapi.c
Version:        1.00
Description:    Search API
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/*
Define which search engine to compile into zserver via the Makefile
*/
#include "gdt.h"
#include <stdio.h>

#ifdef SAPI_ISEARCH
#include <string.h>
#include "db.hxx"
#include "string.hxx"
#include "common.hxx"
#include "squery.hxx"
#include "rset.hxx"
#endif

#ifdef SAPI_CTSS
extern PCHR printRset();
#endif

#ifdef SAPI_SCRIPT
#include "../script/script.h"
#endif

#ifdef SAPI_CONQUEST
#include <string.h>
#include <ctype.h>
#ifndef UNIX_VERSION
  #include <io.h>
#endif
#include "tsrbasic.h"
#include "hl_api.h"
#define EXTFLG
#include "conquest.h"
#endif /* SAPI_CONQUEST */

#ifdef SAPI_NTN
#include "ntn.h"
#endif

#ifndef NEWSTRUCT
#define NEWSTRUCT(a) (a*)calloc(1,sizeof(a))
#endif

#ifndef NEWSTRING
#define NEWSTRING(a) (PCHR)calloc(1,a)
#endif

#ifndef FREE
#define FREE(a) free(a)
#endif

#include "ini.h"
#include "list.h"
#include "strbox.h"
#include "sapi.h"

/* Error messages */
#define SAPI_NOMEM 0
#define SAPI_BADFILE 1
#define SAPI_BADDBLIST 2
 
PCHR SAPI_ERRORMSG[]={
        "Out of memory",
        "Bad sapi.ini file ",
        "No databases in DBList "
};

PSAPI sapi_Create()
{
	PSAPI s;

	if((s=NEWSTRUCT(SAPI))==NULL) {
		fprintf(stderr, "sapi_Create:Out of memory\n");
		return NULL;
	}

	s->ConfigFile = NULL;
	s->Database = list_Create();

	return s;
}

void sapi_Destroy(PSAPI s)
{
	if(s==NULL)
		return;

	if(s->ConfigFile!=NULL)
		FREE(s->ConfigFile);
	if(s->Database) {
		/* KAG punting on freeing list right now ;-)*/
	}

	FREE(s);
}

PSAPI sapi_CreateInitFile(PCHR File, PCHR Group)
{
	PSAPI s;
	FILE *fp;
	CHR Temp[MAXPROFILESTR];
	CHR TempDB[MAXPROFILESTR];
	PSAPI_DBASE DB;
	INT i;

	/* Create a new SAPI structure */
	if((s=sapi_Create())==NULL)
		return NULL;

	/* Copy the configuration file name */
	if((s->ConfigFile=NEWSTRING(strlen(File)+1))==NULL) {
		sapi_Error("sapi_CreateInit", SAPI_NOMEM, "");
		sapi_Destroy(s);
		return NULL;
	}
	strcpy(s->ConfigFile, File);

	/* Does the file exist?  */
	if((fp=fopen(File, "r"))==NULL) {
		sapi_Error("sapi_CreateInit", SAPI_BADFILE, File);
		return NULL;
	}
	fclose(fp);

	/* Open the file */
	if ( CacheProfileFile(File) == 0 ) {
		sapi_Error("sapi_CreateInit", SAPI_BADFILE, File);
		sapi_Destroy(s);
		return NULL;
	}
	
	/* Load the SAPI structures with contents of file */

	/* Read the DBList entry to get list of all database names */
	GetPrivateProfileString(Group, "DBList", "", Temp,sizeof(Temp),File);
	if(Temp[0]=='\0') {
		sapi_Error("sapi_CreateInit", SAPI_BADDBLIST, File);
		sapi_Destroy(s);
		return NULL;
	}
	
	/* Step through each database name in the list */
	for(i=1;i<=strbox_itemCount(Temp);i++) {
		strbox_item(Temp, i, TempDB);	
		if(Temp[0]=='\0') {
			fprintf(stderr,
				"sapi_CreateInitFile:Bad item[%i] in DBList\n",
				i);
			continue;
		}
		DB = dbase_CreateInitFile(File, TempDB);
		if(DB!=NULL)
			sapi_AddDatabase(s, DB);
	}

	/* Close file */
	if (UncacheProfileFile() == 0)
		AbortProfileChanges();

	return s;
}

void sapi_AddDatabase(PSAPI s, PSAPI_DBASE d)
{
	if(s==NULL)
		return;

	if(s->Database==NULL)
		return;

	if(d==NULL)
		return;

	list_InsertAfter(s->Database, list_Last(s->Database),d, LIST_NEAR);
}

INT4 sapi_GetDatabaseCount(PSAPI s) 
{
	if(s==NULL)
		return -1;

	if(s->Database==NULL)
		return -1;

	return list_Length(s->Database);	
}

/*
This only supports a Query of type Stack.  See SAPI_QUERY structure and
methods for details.
*/
PSAPI_RESULTSET sapi_Search(PSAPI s, PSAPI_DBASE Database, PSAPI_QUERY Query)
{
	PSAPI_RESULTSET r;

	if((s==NULL)||(Database==NULL)||(Query==NULL)||(Query->Stack==NULL)) {
		fprintf(stderr, "sapi_Search:Bad input to call\n");
		return NULL;
	}

	if((r=ResultSet_Create())==NULL)
		return NULL;

	ResultSet_SetDatabase(r,Database);

#ifdef SAPI_DEBUG
	fprintf(stderr, "sapi_Search()\n");
	fprintf(stderr, "\tDatabase Name:%s\n", Database->Name);
	fprintf(stderr, "\tLocation:%s\n", Database->Location);
	fprintf(stderr, "\tType:%i\n", Database->Type);
	fprintf(stderr, "\tResults:%i\n", Database->Results);
#endif
	switch(Database->Type) {
#ifdef SAPI_ISEARCH
		case SAPI_DBTYPE_ISEARCH: {
			SQUERY query;
			/*PRSET prset;*/
			RESULT result;
			INT hits;
			STRING s;
			PCHR Term;
	
			/* Has database object been created? */
			if(Database->pdb==NULL) {
				fprintf(stderr,
					"sapi_Search:Invalid database\n");
				return NULL;
			}

			/* Get single term */
			PQUERYATOM Atom = (PQUERYATOM)stack_Top(Query->Stack);
			if(Atom == NULL) {
				fprintf(stderr, "sapi_Search:Null query\n");
				return NULL;
			}
			Term = NEWSTRING(strlen(Atom->Data));
			strcpy(Term, Atom->Data);
			s = Term;
			query.SetTerm(s);

#ifdef SAPI_DEBUG
	fprintf(stderr, "sapi_Search()\n");
	fprintf(stderr, "\tISEARCH search term:%s\n", Term);
#endif
			/* Execute the search */
			r->prset=Database->pdb->Search(query);

			/* How many hits? */
			r->HitCount = r->prset->GetTotalEntries();		
			fprintf(stderr, "sapi_Search:%d hits\n",r->HitCount);

#ifdef SAPI_DEBUG
	fprintf(stderr, "sapi_Search()\n");
	fprintf(stderr, "\tHitCount:%d\n", r->HitCount);
#endif
			break;
		}
#endif
#ifdef SAPI_CGI
		case SAPI_DBTYPE_CGI:
			fprintf(stderr, "sapi_Search:No CGI yet\n");
			r->HitCount = 0;
			break;
#endif
#ifdef SAPI_SCRIPT
		case SAPI_DBTYPE_SCRIPT: {
			/* 
			SCRIPT search engine is a pretty stupid one.
			You need to feed it the terms as a single string
			on the command line, so piece that string
			together now.
			*/	
			PCHR Term;
			PQUERYATOM Atom = (PQUERYATOM)stack_Top(Query->Stack);
			
			if(Atom == NULL) {
				fprintf(stderr, "sapi_Search:Null query\n");
				return NULL;
			}
			
			Term=NEWSTRING(strlen(Atom->Data)+1);
			strcpy(Term, Atom->Data);

			/* Now call the SCRIPT search engine */
			r->HitCount = script_Search(Database->Location,
				Database->Results, Term, &r->Diagnostic);

			break;
		}	
#endif
#ifdef SAPI_CTSS
		case SAPI_DBTYPE_CTSS: {
			r->HitCount = doRankedSearch((PCHR)"",
				Database->Name, Database->Location, 
				"Default", Query->Stack);
			break;
		}
#endif
#ifdef SAPI_CONQUEST
		case SAPI_DBTYPE_CONQUEST: {
			ResultSet_Destroy(r);
			r = conquest_Search(s, Database, Query->Stack);
			break;
		}
#endif
#ifdef SAPI_DBTYPE_ZAGENT
		case SAPI_DBTYPE_ZAGENT: {
			/* 
			Location is database path (no trailing slash)
			Results is Zagent path (full, including Zagent)
			*/	
			CHR Cmd[1024];
			PCHR Term;
			PQUERYATOM Atom = (PQUERYATOM)stack_Top(Query->Stack);
			CHR RSName[16],SessionID[16];

			if(Atom == NULL) {
				fprintf(stderr, "sapi_Search:Null query\n");
				return NULL;
			}
			
			Term=NEWSTRING(strlen(Atom->Data)+1);
			strcpy(Term, Atom->Data);

			sprintf(RSName, "RS%i",getpid());			
			sprintf(SessionID, "SID%i",getpid());			
			sprintf(Cmd,"Operation=Search&DBname=%sDBpath=%s&ResultSetName=%s&SessionID=%s&Query=%s", Database->Name, Database->Location,
				RSName, SessionID, Term);
printf("%s\n", Cmd);
			r->HitCount=0;
			break;
		}
#endif
		default:
			fprintf(stderr, "sapi_Search:Bad DB type[%i]\n",
				Database->Type);
			return NULL;
	}

	return r;
}

/*
Initializes the database and returns a pointer to the SAPI_DBASE object or NULL.
*/
PSAPI_DBASE sapi_GetDatabase(PSAPI s, PCHR DatabaseName)
{
	POSITION *Pos;
	PSAPI_DBASE DBase;
	INT i;

	if(s==NULL)
		return NULL;

	if((DatabaseName==NULL)||(DatabaseName[0]=='\0'))
		return NULL;

	if(s->Database == NULL)
		return NULL;

	Pos = list_First(s->Database);

	/* Step through all database names in list */
	for(i=1;i<=list_Length(s->Database);i++) {
		DBase = (PSAPI_DBASE)list_Retrieve(s->Database, Pos);
		if(DBase == NULL) {
			continue;
		}
		if(!strcasecmp(DBase->Name, DatabaseName))
			break;
		Pos = list_Next(s->Database, Pos);
	}
	if(DBase==NULL) {
		fprintf(stderr, "sapi_GetDatabase:Database unavailable:[%s]\n", 
			DatabaseName);
		return NULL;
	}

	/* Initialize/Validate database */
	switch(DBase->Type) {
		case SAPI_DBTYPE_ISEARCH: {
			STRING DBName;
			STRING DBPathName, DBFileName;

#ifdef SAPI_DEBUG
			fprintf(stderr, "sapi_GetDatabase:Location=%s\n",
				DBase->Location);
			fprintf(stderr, "sapi_GetDatabase:Name=%s\n",
				DBase->Name);
#endif
			/* Open database */
			DBPathName=DBase->Location;
			DBFileName=DBase->Name;
			if((DBase->pdb=new DB(DBPathName, 
				DBFileName))==NULL) {
				fprintf(stderr, 
				"sapi_GetDatabase:ISEARCH database failed\n");
				return NULL;
			}

			/* "Validate" by checking number of records */
			if(DBase->pdb->GetTotalRecords()<=0) {
				fprintf(stderr, 
				"sapi_GetDatabase:ISEARCH database failed\n");
				fprintf(stderr, 
				"sapi_GetDatabase:0 records available\n");
				delete DBase->pdb;	
				return NULL;
			}
#ifdef SAPI_DEBUG
			fprintf(stderr, 
				"records available = %d\n", 
				DBase->pdb->GetTotalRecords());
#endif

			break;
		}
		case SAPI_DBTYPE_CTSS:
			break;
		case SAPI_DBTYPE_SCRIPT: {
			/* Check to be sure the script is available */
			FILE *fp;
			if((fp=fopen(DBase->Location, "r"))==NULL) {
				fprintf(stderr, 
				"sapi_GetDatabase:SCRIPT database failed\n");
				return NULL;
			}
			fclose(fp);	
			break;
		}
		case SAPI_DBTYPE_CONQUEST:
			break;
		case SAPI_DBTYPE_ZAGENT: {
			/* Check to see that the WAIS index is there */
			FILE *fp;
			CHR Temp[512];

			sprintf(Temp, "%s/%s.dct", DBase->Location,
				DBase->Name);
			if((fp=fopen(Temp, "r"))==NULL) {
				fprintf(stderr, 
				"sapi_GetDatabase:ZAGENT database failed[%s]\n",
					DBase->Name);
				return NULL;
			}
			fclose(fp);	
			break;
		}
		default:
			return NULL;
	}

	return DBase;
}

PSAPI_DBASE dbase_Create()
{
	PSAPI_DBASE d;

	if((d=NEWSTRUCT(SAPI_DBASE))==NULL) {
		fprintf(stderr, "dbase_Create:Out of memory\n");
		return NULL;
	}

	d->Name = NULL;
	d->Type = SAPI_DBTYPE_ISEARCH;
	d->Location = NULL;
	d->Results = NULL;
	
	return d;
}

/*
Results is optional and only needed if Type is SAPI_DBTYPES_SCRIPT
or SAPI_DBTYPES_CONQUEST
*/
PSAPI_DBASE dbase_CreateInit(PCHR Name, INT2 Type, PCHR Location, PCHR Results)
{
	PSAPI_DBASE d;

	/* Sanity check */
	if((Name==NULL)||(Name[0]=='\0')) {
		fprintf(stderr, "dbase_CreateInit:Null Name\n");
		return NULL;
	}
	if((Location==NULL)||(Location[0]=='\0')) {
		fprintf(stderr, "dbase_CreateInit:Null Location\n");
		return NULL;
	}
	if((Type!=SAPI_DBTYPE_ISEARCH)&&(Type!=SAPI_DBTYPE_CGI)&&
		(Type!=SAPI_DBTYPE_SCRIPT)&&(Type!=SAPI_DBTYPE_CTSS)&&
		(Type!=SAPI_DBTYPE_CONQUEST)&&(Type!=SAPI_DBTYPE_NTN)&&
		(Type!=SAPI_DBTYPE_ZAGENT)) {
		fprintf(stderr, "dbase_CreateInit:Bad database type\n");
		return NULL;
	}
	if((Type==SAPI_DBTYPE_SCRIPT)||(Type==SAPI_DBTYPE_CONQUEST)) {
		if((Results==NULL)||(Results[0]=='\0')) {
			fprintf(stderr, "dbase_CreateInit:Null Results\n");
			return NULL;
		}
	}

	if((d=dbase_Create())==NULL)
		return NULL;

	if((d->Name=NEWSTRING(strlen(Name)+1))==NULL) {
		fprintf(stderr, "dbase_CreateInit:Out of memory\n");
		dbase_Destroy(d);
		return NULL;
	}
	strcpy(d->Name, Name);

	if((d->Location=NEWSTRING(strlen(Location)+1))==NULL) {
		fprintf(stderr, "dbase_CreateInit:Out of memory\n");
		dbase_Destroy(d);
		return NULL;
	}
	strcpy(d->Location, Location);

	d->Type = Type;

	if((d->Type == SAPI_DBTYPE_SCRIPT)||(d->Type==SAPI_DBTYPE_CONQUEST)) {
		if((d->Results=NEWSTRING(strlen(Results)+1))==NULL) {
			fprintf(stderr, "dbase_CreateInit:Out of memory\n");
			dbase_Destroy(d);
			return NULL;
		}
		strcpy(d->Results, Results);
	}
		
	return d;
}

void dbase_Destroy(PSAPI_DBASE d)
{
	if(d==NULL)
		return;

	if(d->Name!=NULL)
		FREE(d->Name);

	if(d->Location!=NULL)
		FREE(d->Location);
	
	if(d->Results!=NULL)
		FREE(d->Results);
#ifdef SAPI_CONQUEST
	if(d->Type == SAPI_DBTYPE_CONQUEST) {
		cqhl_stop_query(d->QueryThread);
		cqhl_close_query(d->QueryThread);
		cqhl_close_query_thread(d->QueryThread);
		cqhl_close_conquest();
	}
#endif
	FREE(d);
}


/* 
Group is the database name (and the actual group name, of course)
*/
PSAPI_DBASE dbase_CreateInitFile(PCHR File, PCHR Group)
{
	PSAPI_DBASE d;
	CHR Location[MAXPROFILESTR],TempType[MAXPROFILESTR],
		Results[MAXPROFILESTR];
	INT2 Type;

	Results[0]='\0';

	/* Sanity check */
	if((File==NULL)||(File[0]=='\0'))
		return NULL;

	if((Group==NULL) || (Group[0]=='\0'))
		return NULL;
	
	/* Read the Name, Location and Type from the Group */
	GetPrivateProfileString(Group, "Location", "", Location,
		sizeof(Location),File);
	if(Location[0]=='\0') {
		fprintf(stderr,"dbase_CreateInitFile:No Location for [%s]\n",
			Group);
		return NULL;
	}
	GetPrivateProfileString(Group, "Type", "", TempType,sizeof(TempType),File);
	if(TempType[0]=='\0') {
		fprintf(stderr,"dbase_CreateInitFile:No Type for [%s]\n",
			Group);
		return NULL;
	}
	if(!(strcasecmp(TempType, "CTSS")))
		Type=SAPI_DBTYPE_CTSS;
	if(!(strcasecmp(TempType, "ISEARCH")))
		Type=SAPI_DBTYPE_ISEARCH;
	if(!(strcasecmp(TempType, "CGI")))
		Type=SAPI_DBTYPE_CGI;
	if(!(strcasecmp(TempType, "SCRIPT")))
		Type=SAPI_DBTYPE_SCRIPT;
	if(!(strcasecmp(TempType, "CONQUEST")))
		Type=SAPI_DBTYPE_CONQUEST;
	if(!(strcasecmp(TempType, "ZAGENT")))
		Type=SAPI_DBTYPE_ZAGENT;
	/* Defaults to ISEARCH */

	if((Type == SAPI_DBTYPE_SCRIPT)||(Type==SAPI_DBTYPE_CONQUEST)) {
		GetPrivateProfileString(Group, "Results", "", Results,
			sizeof(Results), File);
		if(Results[0]=='\0') {
			fprintf(stderr,
				"dbase_CreateInitFile:No Results for [%s]\n",
				Group);
			return NULL;
		}
	}

	/* Build the database entry */
	if((d=dbase_CreateInit(Group,Type,Location,Results))==NULL)
		return NULL;

	return d;
}

PSAPI_RESULTSET ResultSet_Create()
{
	PSAPI_RESULTSET r;

	if((r=NEWSTRUCT(SAPI_RESULTSET))==NULL) {
		fprintf(stderr, "ResultSet_Create:Out of memory\n");
		return NULL;
	}

	r->Name = NULL;
	r->HitCount = 0;
	r->Diagnostic = 0;

	return r;
}

void ResultSet_Destroy(PSAPI_RESULTSET r)
{
	CHR TempFile[512];

	if(r==NULL)
		return;

	if(r->Name!=NULL)
		FREE(r->Name);

	if(r->Database!=NULL) {
		if(r->Database->Type==SAPI_DBTYPE_SCRIPT) {
			sprintf(TempFile, "%s.%i", r->Database->Results, 
				getpid());
#ifdef SAPI_DEBUG
			fprintf(stderr, "ResultSet_Destroy:Unlinking %s\n", 
				TempFile);
#endif
			unlink(TempFile);
		}
	}
}

void ResultSet_SetDatabase(PSAPI_RESULTSET r, PSAPI_DBASE d)
{
	if(r==NULL)
		return;

	r->Database = d;
}

void ResultSet_SetHitCount(PSAPI_RESULTSET r, INT4 h)
{
	if(r==NULL)
		return;

	r->HitCount = h;
}

INT4 ResultSet_GetHitCount(PSAPI_RESULTSET r)
{
	if(r==NULL)
		return -1;

	return r->HitCount;
}

/*
Callers, don't forget to free the buffer!
*/
PCHR ResultSet_GetRecord(PSAPI_RESULTSET RS, INT4 RecordNum, INT4 MaxLen, 
	PINT4 ActualLen, PCHR ElementSet, PCHR RecordSyntax)
{
	PCHR Record;

	if(RS==NULL)
		return NULL;

	Record = NULL;

	switch(RS->Database->Type) {
#ifdef SAPI_ISEARCH
		case SAPI_DBTYPE_ISEARCH: {
			RESULT RsRecord;
			STRING FileNameObj;
			CHR FileName[512];
			FILE *fp;
			INT Err;

			/* We only know SUTRS */
			strcpy(RecordSyntax, "1.2.840.10003.5.101");
			if((Record = NEWSTRING(MaxLen))==NULL) {
				fprintf(stderr, 
					"ResultSet_GetRecord:No memory\n");
				return NULL;
			}
			RS->prset->GetEntry(RecordNum, &RsRecord);

			*ActualLen = RsRecord.GetRecordData(Record,MaxLen);
/*
			RsRecord.GetFullFileName(&FileNameObj);

			FileNameObj.GetCString(FileName, sizeof(FileName));


			if((fp=fopen(FileName, "r"))==NULL) {
				perror("sapi_GetRecord");
				return NULL;
			}
			fseek(fp,0,2);
			*ActualLen = ftell(fp);
			rewind(fp);

			if((Record=NEWSTRING(*ActualLen+1024))==NULL) {
				fclose(fp);
				fprintf(stderr, "sapi_GetRecord: Out of mem\n");
				return NULL;
			}
			
			if((Err = fread(Record, 1, *ActualLen, fp))<=0) {
				fclose(fp);
				perror("sapi_GetRecord");
				return NULL;
			}
			Record[Err]='\0';

			fclose(fp);
*/


			break;
		}
#endif
#ifdef SAPI_CGI
		case SAPI_DBTYPE_CGI:
			fprintf(stderr, "ResultSet_GetRecord:No CGI yet\n");
			break;
#endif
#ifdef SAPI_SCRIPT
		case SAPI_DBTYPE_SCRIPT:
			return script_GetRecord(RS->Database->Results, 
				RecordNum, MaxLen, ActualLen, ElementSet);
			break;
#endif
#ifdef SAPI_CTSS
		case SAPI_DBTYPE_CTSS:
			return printRset(RecordNum,ElementSet,
				RecordSyntax,ActualLen);
			break;
#endif
#ifdef SAPI_CONQUEST
		case SAPI_DBTYPE_CONQUEST: {
			return conquest_GetRecord(RS, RecordNum, MaxLen, 
				ActualLen, ElementSet, RecordSyntax);
			break;
		}
#endif
		default:
			fprintf(stderr, "ResultSet_GetRecord:bad DB type\n");
			break;

	}
	return Record;
}

void sapi_Error(PCHR Function, INT Code, PCHR Msg)
{
        if((Msg==NULL)||(Msg[0]=='\0'))
                fprintf(stderr, "sapi:%s():%s\n", Function,
                        SAPI_ERRORMSG[Code]);
        else
                fprintf(stderr, "sapi:%s():%s[%s]\n", Function,
                        SAPI_ERRORMSG[Code],Msg);
}

PSAPI_QUERY query_Create()
{
	PSAPI_QUERY q;

	if((q=NEWSTRUCT(SAPI_QUERY))==NULL) {
		fprintf(stderr, "query_Create:Out of memory\n");
		return NULL;
	}

	q->Stack = (STACK *)NULL;
	q->Term = (PCHR)NULL;
	
	return q;
}

PSAPI_QUERY query_CreateStack(STACK *S)
{
	PSAPI_QUERY q;

	if((q=query_Create())==NULL)
		return NULL;

	q->Stack = S;

	return q;
}

void query_Destroy(PSAPI_QUERY q)
{
	if(q==(PSAPI_QUERY)NULL)
		return;

	if(q->Term!=NULL)
		FREE(q->Term);
/* Free the query stack (when you have time ;-)
	if(q->Stack!=NULL)
*/
}
