/*************************************************************************

Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

*************************************************************************/

/*@@@
File:           session.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#include <stdlib.h>
#include <string.h>
#define PLATFORM_INC_MALLOC
#include "platform.h"
#include "imem.h"
#include "ini.h"
#include "session.h"
#include <errno.h>

#ifndef STRCPY
#define STRCPY(a,b) strcpy((PCHR)a, (PCHR)b)
#endif

int ReadProfileSettings(SESSION *S, PCHR File, PCHR Group) {
	CHR TempString[MAXPROFILESTR];

	if ( CacheProfileFile(File) == 0 ) {
                printf("Opening cache failed.\n");
                return 0;
        }
	GetPrivateProfileString(Group, "ServerType",
		S->ServerType, S->ServerType,sizeof(S->ServerType),File);
 
	S->MaxSessions = GetPrivateProfileInt(Group, "MaxSessions", 
		S->MaxSessions, File);
	
	GetPrivateProfileString(Group, "Host",
		S->Host, S->Host,sizeof(S->Host),File);
 
	S->Port = GetPrivateProfileInt(Group, "Port", S->Port, File);
	
	GetPrivateProfileString(Group, "sQuery",
		S->sQuery, S->sQuery,sizeof(S->sQuery),File);

	GetPrivateProfileString(Group, "sDBNames",
		S->sDBNames, S->sDBNames,sizeof(S->sDBNames),File);

	S->sSmallSetUpperBound = 
		GetPrivateProfileLong(Group, "sSmallSetUpperBound", 
		S->sSmallSetUpperBound, File);

	S->sLargeSetLowerBound = 
		GetPrivateProfileLong(Group, "sLargeSetLowerBound", 
		S->sLargeSetLowerBound, File);
        
	S->sMedSetPresentNum = 
		GetPrivateProfileLong(Group, "sMedSetPresentNum", 
		S->sMedSetPresentNum, File);

	GetPrivateProfileString(Group, "sReplaceIndicator",
		"1", TempString, 3,File);

	if(!strcmp(TempString,"1"))
		S->sReplaceIndicator = 255;
	else
		S->sReplaceIndicator = 0;

	S->TimeOut = 
		GetPrivateProfileLong(Group, "TimeOut", 
		S->TimeOut, File);

	GetPrivateProfileString(Group, "OutputFile",
		S->OutputFile, S->OutputFile,sizeof(S->OutputFile),File);

	GetPrivateProfileString(Group, "StatsFile",
		S->StatsFile, S->StatsFile,sizeof(S->StatsFile),File);

	GetPrivateProfileString(Group, "Trace",
		S->Trace, S->Trace,sizeof(S->Trace),File);
	
	GetPrivateProfileString(Group, "TraceLog",
		S->TraceLog, S->TraceLog,sizeof(S->TraceLog),File);

	GetPrivateProfileString(Group, "Stats",
		S->Stats, S->Stats,sizeof(S->Stats),File);
	
	GetPrivateProfileString(Group, "StatsLog",
		S->StatsLog, S->StatsLog,sizeof(S->StatsLog),File);
	
	/* Don't need to load TraceLogFilePointer (TFP) */	

	GetPrivateProfileString(Group, "AccessLog",
		S->AccessLog, S->AccessLog,sizeof(S->AccessLog),File);
	
	GetPrivateProfileString(Group, "SAPI",
		S->SAPI, S->SAPI,sizeof(S->SAPI),File);
	
	GetPrivateProfileString(Group, "ServerPath",
		S->ServerPath, S->ServerPath,sizeof(S->ServerPath),File);
	
	GetPrivateProfileString(Group, "QueryFormat",
		S->QueryFormat, S->QueryFormat,sizeof(S->QueryFormat),File);

	GetPrivateProfileString(Group, "DisplayRecSyntax",
		S->DisplayRecSyntax, S->DisplayRecSyntax,
		sizeof(S->DisplayRecSyntax),File);

	GetPrivateProfileString(Group, "DBList",
		S->DBList, S->DBList,sizeof(S->DBList),File);

	GetPrivateProfileString(Group, "DBPath",
		S->DBPath, S->DBPath,sizeof(S->DBPath),File);

	GetPrivateProfileString(Group, "DiagSetId",
		S->DiagSetId, S->DiagSetId,sizeof(S->DiagSetId),File);

	GetPrivateProfileString(Group, "iAuthentication",
		S->iAuthentication, S->iAuthentication,sizeof(S->iAuthentication),File);

	GetPrivateProfileString(Group, "iImpID",
		S->iImpID, S->iImpID,sizeof(S->iImpID),File);

	GetPrivateProfileString(Group, "iImpName",
		S->iImpName, S->iImpName,sizeof(S->iImpName),File);

	GetPrivateProfileString(Group, "iImpVersion",
		S->iImpVersion, S->iImpVersion,sizeof(S->iImpVersion),File);

	S->iProtocolVersion = GetPrivateProfileInt(Group, 
		"iProtocolVersion", S->iProtocolVersion, File);

	GetPrivateProfileString(Group, "iOptions",
		S->iOptions, S->iOptions,sizeof(S->iOptions),File);

	S->iPreferredMsgSize = 
		GetPrivateProfileLong(Group, "iPreferredMsgSize", 
		S->iPreferredMsgSize, File);

	S->iMaxRecSize = 
		GetPrivateProfileInt(Group, "iMaxRecSize", 
		S->iMaxRecSize, File);

	GetPrivateProfileString(Group, "iUserInformationField",
		S->iUserInformationField, S->iUserInformationField,sizeof(S->iUserInformationField),File);

	GetPrivateProfileString(Group, "iReferenceId",
		S->iReferenceId, S->iReferenceId,sizeof(S->iReferenceId),File);

	S->iResult = 
		GetPrivateProfileLong(Group, "iResult", 
		S->iResult, File);

	GetPrivateProfileString(Group, "sReferenceId",
		S->sReferenceId, S->sReferenceId,sizeof(S->sReferenceId),File);

	S->sResultCount = 
		GetPrivateProfileLong(Group, "sResultCount", 
		S->sResultCount, File);

	GetPrivateProfileString(Group, "sResultSetName",
		S->sResultSetName, S->sResultSetName,sizeof(S->sResultSetName),File);

	GetPrivateProfileString(Group, "sSmallSetElementSetNames",
		S->sSmallSetElementSetNames, S->sSmallSetElementSetNames,sizeof(S->sSmallSetElementSetNames),File);

	GetPrivateProfileString(Group, "sMediumSetElementSetNames",
		S->sMediumSetElementSetNames, S->sMediumSetElementSetNames,sizeof(S->sMediumSetElementSetNames),File);

	GetPrivateProfileString(Group, "sPreferredRecordSyntax",
		S->sPreferredRecordSyntax, S->sPreferredRecordSyntax,sizeof(S->sPreferredRecordSyntax),File);

	GetPrivateProfileString(Group, "sAttributeSet",
		S->sAttributeSet, S->sAttributeSet,sizeof(S->sAttributeSet),File);

	GetPrivateProfileString(Group, "pReferenceId",
		S->pReferenceId, S->pReferenceId,sizeof(S->pReferenceId),File);

	GetPrivateProfileString(Group, "pResultSetId",
		S->pResultSetId, S->pResultSetId,sizeof(S->pResultSetId),File);

	S->pResultSetStartPoint = 
		GetPrivateProfileLong(Group, "pResultSetStartPoint", 
		S->pResultSetStartPoint, File);

	S->pNumRecsReq = 
		GetPrivateProfileLong(Group, "pNumRecsReq", 
		S->pNumRecsReq, File);

	GetPrivateProfileString(Group, "pElementSetNames",
		S->pElementSetNames, S->pElementSetNames,sizeof(S->pElementSetNames),File);

	GetPrivateProfileString(Group, "pPreferredRecSyntax",
		S->pPreferredRecSyntax, S->pPreferredRecSyntax,sizeof(S->pPreferredRecSyntax),File);

	S->pNumRecsReturned = 
		GetPrivateProfileLong(Group, "pNumRecsReturned", 
		S->pNumRecsReturned, File);

	S->pNextResultSetPos = 
		GetPrivateProfileLong(Group, "pNextResultSetPos", 
		S->pNextResultSetPos, File);

	if ( UncacheProfileFile() == 0 ) {
                printf("Error: changes may not have been saved.\n");
                AbortProfileChanges();
                return(0);
        }

	return(1);
}

INT4 GetPrivateProfileLong(PCHR pszSection, PCHR pszEntry, INT4 pszDefault,
	 PCHR pszFileName) {
	CHR defaultBuf[MAXSESSIONSTRING];
	CHR valueBuf[MAXSESSIONSTRING];

	sprintf(defaultBuf,"%d",pszDefault);

	GetPrivateProfileString(pszSection, pszEntry, defaultBuf,
		valueBuf, sizeof(valueBuf), pszFileName);
	
	return(atol(valueBuf));
}

int GetCmdLineSettings(SESSION* S, int ac, PCHR *argv, PCHR profilefn,
		PCHR profilegroup) {
/*
	Returns 0 if OK, or a positive index into argv[] that points to the
	error.  Incorrect filenames, groups, and options are not detected
	as errors!  Only syntax errors in command line usage may be
	detected.
	
	KAG - returns -1 if ReadProfileSettings() fails.  Check errno
		for file error code.
*/
	int x;
	PCHR av;
	PCHR V;
	PCHR p;
	CHR fn[256];
	CHR group[256];
	CHR option[256];
	CHR setting[256];
	int argc = ac - 1;
	if (argc < 1)
		return 0;
	for (x = 1; x <= argc; x++) {
		av = argv[x];
		if ( (av[0] != '-') || ( (av[1] != 'i') && (av[1] != 'o') ) )
			return x;
		V = (PCHR)imem_calloc(1,strlen(av+2)+1);
		STRCPY(V, av+2);
		switch (av[1]) {
			case 'i':
				if (V[0] == '\0') {
					STRCPY(fn, profilefn);
					STRCPY(group, profilegroup);
				} else {
					p = (PCHR)strchr((PCHR)V, ',');
					if (p == NULL) {
						STRCPY(group, profilegroup);
					} else {
						*p = '\0';
						*group = '*';
						STRCPY(group+1, p+1);
					}
					STRCPY(fn, V);
					if (fn[0] == '\0')
						STRCPY(fn, profilefn);
				}
/*
				if(ReadProfileSettings(S, fn, group)==0) {
					free(V);
					return -1;
				}
*/
				ReadProfileSettings(S, fn, group);
				break;
			case 'o':
				p = (PCHR)strchr(V, '=');
				if (p == NULL) {
					setting[0] = '\0';
				} else {
					*p = (CHR)'\0';
					STRCPY(setting, p+1);
				}
				STRCPY(option, V);
				CacheProfileFile(profilefn);
				WritePrivateProfileString(TEMPGROUP,NULL,NULL, 
						(PCHR)profilefn);
				WritePrivateProfileString(TEMPGROUP, 
					(PCHR)option, (PCHR)setting, 
					(PCHR)profilefn);
				UncacheProfileFile();
				ReadProfileSettings(S, (PCHR)profilefn, 
					(PCHR)TEMPGROUP);
				WritePrivateProfileString(TEMPGROUP,NULL,NULL, 
						(PCHR)profilefn);
				break;
		} /* switch */
		free(V);
	}
	return 0;
}

