/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           z3950.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "berutil.h"
#include "z3950.h"
#include "z3950sup.h"
#include "gdt.h"

POCTETSTRING OctetString_CreateTag(INT4 Tag)
{
	POCTETSTRING p;

	if((p= NEWSTRUCT(OCTETSTRING))==NULL) {
		return NULL;
	}

	p->next = NULL_DIR;
	p->user = 0;
	p->ptr.data = NULL;
	p->count = 0;
	p->fldid = Tag;
	p->Class = ASN1_CONTEXT;
	p->form = ASN1_PRIMITIVE;
	return p;
}

void OctetString_Destroy(POCTETSTRING p)
{
	if(p) {
		if(p->ptr.data)
			FREE(p->ptr.data);
		dfree(p);
	}
}

void OctetString_SetData(POCTETSTRING p, PUCHR d, INT4 l)
{
	if(p) {
		p->ptr.data = d;
		p->count = l;
		return;
	}
}

PUCHR OctetString_GetData(POCTETSTRING p)
{
	if(p)
		return p->ptr.data;
	return NULL;
}

INT4 OctetString_GetLength(POCTETSTRING p)
{
	if(p)
		return p->count;
	return -1;
}

PCHR OctetString_GetASCIIString(POCTETSTRING p)
{
	if(p) {
		INT4 Len;
		PCHR s;
		if((Len = OctetString_GetLength(p))<=0)
			return NULL;
		if((s = NEWSTRING(Len + 1))==NULL)
			return NULL;
		MEMCOPY(s,OctetString_GetData(p),Len);
		s[Len]='\0';
		return s;
	}
	return NULL;
}

void OctetString_Printf(POCTETSTRING p, PCHR f)
{
	if(p) {
		PCHR s;
		if((s=OctetString_GetASCIIString(p))==NULL)
			return;
		printf(f,s);
		FREE(s);
	}
}

PBITSTRING BitString_CreateTag(INT4 Tag)
{
	PBITSTRING p;

	if((p= NEWSTRUCT(BITSTRING))==NULL) {
		return NULL;
	}

	p->next = NULL_DIR;
	p->user = 0;
	p->ptr.data = NULL;
	p->count = 0;
	p->fldid = Tag;
	p->Class = ASN1_CONTEXT;
	p->form = ASN1_PRIMITIVE;
	return p;
}

void BitString_Destroy(PBITSTRING p)
{
	dfree(p);
}

void BitString_SetBits(PBITSTRING p, PUCHR bits)
{
	if(p) {
PUCHR data;
PUCHR t;
int i, j, len, num_bytes, unused;
static CHAR mask[8]={0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
		if(p->ptr.data)
			FREE(p->ptr.data);
len=strlen((char *)bits);
num_bytes=1+len/8+((len%8)?1:0);
p->ptr.data=t=data=NEWUSTRING(num_bytes);
if(!data)
return;
unused=8-len%8;
if(unused==8)
unused=0;
*t=unused;
for(i=0, t++; i<len; i+=8, t++)
{
*t='\0';
for(j=0; j<8 && i+j<len; j++, bits++)
if(*bits=='y' || *bits=='Y' || *bits=='t' || *bits=='T' ||
*bits=='1')
t[0]|=mask[j];
}
		p->count = num_bytes;
	}
}

CHR BitString_GetBit(PBITSTRING p, INT4 BitNum)
{
	if(p) {
		PUCHR bits;
		CHR c;
		INT4 j, last_used;
		INT4 i;
		static CHR mask[8]={0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01};
		last_used=8 - p->ptr.data[0];
		bits=NEWUSTRING(8*p->count);
		for(i=1; i<p->count-1; i++)
		for(j=0; j<8; j++)
		bits[(i-1)*8+j]=p->ptr.data[i]&mask[j]?'y':'n';
		for(j=0; j<last_used; j++)
		bits[(i-1)*8+j]=p->ptr.data[i]&mask[j]?'y':'n';
		bits[(i-1)*8+j]='\0';
		c = bits[BitNum];
		FREE(bits);
		return c;
	}
	return '\0';
}

PINITREQUEST InitRequest_Create(void)
{
	PINITREQUEST t;

	if((t=dmake(INITREQUEST_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void InitRequest_Destroy(PINITREQUEST p)
{
	if(p) {
		dfree(p);
	}
}

void InitRequest_SetReferenceId(PINITREQUEST p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID InitRequest_GetReferenceId(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void InitRequest_SetProtocolVersion(PINITREQUEST p, PPROTOCOLVERSION m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PPROTOCOLVERSION InitRequest_GetProtocolVersion(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, PROTOCOLVERSION_TAG);
	return NULL;
}

void InitRequest_SetOptions(PINITREQUEST p, POPTIONS m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POPTIONS InitRequest_GetOptions(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, OPTIONS_TAG);
	return NULL;
}

void InitRequest_SetPreferredMessageSize(PINITREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, PREFERREDMESSAGESIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 InitRequest_GetPreferredMessageSize(PINITREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, PREFERREDMESSAGESIZE_TAG);
	return -1;
}

void InitRequest_SetExceptionalRecordSize(PINITREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, EXCEPTIONALRECORDSIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 InitRequest_GetExceptionalRecordSize(PINITREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, EXCEPTIONALRECORDSIZE_TAG);
	return -1;
}

void InitRequest_SetIdAuthentication(PINITREQUEST p, PANY m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PANY InitRequest_GetIdAuthentication(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, IDAUTHENTICATION_TAG);
	return NULL;
}

void InitRequest_SetImplementationId(PINITREQUEST p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitRequest_GetImplementationId(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, IMPLEMENTATIONID_TAG);
	return NULL;
}

void InitRequest_SetImplementationName(PINITREQUEST p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitRequest_GetImplementationName(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, IMPLEMENTATIONNAME_TAG);
	return NULL;
}

void InitRequest_SetImplementationVersion(PINITREQUEST p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitRequest_GetImplementationVersion(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, IMPLEMENTATIONVERSION_TAG);
	return NULL;
}

void InitRequest_SetUserInformationField(PINITREQUEST p, PEXTERNAL m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PEXTERNAL InitRequest_GetUserInformationField(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, USERINFORMATIONFIELD_TAG);
	return NULL;
}

void InitRequest_SetOtherInfo(PINITREQUEST p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO InitRequest_GetOtherInfo(PINITREQUEST p)
{
	if(p)
		return (PINITREQUEST)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

PINITRESPONSE InitResponse_Create(void)
{
	PINITRESPONSE t;

	if((t=dmake(INITRESPONSE_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void InitResponse_Destroy(PINITRESPONSE p)
{
	if(p) {
		dfree(p);
	}
}

void InitResponse_SetReferenceId(PINITRESPONSE p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID InitResponse_GetReferenceId(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void InitResponse_SetProtocolVersion(PINITRESPONSE p, PPROTOCOLVERSION m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PPROTOCOLVERSION InitResponse_GetProtocolVersion(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, PROTOCOLVERSION_TAG);
	return NULL;
}

void InitResponse_SetOptions(PINITRESPONSE p, POPTIONS m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POPTIONS InitResponse_GetOptions(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, OPTIONS_TAG);
	return NULL;
}

void InitResponse_SetPreferredMessageSize(PINITRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, PREFERREDMESSAGESIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 InitResponse_GetPreferredMessageSize(PINITRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, PREFERREDMESSAGESIZE_TAG);
	return -1;
}

void InitResponse_SetExceptionalRecordSize(PINITRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, EXCEPTIONALRECORDSIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 InitResponse_GetExceptionalRecordSize(PINITRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, EXCEPTIONALRECORDSIZE_TAG);
	return -1;
}

void InitResponse_SetResult(PINITRESPONSE p, BOOLEAN m)
{
	if(p) {
		if(daddnum(p, RESULT_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

BOOLEAN InitResponse_GetResult(PINITRESPONSE p)
{
	if(p)
		return (BOOLEAN)zutil_GetTaggedInt(p, RESULT_TAG);
	return 0;
}

void InitResponse_SetImplementationId(PINITRESPONSE p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitResponse_GetImplementationId(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, IMPLEMENTATIONID_TAG);
	return NULL;
}

void InitResponse_SetImplementationName(PINITRESPONSE p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitResponse_GetImplementationName(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, IMPLEMENTATIONNAME_TAG);
	return NULL;
}

void InitResponse_SetImplementationVersion(PINITRESPONSE p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING InitResponse_GetImplementationVersion(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, IMPLEMENTATIONVERSION_TAG);
	return NULL;
}

void InitResponse_SetUserInfoField(PINITRESPONSE p, PEXTERNAL m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PEXTERNAL InitResponse_GetUserInfoField(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, USERINFOFIELD_TAG);
	return NULL;
}

void InitResponse_SetOtherInfo(PINITRESPONSE p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO InitResponse_GetOtherInfo(PINITRESPONSE p)
{
	if(p)
		return (PINITRESPONSE)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

POTHERINFO OtherInfo_Create(void)
{
	POTHERINFO t;

	if((t=dmake(OTHERINFO_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void OtherInfo_Destroy(POTHERINFO p)
{
	if(p) {
		dfree(p);
	}
}

PSEARCHREQUEST SearchRequest_Create(void)
{
	PSEARCHREQUEST t;

	if((t=dmake(SEARCHREQUEST_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void SearchRequest_Destroy(PSEARCHREQUEST p)
{
	if(p) {
		dfree(p);
	}
}

void SearchRequest_SetReferenceId(PSEARCHREQUEST p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID SearchRequest_GetReferenceId(PSEARCHREQUEST p)
{
	if(p)
		return (PSEARCHREQUEST)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void SearchRequest_SetSmallSetUpperBound(PSEARCHREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, SMALLSETUPPERBOUND_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchRequest_GetSmallSetUpperBound(PSEARCHREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, SMALLSETUPPERBOUND_TAG);
	return -1;
}

void SearchRequest_SetLargeSetLowerBound(PSEARCHREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, LARGESETLOWERBOUND_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchRequest_GetLargeSetLowerBound(PSEARCHREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, LARGESETLOWERBOUND_TAG);
	return -1;
}

void SearchRequest_SetMediumSetPresentNumber(PSEARCHREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, MEDIUMSETPRESENTNUMBER_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchRequest_GetMediumSetPresentNumber(PSEARCHREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, MEDIUMSETPRESENTNUMBER_TAG);
	return -1;
}

void SearchRequest_SetReplaceIndicator(PSEARCHREQUEST p, BOOLEAN m)
{
	if(p) {
		UCHR temp;
		if(m==1) 
			temp=255;
		else
			temp=0;
		if(daddchar(p, REPLACEINDICATOR_TAG, ASN1_CONTEXT, 
				(PUCHR)&temp, sizeof(UCHR))==NULL) {
		}
	}
}

BOOLEAN SearchRequest_GetReplaceIndicator(PSEARCHREQUEST p)
{
	if(p)
		return (BOOLEAN)zutil_GetTaggedInt(p, REPLACEINDICATOR_TAG);
	return 0;
}

void SearchRequest_SetResultSetName(PSEARCHREQUEST p, PINTERNATIONALSTRING m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PINTERNATIONALSTRING SearchRequest_GetResultSetName(PSEARCHREQUEST p)
{
	if(p)
		return (PSEARCHREQUEST)zutil_GetTaggedObject(p, RESULTSETNAME_TAG);
	return NULL;
}

POBJECTIDENTIFIER SearchRequest_GetPreferredRecordSyntax(PSEARCHREQUEST p)
{
	if(p)
		return (PSEARCHREQUEST)zutil_GetTaggedObject(p, PREFERREDRECORDSYNTAX_TAG);
	return NULL;
}

void SearchRequest_SetAdditionalSearchInfo(PSEARCHREQUEST p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO SearchRequest_GetAdditionalSearchInfo(PSEARCHREQUEST p)
{
	if(p)
		return (PSEARCHREQUEST)zutil_GetTaggedObject(p, ADDITIONALSEARCHINFO_TAG);
	return NULL;
}

void SearchRequest_SetOtherInfo(PSEARCHREQUEST p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO SearchRequest_GetOtherInfo(PSEARCHREQUEST p)
{
	if(p)
		return (PSEARCHREQUEST)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

PSEARCHRESPONSE SearchResponse_Create(void)
{
	PSEARCHRESPONSE t;

	if((t=dmake(SEARCHRESPONSE_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void SearchResponse_Destroy(PSEARCHRESPONSE p)
{
	if(p) {
		dfree(p);
	}
}

void SearchResponse_SetReferenceId(PSEARCHRESPONSE p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID SearchResponse_GetReferenceId(PSEARCHRESPONSE p)
{
	if(p)
		return (PSEARCHRESPONSE)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void SearchResponse_SetResultCount(PSEARCHRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, RESULTCOUNT_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchResponse_GetResultCount(PSEARCHRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, RESULTCOUNT_TAG);
	return NULL;
}

void SearchResponse_SetNumberOfRecordsReturned(PSEARCHRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, NUMBEROFRECORDSRETURNED_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchResponse_GetNumberOfRecordsReturned(PSEARCHRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, NUMBEROFRECORDSRETURNED_TAG);
	return NULL;
}

void SearchResponse_SetNextResultSetPosition(PSEARCHRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, NEXTRESULTSETPOSITION_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchResponse_GetNextResultSetPosition(PSEARCHRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, NEXTRESULTSETPOSITION_TAG);
	return NULL;
}

void SearchResponse_SetSearchStatus(PSEARCHRESPONSE p, BOOLEAN m)
{
	if(p) {
		if(daddnum(p, SEARCHSTATUS_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

BOOLEAN SearchResponse_GetSearchStatus(PSEARCHRESPONSE p)
{
	if(p)
		return (BOOLEAN)zutil_GetTaggedInt(p, SEARCHSTATUS_TAG);
	return 0;
}

void SearchResponse_SetResultSetStatus(PSEARCHRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, RESULTSETSTATUS_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchResponse_GetResultSetStatus(PSEARCHRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, RESULTSETSTATUS_TAG);
	return -1;
}

void SearchResponse_SetPresentStatus(PSEARCHRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, PRESENTSTATUS_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 SearchResponse_GetPresentStatus(PSEARCHRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, PRESENTSTATUS_TAG);
	return -1;
}

DATA_DIR *SearchResponse_GetRecords(PSEARCHRESPONSE p)
{
	DATA_DIR *d;

	if(p) {
		if((d = zutil_GetTaggedObject(p, RESPONSERECORDS_TAG))!=NULL)
			return d;
		if((d = zutil_GetTaggedObject(p, NONSURROGATEDIAGNOSTIC_TAG))!=NULL)
			return d;
		if((d = zutil_GetTaggedObject(p, MULTIPLENONSURDIAGNOSTICS_TAG))!=NULL)
			return d;
		return NULL;
	}
	return NULL;
}

void SearchResponse_SetAdditionalSearchInfo(PSEARCHRESPONSE p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO SearchResponse_GetAdditionalSearchInfo(PSEARCHRESPONSE p)
{
	if(p)
		return (PSEARCHRESPONSE)zutil_GetTaggedObject(p, ADDITIONALSEARCHINFO_TAG);
	return NULL;
}

void SearchResponse_SetOtherInfo(PSEARCHRESPONSE p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO SearchResponse_GetOtherInfo(PSEARCHRESPONSE p)
{
	if(p)
		return (PSEARCHRESPONSE)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

PPRESENTREQUEST PresentRequest_Create(void)
{
	PPRESENTREQUEST t;

	if((t=dmake(PRESENTREQUEST_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void PresentRequest_Destroy(PPRESENTREQUEST p)
{
	if(p) {
		dfree(p);
	}
}

void PresentRequest_SetReferenceId(PPRESENTREQUEST p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID PresentRequest_GetReferenceId(PPRESENTREQUEST p)
{
	if(p)
		return (PPRESENTREQUEST)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void PresentRequest_SetResultSetId(PPRESENTREQUEST p, PRESULTSETID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PRESULTSETID PresentRequest_GetResultSetId(PPRESENTREQUEST p)
{
	if(p)
		return (PPRESENTREQUEST)zutil_GetTaggedObject(p, RESULTSETID_TAG);
	return NULL;
}

void PresentRequest_SetResultSetStartPoint(PPRESENTREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, RESULTSETSTARTPOINT_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentRequest_GetResultSetStartPoint(PPRESENTREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, RESULTSETSTARTPOINT_TAG);
	return -1;
}

void PresentRequest_SetNumberOfRecordsRequested(PPRESENTREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, NUMBEROFRECORDSREQUESTED_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentRequest_GetNumberOfRecordsRequested(PPRESENTREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, NUMBEROFRECORDSREQUESTED_TAG);
	return -1;
}

POBJECTIDENTIFIER PresentRequest_GetPreferredRecordSyntax(PPRESENTREQUEST p)
{
	if(p)
		return (PPRESENTREQUEST)zutil_GetTaggedObject(p, PREFERREDRECORDSYNTAX_TAG);
	return NULL;
}

void PresentRequest_SetMaxSegmentCount(PPRESENTREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, MAXSEGMENTCOUNT_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentRequest_GetMaxSegmentCount(PPRESENTREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, MAXSEGMENTCOUNT_TAG);
	return -1;
}

void PresentRequest_SetMaxRecordSize(PPRESENTREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, MAXRECORDSIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentRequest_GetMaxRecordSize(PPRESENTREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, MAXRECORDSIZE_TAG);
	return -1;
}

void PresentRequest_SetMaxSegmentSize(PPRESENTREQUEST p, INT4 m)
{
	if(p) {
		if(daddnum(p, MAXSEGMENTSIZE_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentRequest_GetMaxSegmentSize(PPRESENTREQUEST p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, MAXSEGMENTSIZE_TAG);
	return -1;
}

void PresentRequest_SetOtherInfo(PPRESENTREQUEST p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO PresentRequest_GetOtherInfo(PPRESENTREQUEST p)
{
	if(p)
		return (PPRESENTREQUEST)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

PPRESENTRESPONSE PresentResponse_Create(void)
{
	PPRESENTRESPONSE t;

	if((t=dmake(PRESENTRESPONSE_TAG, ASN1_CONTEXT, 15))==NULL) {
		/* Enter any error logging here */
		return NULL;
	}

	return t;
}

void PresentResponse_Destroy(PPRESENTRESPONSE p)
{
	if(p) {
		dfree(p);
	}
}

void PresentResponse_SetReferenceId(PPRESENTRESPONSE p, PREFERENCEID m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

PREFERENCEID PresentResponse_GetReferenceId(PPRESENTRESPONSE p)
{
	if(p)
		return (PPRESENTRESPONSE)zutil_GetTaggedObject(p, REFERENCEID_TAG);
	return NULL;
}

void PresentResponse_SetNumberOfRecordsReturned(PPRESENTRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, NUMBEROFRECORDSRETURNED_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentResponse_GetNumberOfRecordsReturned(PPRESENTRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, NUMBEROFRECORDSRETURNED_TAG);
	return -1;
}

void PresentResponse_SetNextResultSetPosition(PPRESENTRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, NEXTRESULTSETPOSITION_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentResponse_GetNextResultSetPosition(PPRESENTRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, NEXTRESULTSETPOSITION_TAG);
	return -1;
}

void PresentResponse_SetPresentStatus(PPRESENTRESPONSE p, INT4 m)
{
	if(p) {
		if(daddnum(p, PRESENTSTATUS_TAG, ASN1_CONTEXT, (PUCHR)&m, sizeof(m))==NULL) {
		}
	}
}

INT4 PresentResponse_GetPresentStatus(PPRESENTRESPONSE p)
{
	if(p)
		return (INT4)zutil_GetTaggedInt(p, PRESENTSTATUS_TAG);
	return -1;
}

void PresentResponse_SetOtherInfo(PPRESENTRESPONSE p, POTHERINFO m)
{
	if(p && m) {
		if(m->ptr.data && m->ptr.data[0]!='\0' && m->count > 0) {
			if(dadddir(p, m)==NULL) {
			}
		}
	}
}

POTHERINFO PresentResponse_GetOtherInfo(PPRESENTRESPONSE p)
{
	if(p)
		return (PPRESENTRESPONSE)zutil_GetTaggedObject(p, OTHERINFO_TAG);
	return NULL;
}

