/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           z3950.h
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#ifndef _z3950_
#define _z3950_

#include "berutil.h"
#ifdef USE_MEMH
#include <mem.h>
#endif /* USE_MEMH */
#ifdef USE_ALLOCH
#include <alloc.h>
#endif /* USE_ALLOCH */
#include "gdt.h"
#include "imem.h"
#include "zutil.h"

/* Definitions */

#define REFERENCEID_TAG 2
#define RESULTSETID_TAG 31
#define ELEMENTSETNAME_TAG 103
#define PREFERREDMESSAGESIZE_TAG 5
#define EXCEPTIONALRECORDSIZE_TAG 6
#define IDAUTHENTICATION_TAG 7
#define IMPLEMENTATIONID_TAG 110
#define IMPLEMENTATIONNAME_TAG 111
#define IMPLEMENTATIONVERSION_TAG 112
#define USERINFORMATIONFIELD_TAG 11
#define PREFERREDMESSAGESIZE_TAG 5
#define EXCEPTIONALRECORDSIZE_TAG 6
#define RESULT_TAG 12
#define IMPLEMENTATIONID_TAG 110
#define IMPLEMENTATIONNAME_TAG 111
#define IMPLEMENTATIONVERSION_TAG 112
#define USERINFOFIELD_TAG 11
#define OTHERINFO_TAG 201
#define SMALLSETUPPERBOUND_TAG 13
#define LARGESETLOWERBOUND_TAG 14
#define MEDIUMSETPRESENTNUMBER_TAG 15
#define REPLACEINDICATOR_TAG 16
#define RESULTSETNAME_TAG 17
#define PREFERREDRECORDSYNTAX_TAG 104
#define ADDITIONALSEARCHINFO_TAG 203
#define RESULTCOUNT_TAG 23
#define NUMBEROFRECORDSRETURNED_TAG 24
#define NEXTRESULTSETPOSITION_TAG 25
#define SEARCHSTATUS_TAG 22
#define RESULTSETSTATUS_TAG 26
#define PRESENTSTATUS_TAG 27
#define ADDITIONALSEARCHINFO_TAG 203
#define RESULTSETSTARTPOINT_TAG 30
#define NUMBEROFRECORDSREQUESTED_TAG 29
#define PREFERREDRECORDSYNTAX_TAG 104
#define MAXSEGMENTCOUNT_TAG 204
#define MAXRECORDSIZE_TAG 206
#define MAXSEGMENTSIZE_TAG 207
#define NUMBEROFRECORDSRETURNED_TAG 24
#define NEXTRESULTSETPOSITION_TAG 25
#define PRESENTSTATUS_TAG 27
#define OPTIONS_TAG 4
#define PROTOCOLVERSION_TAG 3
#define INITREQUEST_TAG 20
#define INITRESPONSE_TAG 21
#define SEARCHREQUEST_TAG 22
#define SEARCHRESPONSE_TAG 23
#define PRESENTREQUEST_TAG 24
#define PRESENTRESPONSE_TAG 25
#define ELEMENTSETNAMES_TAG 

#define ReferenceId_Create() (PREFERENCEID)OctetString_CreateTag(REFERENCEID_TAG)
#define ReferenceId_Destroy(a) OctetString_Destroy(a)
#define ReferenceId_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ReferenceId_GetData(a) OctetString_GetData(a)
#define ReferenceId_GetLength(a) OctetString_GetLength(a)
#define ReferenceId_GetASCIIString(a) OctetString_GetASCIIString(a)
#define ReferenceId_Printf(a,b) OctetString_Printf(a,b)
#define ResultSetId_Create() (PRESULTSETID)OctetString_CreateTag(RESULTSETID_TAG)
#define ResultSetId_Destroy(a) OctetString_Destroy(a)
#define ResultSetId_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ResultSetId_GetData(a) OctetString_GetData(a)
#define ResultSetId_GetLength(a) OctetString_GetLength(a)
#define ResultSetId_GetASCIIString(a) OctetString_GetASCIIString(a)
#define ResultSetId_Printf(a,b) OctetString_Printf(a,b)
#define ElementSetName_Create() (PELEMENTSETNAME)OctetString_CreateTag(ELEMENTSETNAME_TAG)
#define ElementSetName_Destroy(a) OctetString_Destroy(a)
#define ElementSetName_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ElementSetName_GetData(a) OctetString_GetData(a)
#define ElementSetName_GetLength(a) OctetString_GetLength(a)
#define ElementSetName_GetASCIIString(a) OctetString_GetASCIIString(a)
#define ElementSetName_Printf(a,b) OctetString_Printf(a,b)
#define Options_Create() (POPTIONS)BitString_CreateTag(OPTIONS_TAG)
#define Options_Destroy(a) BitString_Destroy(a)
#define Options_SetBits(a,b) BitString_SetBits(a,b)
#define Options_GetBit(a,b) BitString_GetBit(a,b)
#define ProtocolVersion_Create() (PPROTOCOLVERSION)BitString_CreateTag(PROTOCOLVERSION_TAG)
#define ProtocolVersion_Destroy(a) BitString_Destroy(a)
#define ProtocolVersion_SetBits(a,b) BitString_SetBits(a,b)
#define ProtocolVersion_GetBit(a,b) BitString_GetBit(a,b)
#define ImplementationId_Create() (PIMPLEMENTATIONID)OctetString_CreateTag(IMPLEMENTATIONID_TAG)
#define ImplementationId_Destroy(a) OctetString_Create(a)
#define ImplementationId_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ImplementationId_GetData(a) OctetString_GetData(a)
#define ImplementationId_GetLength(a) OctetString_GetLength(a)
#define ImplementationName_Create() (PIMPLEMENTATIONNAME)OctetString_CreateTag(IMPLEMENTATIONNAME_TAG)
#define ImplementationName_Destroy(a) OctetString_Create(a)
#define ImplementationName_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ImplementationName_GetData(a) OctetString_GetData(a)
#define ImplementationName_GetLength(a) OctetString_GetLength(a)
#define ImplementationVersion_Create() (PIMPLEMENTATIONVERSION)OctetString_CreateTag(IMPLEMENTATIONVERSION_TAG)
#define ImplementationVersion_Destroy(a) OctetString_Create(a)
#define ImplementationVersion_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ImplementationVersion_GetData(a) OctetString_GetData(a)
#define ImplementationVersion_GetLength(a) OctetString_GetLength(a)
#define ResultSetName_Create() (PRESULTSETNAME)OctetString_CreateTag(RESULTSETNAME_TAG)
#define ResultSetName_Destroy(a) OctetString_Create(a)
#define ResultSetName_SetData(a,b,c) OctetString_SetData(a,b,c)
#define ResultSetName_GetData(a) OctetString_GetData(a)
#define ResultSetName_GetLength(a) OctetString_GetLength(a)

#define INITREQUEST_TAG 20

#define MEMCOPY(a,b,c) memcpy(a,b,c)
#define NEWSTRUCT(a) (a far *)imem_calloc(1,sizeof(a))
#define NEWSTRING(a) (PCHR)imem_calloc(1,a)
#define NEWUSTRING(a) (PUCHR)imem_calloc(1,a)
#define FREE(a) imem_free(a)
#define Z3950_VERSION_2 "11000000"
#define Z3950_VERSION_3 "11100000"
#define InitRequest_GetTag(a) (a)->fldid
#define InitResponse_GetTag(a) (a)->fldid
#define LOG_CRITICAL 0
#define STATE_CRITICAL 0

/* Type Definitions */
typedef DATA_DIR far *PREFERENCEID;
typedef DATA_DIR far *PRESULTSETID;
typedef DATA_DIR far *PELEMENTSETNAME;
typedef DATA_DIR far *PINITREQUEST;
typedef DATA_DIR far *PINITRESPONSE;
typedef DATA_DIR far *POTHERINFO;
typedef DATA_DIR far *PSEARCHREQUEST;
typedef DATA_DIR far *PSEARCHRESPONSE;
typedef DATA_DIR far *PPRESENTREQUEST;
typedef DATA_DIR far *PPRESENTRESPONSE;
typedef DATA_DIR far *POPTIONS;
typedef DATA_DIR far *PPROTOCOLVERSION;
typedef DATA_DIR OBJECTIDENTIFIER;
typedef DATA_DIR far *POBJECTIDENTIFIER;
typedef DATA_DIR OCTETSTRING;
typedef DATA_DIR far *POCTETSTRING;
typedef DATA_DIR BITSTRING;
typedef DATA_DIR far *PBITSTRING;
typedef DATA_DIR INTERNATIONALSTRING;
typedef DATA_DIR far *PINTERNATIONALSTRING;
typedef DATA_DIR EXTERNAL;
typedef DATA_DIR far *PEXTERNAL;
typedef DATA_DIR ANY;
typedef DATA_DIR far *PANY;
typedef DATA_DIR VISIBLESTRING;
typedef DATA_DIR far *PVISIBLESTRING;
typedef OCTETSTRING IMPLEMENTATIONID;
typedef POCTETSTRING PIMPLEMENTATIONID;
typedef OCTETSTRING IMPLEMENTATIONNAME;
typedef POCTETSTRING PIMPLEMENTATIONNAME;
typedef OCTETSTRING IMPLEMENTATIONVERSION;
typedef POCTETSTRING PIMPLEMENTATIONVERSION;
typedef OCTETSTRING RESULTSETNAME;
typedef POCTETSTRING PRESULTSETNAME;
typedef OCTETSTRING PREFERREDRECORDSYNTAX;
typedef POCTETSTRING PPREFERREDRECORDSYNTAX;


/* Method Prototypes */

/* InitRequest*/
PINITREQUEST InitRequest_Create(void);
void InitRequest_Destroy(PINITREQUEST P);
void InitRequest_SetReferenceId(PINITREQUEST P, PREFERENCEID ReferenceId);
PREFERENCEID InitRequest_GetReferenceId(PINITREQUEST P);
void InitRequest_SetProtocolVersion(PINITREQUEST P, PPROTOCOLVERSION ProtocolVersion);
PPROTOCOLVERSION InitRequest_GetProtocolVersion(PINITREQUEST P);
void InitRequest_SetOptions(PINITREQUEST P, POPTIONS Options);
POPTIONS InitRequest_GetOptions(PINITREQUEST P);
void InitRequest_SetPreferredMessageSize(PINITREQUEST P, INT4 PreferredMessageSize);
INT4 InitRequest_GetPreferredMessageSize(PINITREQUEST P);
void InitRequest_SetExceptionalRecordSize(PINITREQUEST P, INT4 ExceptionalRecordSize);
INT4 InitRequest_GetExceptionalRecordSize(PINITREQUEST P);
void InitRequest_SetIdAuthentication(PINITREQUEST P, PANY IdAuthentication);
PANY InitRequest_GetIdAuthentication(PINITREQUEST P);
void InitRequest_SetImplementationId(PINITREQUEST P, PINTERNATIONALSTRING ImplementationId);
PINTERNATIONALSTRING InitRequest_GetImplementationId(PINITREQUEST P);
void InitRequest_SetImplementationName(PINITREQUEST P, PINTERNATIONALSTRING ImplementationName);
PINTERNATIONALSTRING InitRequest_GetImplementationName(PINITREQUEST P);
void InitRequest_SetImplementationVersion(PINITREQUEST P, PINTERNATIONALSTRING ImplementationVersion);
PINTERNATIONALSTRING InitRequest_GetImplementationVersion(PINITREQUEST P);
void InitRequest_SetUserInformationField(PINITREQUEST P, PEXTERNAL UserInformationField);
PEXTERNAL InitRequest_GetUserInformationField(PINITREQUEST P);
void InitRequest_SetOtherInfo(PINITREQUEST P, POTHERINFO OtherInfo);
POTHERINFO InitRequest_GetOtherInfo(PINITREQUEST P);
/* InitResponse*/
PINITRESPONSE InitResponse_Create(void);
void InitResponse_Destroy(PINITRESPONSE P);
void InitResponse_SetReferenceId(PINITRESPONSE P, PREFERENCEID ReferenceId);
PREFERENCEID InitResponse_GetReferenceId(PINITRESPONSE P);
void InitResponse_SetProtocolVersion(PINITRESPONSE P, PPROTOCOLVERSION ProtocolVersion);
PPROTOCOLVERSION InitResponse_GetProtocolVersion(PINITRESPONSE P);
void InitResponse_SetOptions(PINITRESPONSE P, POPTIONS Options);
POPTIONS InitResponse_GetOptions(PINITRESPONSE P);
void InitResponse_SetPreferredMessageSize(PINITRESPONSE P, INT4 PreferredMessageSize);
INT4 InitResponse_GetPreferredMessageSize(PINITRESPONSE P);
void InitResponse_SetExceptionalRecordSize(PINITRESPONSE P, INT4 ExceptionalRecordSize);
INT4 InitResponse_GetExceptionalRecordSize(PINITRESPONSE P);
void InitResponse_SetResult(PINITRESPONSE P, BOOLEAN Result);
BOOLEAN InitResponse_GetResult(PINITRESPONSE P);
void InitResponse_SetImplementationId(PINITRESPONSE P, PINTERNATIONALSTRING ImplementationId);
PINTERNATIONALSTRING InitResponse_GetImplementationId(PINITRESPONSE P);
void InitResponse_SetImplementationName(PINITRESPONSE P, PINTERNATIONALSTRING ImplementationName);
PINTERNATIONALSTRING InitResponse_GetImplementationName(PINITRESPONSE P);
void InitResponse_SetImplementationVersion(PINITRESPONSE P, PINTERNATIONALSTRING ImplementationVersion);
PINTERNATIONALSTRING InitResponse_GetImplementationVersion(PINITRESPONSE P);
void InitResponse_SetUserInfoField(PINITRESPONSE P, PEXTERNAL UserInfoField);
PEXTERNAL InitResponse_GetUserInfoField(PINITRESPONSE P);
void InitResponse_SetOtherInfo(PINITRESPONSE P, POTHERINFO OtherInfo);
POTHERINFO InitResponse_GetOtherInfo(PINITRESPONSE P);
/* OtherInfo*/
POTHERINFO OtherInfo_Create(void);
void OtherInfo_Destroy(POTHERINFO P);
/* SearchRequest*/
PSEARCHREQUEST SearchRequest_Create(void);
void SearchRequest_Destroy(PSEARCHREQUEST P);
void SearchRequest_SetReferenceId(PSEARCHREQUEST P, PREFERENCEID ReferenceId);
PREFERENCEID SearchRequest_GetReferenceId(PSEARCHREQUEST P);
void SearchRequest_SetSmallSetUpperBound(PSEARCHREQUEST P, INT4 SmallSetUpperBound);
INT4 SearchRequest_GetSmallSetUpperBound(PSEARCHREQUEST P);
void SearchRequest_SetLargeSetLowerBound(PSEARCHREQUEST P, INT4 LargeSetLowerBound);
INT4 SearchRequest_GetLargeSetLowerBound(PSEARCHREQUEST P);
void SearchRequest_SetMediumSetPresentNumber(PSEARCHREQUEST P, INT4 MediumSetPresentNumber);
INT4 SearchRequest_GetMediumSetPresentNumber(PSEARCHREQUEST P);
void SearchRequest_SetReplaceIndicator(PSEARCHREQUEST P, BOOLEAN ReplaceIndicator);
BOOLEAN SearchRequest_GetReplaceIndicator(PSEARCHREQUEST P);
void SearchRequest_SetResultSetName(PSEARCHREQUEST P, PINTERNATIONALSTRING ResultSetName);
PINTERNATIONALSTRING SearchRequest_GetResultSetName(PSEARCHREQUEST P);
void SearchRequest_SetPreferredRecordSyntax(PSEARCHREQUEST P, POBJECTIDENTIFIER PreferredRecordSyntax);
POBJECTIDENTIFIER SearchRequest_GetPreferredRecordSyntax(PSEARCHREQUEST P);
void SearchRequest_SetAdditionalSearchInfo(PSEARCHREQUEST P, POTHERINFO AdditionalSearchInfo);
POTHERINFO SearchRequest_GetAdditionalSearchInfo(PSEARCHREQUEST P);
void SearchRequest_SetOtherInfo(PSEARCHREQUEST P, POTHERINFO OtherInfo);
POTHERINFO SearchRequest_GetOtherInfo(PSEARCHREQUEST P);
/* SearchResponse*/
PSEARCHRESPONSE SearchResponse_Create(void);
void SearchResponse_Destroy(PSEARCHRESPONSE P);
void SearchResponse_SetReferenceId(PSEARCHRESPONSE P, PREFERENCEID ReferenceId);
PREFERENCEID SearchResponse_GetReferenceId(PSEARCHRESPONSE P);
void SearchResponse_SetResultCount(PSEARCHRESPONSE P, INT4 ResultCount);
INT4 SearchResponse_GetResultCount(PSEARCHRESPONSE P);
void SearchResponse_SetNumberOfRecordsReturned(PSEARCHRESPONSE P, INT4 NumberOfRecordsReturned);
INT4 SearchResponse_GetNumberOfRecordsReturned(PSEARCHRESPONSE P);
void SearchResponse_SetNextResultSetPosition(PSEARCHRESPONSE P, INT4 NextResultSetPosition);
INT4 SearchResponse_GetNextResultSetPosition(PSEARCHRESPONSE P);
void SearchResponse_SetSearchStatus(PSEARCHRESPONSE P, BOOLEAN SearchStatus);
BOOLEAN SearchResponse_GetSearchStatus(PSEARCHRESPONSE P);
void SearchResponse_SetResultSetStatus(PSEARCHRESPONSE P, INT4 ResultSetStatus);
INT4 SearchResponse_GetResultSetStatus(PSEARCHRESPONSE P);
void SearchResponse_SetPresentStatus(PSEARCHRESPONSE P, INT4 PresentStatus);
INT4 SearchResponse_GetPresentStatus(PSEARCHRESPONSE P);
void SearchResponse_SetAdditionalSearchInfo(PSEARCHRESPONSE P, POTHERINFO AdditionalSearchInfo);
POTHERINFO SearchResponse_GetAdditionalSearchInfo(PSEARCHRESPONSE P);
void SearchResponse_SetOtherInfo(PSEARCHRESPONSE P, POTHERINFO OtherInfo);
POTHERINFO SearchResponse_GetOtherInfo(PSEARCHRESPONSE P);
/* PresentRequest*/
PPRESENTREQUEST PresentRequest_Create(void);
void PresentRequest_Destroy(PPRESENTREQUEST P);
void PresentRequest_SetReferenceId(PPRESENTREQUEST P, PREFERENCEID ReferenceId);
PREFERENCEID PresentRequest_GetReferenceId(PPRESENTREQUEST P);
void PresentRequest_SetResultSetId(PPRESENTREQUEST P, PRESULTSETID ResultSetId);
PRESULTSETID PresentRequest_GetResultSetId(PPRESENTREQUEST P);
void PresentRequest_SetResultSetStartPoint(PPRESENTREQUEST P, INT4 ResultSetStartPoint);
INT4 PresentRequest_GetResultSetStartPoint(PPRESENTREQUEST P);
void PresentRequest_SetNumberOfRecordsRequested(PPRESENTREQUEST P, INT4 NumberOfRecordsRequested);
INT4 PresentRequest_GetNumberOfRecordsRequested(PPRESENTREQUEST P);
void PresentRequest_SetPreferredRecordSyntax(PPRESENTREQUEST P, POBJECTIDENTIFIER PreferredRecordSyntax);
POBJECTIDENTIFIER PresentRequest_GetPreferredRecordSyntax(PPRESENTREQUEST P);
void PresentRequest_SetMaxSegmentCount(PPRESENTREQUEST P, INT4 MaxSegmentCount);
INT4 PresentRequest_GetMaxSegmentCount(PPRESENTREQUEST P);
void PresentRequest_SetMaxRecordSize(PPRESENTREQUEST P, INT4 MaxRecordSize);
INT4 PresentRequest_GetMaxRecordSize(PPRESENTREQUEST P);
void PresentRequest_SetMaxSegmentSize(PPRESENTREQUEST P, INT4 MaxSegmentSize);
INT4 PresentRequest_GetMaxSegmentSize(PPRESENTREQUEST P);
void PresentRequest_SetOtherInfo(PPRESENTREQUEST P, POTHERINFO OtherInfo);
POTHERINFO PresentRequest_GetOtherInfo(PPRESENTREQUEST P);
/* PresentResponse*/
PPRESENTRESPONSE PresentResponse_Create(void);
void PresentResponse_Destroy(PPRESENTRESPONSE P);
void PresentResponse_SetReferenceId(PPRESENTRESPONSE P, PREFERENCEID ReferenceId);
PREFERENCEID PresentResponse_GetReferenceId(PPRESENTRESPONSE P);
void PresentResponse_SetNumberOfRecordsReturned(PPRESENTRESPONSE P, INT4 NumberOfRecordsReturned);
INT4 PresentResponse_GetNumberOfRecordsReturned(PPRESENTRESPONSE P);
void PresentResponse_SetNextResultSetPosition(PPRESENTRESPONSE P, INT4 NextResultSetPosition);
INT4 PresentResponse_GetNextResultSetPosition(PPRESENTRESPONSE P);
void PresentResponse_SetPresentStatus(PPRESENTRESPONSE P, INT4 PresentStatus);
INT4 PresentResponse_GetPresentStatus(PPRESENTRESPONSE P);
void PresentResponse_SetOtherInfo(PPRESENTRESPONSE P, POTHERINFO OtherInfo);
POTHERINFO PresentResponse_GetOtherInfo(PPRESENTRESPONSE P);
/* OctetString */

POCTETSTRING OctetString_Create(void);
POCTETSTRING OctetString_CreateTag(INT4 Tag);
void OctetString_Destroy(POCTETSTRING p);
void OctetString_SetData(POCTETSTRING p, PUCHR d, INT4 l);
PUCHR OctetString_GetData(POCTETSTRING p);
INT4 OctetString_GetLength(POCTETSTRING p);
PCHR OctetString_GetASCIIString(POCTETSTRING p);
void OctetString_Printf(POCTETSTRING p, PCHR f);

DATA_DIR *SearchResponse_GetRecords(PSEARCHRESPONSE p);

#endif /*_z3950_ */
