/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           z3950sup.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/* 
These are routines that I'm generating by hand that will eventually
come from the asn.1 interpretter.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "gdt.h"
#include "libcnidr.h"
#include "berutil.h"
#include "z3950.h"
#include "z3950sup.h"
#include "zmacros.h"
 
PINITREQUEST InitRequest_CreateInitAllASCII(PCHR ReferenceId,
        PCHR ProtocolVersion, PCHR Options, INT4 PreferredMessageSize,
        INT4 ExceptionalRecordSize, PCHR ImplementationId,
        PCHR ImplementationName, PCHR ImplementationVersion,
        PCHR UserInformation)
{
        PREFERENCEID RefId;
        POPTIONS Opt;
        PPROTOCOLVERSION ProtVer;
        PIMPLEMENTATIONID ImpId;
        PIMPLEMENTATIONNAME ImpName;
        PIMPLEMENTATIONVERSION ImpVersion;
        PINITREQUEST Init;

        if((Init = InitRequest_Create())==NULL)
		return NULL;

	if(ReferenceId && ReferenceId[0]!='\0') {
        	RefId = ReferenceId_Create();
        	ReferenceId_SetData(RefId,(PUCHR)ReferenceId, 
			STRLEN(ReferenceId));
        	InitRequest_SetReferenceId(Init, RefId);
	}

        ProtVer = ProtocolVersion_Create();
        if(ProtocolVersion && ProtocolVersion[0]!='\0') {
                ProtocolVersion_SetBits(ProtVer, (PUCHR)ProtocolVersion);
        } else {
                ProtocolVersion_SetBits(ProtVer, (PUCHR)"11");
        }
        InitRequest_SetProtocolVersion(Init, ProtVer);

        Opt = (POPTIONS)Options_Create();
        if(Options && Options[0]!='\0') {
                Options_SetBits(Opt, (PUCHR)Options);
        } else {
                Options_SetBits(Opt, (PUCHR)"11");
        }
        InitRequest_SetOptions(Init, Opt);

        InitRequest_SetPreferredMessageSize(Init, (long)PreferredMessageSize);
        InitRequest_SetExceptionalRecordSize(Init,(long)ExceptionalRecordSize);

	if(ImplementationId && ImplementationId[0]!='\0') {
        	ImpId = ImplementationId_Create();
        	ImplementationId_SetData(ImpId, (PUCHR)ImplementationId,
                	STRLEN(ImplementationId));
        	InitRequest_SetImplementationId(Init, ImpId);
	}

	if(ImplementationName && ImplementationName[0]!='\0') {
        	ImpName = ImplementationName_Create();
        	ImplementationName_SetData(ImpName, (PUCHR)ImplementationName,
                	STRLEN(ImplementationName));
        	InitRequest_SetImplementationName(Init, ImpName);
	}

	if(ImplementationVersion && ImplementationVersion[0]!='\0') {
        	ImpVersion = ImplementationVersion_Create();
        	ImplementationVersion_SetData(ImpVersion, 
			(PUCHR)ImplementationVersion, 
			STRLEN(ImplementationVersion));
        	InitRequest_SetImplementationVersion(Init, ImpVersion);
	}

 
        return Init;
}

PINITRESPONSE InitResponse_CreateInitAllASCII(PCHR ReferenceId,
        PCHR ProtocolVersion, PCHR Options, INT4 PreferredMessageSize,
        INT4 ExceptionalRecordSize, BOOLEAN Result, PCHR ImplementationId,
        PCHR ImplementationName, PCHR ImplementationVersion,
        PCHR UserInformation)
{
        PREFERENCEID RefId;
        POPTIONS Opt;
        PPROTOCOLVERSION ProtVer;
        PIMPLEMENTATIONID ImpId;
        PIMPLEMENTATIONNAME ImpName;
        PIMPLEMENTATIONVERSION ImpVersion;
        PINITRESPONSE Init;
 
        RefId = ReferenceId_Create();
        ReferenceId_SetData(RefId, (PUCHR)ReferenceId, STRLEN(ReferenceId));
        ProtVer = ProtocolVersion_Create();
        if(ProtocolVersion && ProtocolVersion[0]!='\0') {
                ProtocolVersion_SetBits(ProtVer, (PUCHR)ProtocolVersion);
        } else {
                ProtocolVersion_SetBits(ProtVer, (PUCHR)"11");
        }
        Opt = (POPTIONS)Options_Create();
        if(Options && Options[0]!='\0') {
                Options_SetBits(Opt, (PUCHR)Options);
        } else {
                Options_SetBits(Opt, (PUCHR)"11");
        }
        ImpId = ImplementationId_Create();
        ImplementationId_SetData(ImpId, (PUCHR)ImplementationId,
                STRLEN(ImplementationId));
        ImpName = ImplementationName_Create();
        ImplementationName_SetData(ImpName, (PUCHR)ImplementationName,
                STRLEN(ImplementationName));
        ImpVersion = ImplementationVersion_Create();
        ImplementationVersion_SetData(ImpVersion, (PUCHR)ImplementationVersion,
                STRLEN(ImplementationVersion));
        Init = InitResponse_Create();
        InitResponse_SetReferenceId(Init, RefId);
        InitResponse_SetProtocolVersion(Init, ProtVer);
        InitResponse_SetOptions(Init, Opt);
        InitResponse_SetPreferredMessageSize(Init, PreferredMessageSize);
        InitResponse_SetExceptionalRecordSize(Init, ExceptionalRecordSize);
        InitResponse_SetResult(Init, Result);
        InitResponse_SetImplementationId(Init, ImpId);
        InitResponse_SetImplementationName(Init, ImpName);
        InitResponse_SetImplementationVersion(Init, ImpVersion);
 
        return Init;
}
void OctetString_CopyToCString(POCTETSTRING o, PCHR s)
{
	INT4 Len;

	if(o && OctetString_GetData(o) && s) {
		Len = OctetString_GetLength(o);
		MEMCPY(s, OctetString_GetData(o), Len);
		s[Len]='\0';
	}
}

INT ProtocolVersion_GetVersion(PPROTOCOLVERSION p)
{
	INT Version=0;
	INT i;

	if(p) {
		for(i=0;i<8;i++) {
			UCHR bit;
			bit = BitString_GetBit(p, i);
			if(bit == 'y')
				Version++;
			else break;
		}
	}

	return Version;	
}

/*
The Query parameter should be encoded using the internationally known
KWAQS syntax (Kevin's wacky, arcane query syntax :-)

	dog		- search for dog, no attributes
	dog[1,4,2,3]	- search title field for = dog
	and(dog,cat)
	or(dog,cat)
	andnot(dog,cat)
	and(dog,or(cat,mouse))
	etc
*/
PSEARCHREQUEST SearchRequest_CreateInitAllASCII(PCHR ReferenceId,
	INT4 SmallSetUpperBound, INT4 LargeSetLowerBound,
	INT4 MediumSetPresentNumber, BOOLEAN ReplaceIndicator,
	PCHR ResultSetName, PCHR DatabaseNames, 
	PCHR SmallSetElementSetNames, PCHR MediumSetElementSetNames,
	PCHR PreferredRecordSyntax, PCHR Query, PCHR AttributeSetId)
{
        PREFERENCEID RefId;
        PSEARCHREQUEST pdu;
	PRESULTSETNAME RSN;
	PDATABASENAMES DBNames;
       	PUCHR t;
	INT4 i;
	PDATABASENAME DB;
	DATA_DIR *query;
 
	if((pdu = SearchRequest_Create())==NULL)
		return NULL;

	if(ReferenceId && ReferenceId[0]!='\0') {
        	RefId = ReferenceId_Create();
        	ReferenceId_SetData(RefId,(PUCHR)ReferenceId, 
			STRLEN(ReferenceId));
        	SearchRequest_SetReferenceId(pdu, RefId);
	}
	 
        SearchRequest_SetSmallSetUpperBound(pdu, SmallSetUpperBound);
        SearchRequest_SetLargeSetLowerBound(pdu, LargeSetLowerBound);
        SearchRequest_SetMediumSetPresentNumber(pdu, MediumSetPresentNumber);
        SearchRequest_SetReplaceIndicator(pdu, ReplaceIndicator);

       	RSN = ResultSetName_Create();
       	ResultSetName_SetData(RSN,(PUCHR)ResultSetName, 
		STRLEN(ResultSetName));
       	SearchRequest_SetResultSetName(pdu, RSN);

       	DBNames = DatabaseNames_Create();
	for(i=1;i<=strbox_itemCount(DatabaseNames);i++) {
		t = NEWUSTRING(128);
		strbox_item(DatabaseNames, i, (PCHR)t);
		DB = (PDATABASENAME)OctetString_CreateTag(DATABASENAME_TAG);
        	DatabaseName_SetData(DB, (PUCHR)t, STRLEN(t));
		DatabaseNames_Add(DBNames, DB);
	}
	SearchRequest_SetDatabaseNames(pdu, DBNames);
/*
        SearchRequest_SetSmallSetElementSetNames(pdu, );
        SearchRequest_SetMediumSetElementSetNames(pdu, );

	PRS = PreferredRecordSyntax_Create();
	PreferredRecordSyntax_SetData(PRS, (PUCHR)PreferredRecordSyntax,
		STRLEN(PreferredRecordSyntax));
        SearchRequest_SetPreferredRecordSyntax(pdu, PRS);
	
*/
        query = Query_CreateKWAQS(AttributeSetId, Query);
	if(!query) {
		dfree(pdu);
		return NULL;
	}

        SearchRequest_SetQuery(pdu, query);

        return pdu;
}

PQUERY Query_CreateKWAQS(PCHR AttrSet, PCHR S)
{
	PQUERY q;
	PTYPE1QUERY type1;
	PATTRIBUTESETID ATR;
	PRPNSTRUCTURE RpnStruct;
	
	/* Create the query node */
 	q = Query_Create();

	/* Create the type-1 query node */
	type1 = Type1Query_Create();

	/* Hang it under the query node */
	Query_SetType(q, type1);

	/* Create an attribute set id node */
	ATR = AttributeSetId_Create();
	AttributeSetId_SetData(ATR, (PUCHR)AttrSet, STRLEN(AttrSet));
	
	/* Hang it under the type 1 query node */
	Type1Query_SetAttributeSetId(type1, ATR);

	/* Create an operand or rpnrpnop node */
	RpnStruct = RpnStructure_CreateKWAQS(S);
	if(!RpnStruct) {
		dfree(q);
		return NULL;
	}

	/* Hang it under the type 1 query node */
	Type1Query_SetRpnStructure(type1, RpnStruct);

	/* And thats it... */
	return q;
}

DATA_DIR *RpnStructure_CreateKWAQS(PCHR S)
{
	PATTRIBUTESPLUSTERM APT;
	INT op;
	
	if((op = zutil_IsBooleanOp(S))!=-1) {
		PRPNRPNOP rro;
		PCHR arg1, arg2;
		POPERATOR Operator;
		DATA_DIR *OP;

		/* Create rpnrpnop node */
		rro = RpnRpnOp_Create();

		/* Split the string into two operands */
		arg1 = NEWSTRING(strlen(S)+1);
		arg2 = NEWSTRING(strlen(S)+1);
		if(!(arg1) || !(arg2)) {
			fprintf(stderr, "RpnStruture_CreateKWAQS:no memory\n");
			return NULL;
		}
		if(zutil_GetArgs(S, arg1, arg2)==0)
			return NULL;

		/* Add operand 1 */
		RpnRpnOp_SetRpn1(rro, RpnStructure_CreateKWAQS(arg1));
		
		/* Add operand 2 */
		RpnRpnOp_SetRpn2(rro, RpnStructure_CreateKWAQS(arg2));

		/* Add boolean operator */

		/* Create Operator node */
		Operator = Operator_Create();

		/* Create actual Operator type */
		switch(op) {
			case AND_TAG:
				OP = And_Create();
				break;
			case OR_TAG:
				OP = Or_Create();
				break;
			case ANDNOT_TAG:
				OP = AndNot_Create();
				break;
			case PROX_TAG:
				/*
				OP = Proximity_Create();
				*/
				/* Do some other things here */
				break;
			default:
				break;
		}
		if(!OP)
			/* oh well, guess we're done */
			return rro;
	
		/* Hang it under the Operator node */
		Operator_SetType(Operator, OP);

		RpnRpnOp_SetOp(rro, Operator);
		
		return rro;
	} else {
		POPERAND Operand;
		POCTETSTRING Term;
		PATTRIBUTELIST AttrList;

		/* Create operand node */
		if((Operand = Operand_Create())==NULL)
			return NULL;

		/* Create attributes plus term node */
		if((APT = AttributesPlusTerm_Create())==NULL)
			return NULL;

		/* Hang it under the Operand node */
		Operand_SetType(Operand, APT);

		/* Any attributes to encode? */
		AttrList = AttributeList_CreateKWAQS(S);

		/* Hang it under the attributes plus term node */
		if(AttrList)
			AttributesPlusTerm_SetAttributeList(APT, AttrList);
		
		/* Create the term (general) node */
		Term = OctetString_CreateTag(TERMGENERAL_TAG);
		zutil_StripAttributes(S);
		OctetString_SetData(Term, (PUCHR)S, STRLEN(S));
	
		/* Hang it under the attributes plus term */	
		AttributesPlusTerm_SetTermGeneral(APT, Term);
	
		return Operand;
	}
}
	
void SearchRequest_SetDatabaseNames(PSEARCHREQUEST s, PDATABASENAMES d)
{
        if(s && d) {
		if(dadddir(s, d)==NULL) {
                }
        }
}

void SearchRequest_SetQuery(PSEARCHREQUEST s, PQUERY q)
{
        if(s && q) {
		if(dadddir(s, q)==NULL) {
                }
        }
}

void Query_SetType(PQUERY q, DATA_DIR *d)
{
        if(q && d) {
		if(dadddir(q, d)==NULL) {
                }
        }
}

PSEARCHRESPONSE SearchResponse_CreateInitAllASCII(PCHR ReferenceId,
        INT4 ResultCount, INT4 NumRecordsReturned, INT4 NextResultSetPos,
	BOOLEAN SearchStatus, INT4 ResultSetStatus, INT4 PresentStatus,
	DATA_DIR *Records,  DATA_DIR *AdditionalSearchInfo,
	DATA_DIR *OtherInfo)
{
        PREFERENCEID RefId;
        PSEARCHRESPONSE pdu;
 
        RefId = ReferenceId_Create();
        ReferenceId_SetData(RefId, (PUCHR)ReferenceId, STRLEN(ReferenceId));
        pdu = SearchResponse_Create();
        SearchResponse_SetReferenceId(pdu, RefId);
        SearchResponse_SetResultCount(pdu, ResultCount);
        SearchResponse_SetNumberOfRecordsReturned(pdu, NumRecordsReturned);
        SearchResponse_SetNextResultSetPosition(pdu, NextResultSetPos);
        SearchResponse_SetSearchStatus(pdu, SearchStatus);
        SearchResponse_SetResultSetStatus(pdu, ResultSetStatus);
	if(Records!=NULL)
		SearchResponse_SetRecords(pdu, Records);	 

        return pdu;
}

PPRESENTREQUEST PresentRequest_CreateInitAllASCII(PCHR ReferenceId,
	PCHR ResultSetId, INT4 ResultSetStartPoint, 
	INT4 NumberOfRecordsRequested, PCHR ElementSetName,
	PCHR PreferredRecordSyntax)
{
        PREFERENCEID RefId;
        PSEARCHREQUEST pdu;
	POCTETSTRING RSI;
	PELEMENTSETNAME ESN;
	PPREFERREDRECORDSYNTAX PRS;
	PRECORDCOMPOSITION RC;
 
	if((pdu = PresentRequest_Create())==NULL)
		return NULL;

	if(ReferenceId && ReferenceId[0]!='\0') {
        	RefId = ReferenceId_Create();
        	ReferenceId_SetData(RefId,(PUCHR)ReferenceId, 
			STRLEN(ReferenceId));
        	PresentRequest_SetReferenceId(pdu, RefId);
	}

       	RSI = ResultSetId_Create();
       	ResultSetId_SetData(RSI,(PUCHR)ResultSetId, 
		STRLEN(ResultSetId));
       	PresentRequest_SetResultSetName(pdu, RSI);
        
	PresentRequest_SetResultSetStartPoint(pdu, ResultSetStartPoint);
        PresentRequest_SetNumberOfRecordsRequested(pdu,
		NumberOfRecordsRequested);


	RC = RecordComposition_CreateSimple();

       	ESN = OctetString_CreateTag(0);
       	OctetString_SetData(ESN,(PUCHR)ElementSetName, 
		STRLEN(ElementSetName));

	RecordComposition_SetSimpleGeneric(RC, ESN);

	PresentRequest_SetRecordComposition(pdu, RC);

	PRS = PreferredRecordSyntax_Create();
	PreferredRecordSyntax_SetData(PRS, (PUCHR)PreferredRecordSyntax,
		STRLEN(PreferredRecordSyntax));
        PresentRequest_SetPreferredRecordSyntax(pdu, PRS);
	
        return pdu;
}
PPRESENTRESPONSE PresentResponse_CreateInitAllASCII(PCHR ReferenceId,
        INT4 NumRecordsReturned, INT4 NextResultSetPos,
	INT4 PresentStatus, DATA_DIR *Records, DATA_DIR *OtherInfo)
{
        PREFERENCEID RefId;
        PPRESENTRESPONSE pdu;
 
        RefId = ReferenceId_Create();
        ReferenceId_SetData(RefId, (PUCHR)ReferenceId, STRLEN(ReferenceId));
        pdu = PresentResponse_Create();
        PresentResponse_SetReferenceId(pdu, RefId);
        PresentResponse_SetNumberOfRecordsReturned(pdu, NumRecordsReturned);
        PresentResponse_SetNextResultSetPosition(pdu, NextResultSetPos);
        PresentResponse_SetPresentStatus(pdu, PresentStatus);
	if(Records)
		PresentResponse_SetRecords(pdu, Records); 

        return pdu;
}

POCTETSTRING OctetString_CreateTagUniv(INT4 Tag)
{
        POCTETSTRING p;
 
        if((p= NEWSTRUCT(OCTETSTRING))==NULL) {
                return NULL;
        }
 
        p->next = NULL_DIR;
        p->user = 0;
        p->ptr.data = NULL;
        p->count = 0;
        p->fldid = Tag;
        p->Class = ASN1_UNIVERSAL;
        p->form = ASN1_PRIMITIVE;
        return p;
}

PRESPONSERECORDS ResponseRecords_Create(void)
{
	PRESPONSERECORDS t;

	if((t=dmake(RESPONSERECORDS_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

void ResponseRecords_Destroy(PRESPONSERECORDS r)
{
	if(r) {
		dfree(r);
	}
}

/*
n must be a sequence tag with name plus record hanging under it
*/
void ResponseRecords_Add(PRESPONSERECORDS r, PNAMEPLUSRECORD n)
{
        if(r && n) {
		if(dadddir(r, n)==NULL) {
                }
        }

}

void PresentRequest_SetResultSetName(PPRESENTREQUEST r, PRESULTSETNAME n)
{
        if(r && n) {
		if(dadddir(r, n)==NULL) {
                }
        }

}
void ResponseRecords_Delete(PRESPONSERECORDS r, INT4 c)
{
}

INT4 ResponseRecords_GetCount(PRESPONSERECORDS r)
{
	if(r) {
		return r->count;
	}
	return -1;
}

/* 
starts with 1
*/
PRETRIEVALRECORD ResponseRecords_GetRecord(PRESPONSERECORDS r, INT4 c)
{
	if(r) {	
		INT4 i;
		DATA_DIR *d=r->ptr.child, *t;
		if(d) {
			for(t=d,i=1;i<c;i++) {
				t = t->next;	
			}	
			if((t) && (t->ptr.child))
				return (PRECORD)zutil_GetTaggedObject(t,
					RECORD_TAG);
		}
		return NULL;
	}
	return NULL;
	
}

PNAMEPLUSRECORD ResponseRecords_GetName(PRESPONSERECORDS r, INT4 c)
{
	if(r) {	
		INT4 i;
		DATA_DIR *d=r->ptr.child, *t;
		if(d) {
			for(t=d,i=1;i<c;i++) {
				t = t->next;	
			}	
			if((t) && (t->ptr.child))
				return (PDATABASENAME)zutil_GetTaggedObject(t,
					0);
		}
		return NULL;
	}
	return NULL;
	
}

void ResponseRecords_AddSUTRS(PRESPONSERECORDS r, PCHR db, PCHR d, INT4 l)
{
}

PQUERY Query_Create(void)
{
	PQUERY t;

	if((t=dmake(QUERY_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

 
PNAMEPLUSRECORD NamePlusRecord_Create(void)
{
	PNAMEPLUSRECORD t;

	if((t=dmake(ASN1_SEQUENCE, ASN1_UNIVERSAL, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

void NamePlusRecord_Destroy(PNAMEPLUSRECORD n)
{
	if(n)
		dfree(n);
}

void NamePlusRecord_SetName(PNAMEPLUSRECORD n, PDATABASENAME d)
{
	if(n && d) {
                if(d->ptr.data && d->ptr.data[0]!='\0' && d->count > 0) {
                        if(dadddir(n, d)==NULL) {
                        }
                }
	}
}

PDATABASENAME NamePlusRecord_GetName(PNAMEPLUSRECORD n)
{
        if(n)
                return (PDATABASENAME)zutil_GetTaggedObject(n,DATABASENAME_TAG);
	return NULL;
}

INT4 NamePlusRecord_GetRecordType(PNAMEPLUSRECORD n)
{
	if(n)
		return n->fldid;
	return -1;
}

PEXTERNAL NamePlusRecord_GetRetrievalRecord(PNAMEPLUSRECORD n)
{
        if(n)
                return (PEXTERNAL)zutil_GetTaggedObject(n,RETRIEVALRECORD_TAG);
	return NULL;
	
}

PDIAGREC NamePlusRecord_GetSurrogateDiagnostic(PNAMEPLUSRECORD n)
{
	return NULL;
}

void NamePlusRecord_SetRetrievalRecord(PNAMEPLUSRECORD n, PEXTERNAL r)
{
	if(n) {
		DATA_DIR *d=daddtag(n, RECORD_TAG, ASN1_CONTEXT);
		if(d) {
			DATA_DIR *d1=daddtag(d, RETRIEVALRECORD_TAG, 
				ASN1_CONTEXT);
			if(d1) {
				if(!dadddir(d1,r)) {
					fprintf(stderr, "NamePlusRecord_SetRetrievalRecord()\n");
				}
			}
		}	
	}
}

void NamePlusRecord_SetSurrogateDiagnostic(PNAMEPLUSRECORD n, PDIAGREC r)
{
}

PEXTERNAL External_Create(void)
{
	PEXTERNAL t;

	if((t=dmake(ASN1_EXTERNAL, ASN1_UNIVERSAL, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

void External_Destroy(PEXTERNAL e)
{
	if(e)
		dfree(e);
}

void External_SetDirectReference(PEXTERNAL e, POBJECTIDENTIFIER o)
{
        /*
        cheat until I do it properly.  User should never notice the
        difference though!
        */
        if(e && o) {
                PCHR t;
                t = NEWSTRING(o->count);
                strncpy((PCHR)t, (PCHR)o->ptr.data, o->count);
                t[o->count]='\0';
                if(daddoid(e, ASN1_OBJECTIDENTIFIER, ASN1_UNIVERSAL,t)==NULL){
                }
        }
}

void External_SetEncodingSingle(PEXTERNAL e, PANY a)
{
	if(e && a) {
		DATA_DIR *d=daddtag(e, SINGLE_ASN1_TYPE_TAG, ASN1_CONTEXT);
		if(d) {
			if(!dadddir(d,a)) {
				fprintf(stderr, "NamePlusRecord_SetRetrievalRecord()\n");
			}
		}	
	}
	
}

POBJECTIDENTIFIER External_GetDirectReference(PEXTERNAL e)
{
        if(e)
                return (POBJECTIDENTIFIER)zutil_GetTaggedObject(e,
			DIRECTREFERENCE_TAG);
	return NULL;
}

PANY External_GetEncodingSingle(PEXTERNAL e)
{
        if(e)
                return (PANY)zutil_GetTaggedObject(e,
			0);
	return NULL;
}

PANY External_GetEncodingAligned(PEXTERNAL e)
{
        if(e)
                return (PANY)zutil_GetTaggedObject(e, 1);
	
	return NULL;
}

INT4 External_GetEncodingType(PEXTERNAL e)
{
	return -1;
}

PRESPONSERECORDS PresentResponse_GetResponseRecords(PPRESENTRESPONSE p)
{
        if(p!=NULL)
                return (PRESPONSERECORDS)zutil_GetTaggedObject(p,
			RESPONSERECORDS_TAG);
	return NULL;
}

PNONSURROGATEDIAGNOSTIC PresentResponse_GetNonSurrogateDiagnostic(PPRESENTRESPONSE p)
{
        if(p)
                return (PNONSURROGATEDIAGNOSTIC)zutil_GetTaggedObject(p,
			NONSURROGATEDIAGNOSTIC_TAG);
	return NULL;
}

INT PresentResponse_GetRecordType(PPRESENTRESPONSE p)
{
        if(p!=NULL) {
                DATA_DIR *d;
		d = (PNONSURROGATEDIAGNOSTIC)zutil_GetTaggedObject(p,
			NONSURROGATEDIAGNOSTIC_TAG);
		if(d!=NULL)
			return NONSURROGATEDIAGNOSTIC_TAG;
                d = (PRESPONSERECORDS)zutil_GetTaggedObject(p,
			RESPONSERECORDS_TAG);
		if(d!=NULL)
			return RESPONSERECORDS_TAG;
		/* Don't know what type of records we have! */
		return -1;
	}
	return NULL;
}

void SearchResponse_SetRecords(PSEARCHRESPONSE p, DATA_DIR *r)
{
        if((p!=NULL) && (r!=NULL)) {
		if(dadddir(p, r)==NULL) {
                }
        }
}

void PresentResponse_SetRecords(PPRESENTRESPONSE p, PRESPONSERECORDS r)
{
        if(p && r) {
		if(dadddir(p, r)==NULL) {
                }
        }
}

INT4 PresentRequest_GetRecordCompositionType(PPRESENTREQUEST p)
{
	return -1;
}

PRECORDCOMPOSITION RecordComposition_CreateSimple()
{
	PRECORDCOMPOSITION t;

	if((t=dmake(SIMPLE_TAG, ASN1_CONTEXT, 3))==NULL) {
                /* Enter any error logging here */
		fprintf(stderr, "RecordComposition_CreateSimple():\n");
                return NULL;
        }
	return t;
 
}

void RecordComposition_SetSimpleGeneric(PRECORDCOMPOSITION r,
	PINTERNATIONALSTRING i)
{
        if(r && i) {
		if(dadddir(r, i)==NULL) {
                }
        }
}

void PresentRequest_SetRecordComposition(PPRESENTREQUEST p, PRECORDCOMPOSITION e)
{
        if(p && e) {
		if(dadddir(p, e)==NULL) {
			fprintf(stderr, "Can't set record comp\n");
                }
        }
}

PELEMENTSETNAMES PresentRequest_GetSimple(PPRESENTREQUEST p)
{
        if(p)
                return (PELEMENTSETNAMES)zutil_GetTaggedObject(p,
			SIMPLE_TAG);
	return NULL;
}

void ElementSetNames_SetGeneric(PELEMENTSETNAMES e, PINTERNATIONALSTRING i)
{
}

PINTERNATIONALSTRING ElementSetNames_GetGeneric(PELEMENTSETNAMES e)
{
	if(e)
                return (PINTERNATIONALSTRING)zutil_GetTaggedObject(e,
			GENERICELEMENTSETNAME_TAG);
	return NULL;
}

DATA_DIR *PresentRequest_GetComplex(PPRESENTREQUEST p)
{
	return NULL;
}

/* 
PRE:	stack has been created using stack_Create()
	rpn structure is valid
POST:	Returns a stack with ATOMs pushed onto it relative to the search
	terms.
NOTE: Doesn't support result set names as query terms !
*/
STACK *RpnQuery_GetStack(PRPNQUERY rpn, STACK *stack)
{
	PQUERYATOM atom;
	
	atom=NEWSTRUCT(QUERYATOM);
	if(rpn) {
		POPERAND Operand;
		if(zutil_GetTag(rpn) == OP_TAG) {
			/* single term */
			(INT)atom->which = OP_TAG;
			Operand = rpn;

			if(Operand_GetType(Operand)==ATTRIBUTESPLUSTERM_TAG) {
				PATTRIBUTESPLUSTERM apt;
				POCTETSTRING Term;
				PATTRIBUTELIST al;
				INT4 ECount, i;

				apt=Operand_GetAttributesPlusTerm(Operand);
				Term = AttributesPlusTerm_GetTermGeneral(apt);

				/* Copy the query term into the atom */
				atom->Data=NEWSTRING(
					OctetString_GetLength(Term)+256);
				strncpy(atom->Data, 
					(PCHR)OctetString_GetData(Term),
					OctetString_GetLength(Term));
				atom->Data[OctetString_GetLength(Term)]='\0';
			
				/* Copy the attribute list into the atom */
				al = AttributesPlusTerm_GetAttributeList(apt);
				if(al) {
				strcat(atom->Data,"[");
				ECount = AttributeList_GetElementCount(al);
				for(i=1;i<=ECount;i++) {
					PATTRIBUTEELEMENT e;
					CHR Temp[64];

					e = AttributeList_GetElement(al,i);
					if(!e) {
						continue;
}

					sprintf(Temp,"%d,%d",
					AttributeElement_GetType(e),
					AttributeElement_GetValueNumeric(e));

					strcat(atom->Data,Temp);
					if(i<ECount)
						strcat(atom->Data,",");
					else
						strcat(atom->Data,"]");		
				}
				} 
				stack_Push(stack,atom,LIST_NEAR);
			} else {
				/* Its not AttributesPlusTerm */
				return NULL;
			}
		} else {
			/* boolean */
			/* Push the operator onto the stack */
			PRPNRPNOP RpnRpnOp;
			POPERATOR Operator;

			RpnRpnOp = rpn;

			Operator = RpnRpnOp_GetOperator(RpnRpnOp);

			atom->which = OPERATOR_TAG;
			atom->Data = (PCHR)imem_calloc(1,64);

			switch(Operator_GetType(Operator)) {
				case AND_TAG:
					strcpy(atom->Data,"AND");
					break;
				case OR_TAG:
					strcpy(atom->Data,"OR");
					break;
				case ANDNOT_TAG:
					strcpy(atom->Data,"AND-NOT");
					break;
				case PROX_TAG:
					/* need to decode prox operator
					structure here and perhaps
					append the info to this operator */
					strcpy(atom->Data,"PROX");
					break;
			}	
			stack_Push(stack,atom,LIST_NEAR);
			/* Push operand #2 onto the stack */
			RpnQuery_GetStack(RpnRpnOp->ptr.child->next, stack);
	
			/* Push operand #1 onto the stack */
			RpnQuery_GetStack(RpnRpnOp->ptr.child, stack);
		}
	}
/*
FIX THIS - memory freeing code!
	atom_Free(atom);
*/

	return(stack);
}

PQUERY SearchRequest_GetQuery(PSEARCHREQUEST p)
{
	if(p)
                return (PQUERY)zutil_GetTaggedObject(p,
			QUERY_TAG);
	return NULL;
}

INT4 Query_GetQueryType(PQUERY p)
{
	if((p) && (p->form!=0) && (p->ptr.child)) {
		return p->ptr.child->fldid;
	}
	return -1;
}

PRPNQUERY Query_GetQueryType1(PQUERY p)
{
	if(p)
                return zutil_GetTaggedObject(p,TYPE1_TAG);
	return NULL;
}

PRPNQUERY Type1Query_Create()
{
	PTYPE1QUERY r;

	if((r=dmake(TYPE1_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return r;
}

PATTRIBUTESETID RpnQuery_GetAttributeSetId(PRPNQUERY r)
{
	if(r)
		return zutil_GetTaggedObject(r,ATTRIBUTESETID_TAG);

	return NULL;
}

POPERAND Operand_Create()
{
	PRPNQUERY r;

	if((r=dmake(OP_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
	return r; 
}

void Operand_SetType(POPERAND p, DATA_DIR *t)
{
	if(p && t)
		dadddir(p,t);
}


INT4 RpnQuery_GetRpnType(PRPNQUERY r)
{
	DATA_DIR *d;
	if((d=zutil_GetTaggedObject(r,OP_TAG)))
		return zutil_GetTag(d);
	if((d=zutil_GetTaggedObject(r,RPNRPNOP_TAG)))
		return zutil_GetTag(d);
	return -1;
}

POPERAND RpnQuery_GetRpnOp(PRPNQUERY r)
{
	if(r)
                return (POPERAND)zutil_GetTaggedObject(r, OP_TAG);
	return NULL;

}

PRPNRPNOP RpnQuery_GetRpnRpnRpnOp(PRPNQUERY r)
{
	if(r)
                return (PRPNRPNOP)zutil_GetTaggedObject(r, RPNRPNOP_TAG);
	return NULL;
}

INT4 Operand_GetType(POPERAND o)
{
	DATA_DIR *d;

	if((d=zutil_GetTaggedObject(o,ATTRIBUTESPLUSTERM_TAG)))
		return zutil_GetTag(d);
	if((d=zutil_GetTaggedObject(o,RESULTSETID_TAG)))
		return zutil_GetTag(d);
	if((d=zutil_GetTaggedObject(o,RESULTSETPLUSATTRIBUTES_TAG)))
		return zutil_GetTag(d);
	return -1;
}

PATTRIBUTESPLUSTERM Operand_GetAttributesPlusTerm(POPERAND o)
{
	if(o)
		return (PATTRIBUTESPLUSTERM)zutil_GetTaggedObject(o,
			ATTRIBUTESPLUSTERM_TAG);
	return NULL;
}

PATTRIBUTESPLUSTERM AttributesPlusTerm_Create()
{
	PATTRIBUTESPLUSTERM t;

	if((t=dmake(ATTRIBUTESPLUSTERM_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;

}

PATTRIBUTELIST AttributeList_Create()
{
	PATTRIBUTELIST t;

	if((t=dmake(ATTRIBUTELIST_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

void AttributeList_Add(PATTRIBUTELIST a, PATTRIBUTEELEMENT e)
{
        if(a && e) {
		if(dadddir(a, e)==NULL) {
                }
        }
}

void AttributesPlusTerm_SetTermGeneral(PATTRIBUTESPLUSTERM a, POCTETSTRING t)
{
        if(a && t) {
		if(dadddir(a, t)==NULL) {
                }
        }
}

void AttributesPlusTerm_SetAttributeList(PATTRIBUTESPLUSTERM a,
	PATTRIBUTELIST l)
{
        if(a && l) {
		if(dadddir(a, l)==NULL) {
                }
        }

}

PATTRIBUTELIST AttributesPlusTerm_GetAttributeList(PATTRIBUTESPLUSTERM a)
{
	if(a)
		return (PATTRIBUTELIST)zutil_GetTaggedObject(a,
			ATTRIBUTELIST_TAG);
	return NULL;
}

POCTETSTRING AttributesPlusTerm_GetTermGeneral(PATTRIBUTESPLUSTERM a)
{
	if(a)
		return (POCTETSTRING)zutil_GetTaggedObject(a,
			TERMGENERAL_TAG);
	return NULL;
}

INT4 AttributeList_GetElementCount(PATTRIBUTELIST a)
{
	if(a)
		return a->count;
	return -1;
}

PATTRIBUTEELEMENT AttributeList_GetElement(PATTRIBUTELIST a, INT4 e)
{
	if(a) {
		INT4 i=1;
		DATA_DIR *d=a->ptr.child; 
		while((d) && i<e) {
			d=d->next;
			i++;
		}
		if(e==i)
			return d;
	}
	return NULL;
}

void AttributeElement_SetType(PATTRIBUTEELEMENT a, INT4 t)
{
        if(a) {
                if(daddnum(a, ATTRIBUTETYPE_TAG, ASN1_CONTEXT, (PUCHR)&t, 
			sizeof(t))==NULL) {
                }
        }
}

void AttributeElement_SetValueNumeric(PATTRIBUTELIST a, INT4 t)
{
        if(a) {
                if(daddnum(a, NUMERIC_TAG, ASN1_CONTEXT, (PUCHR)&t, 
			sizeof(t))==NULL) {
                }
        }
}

void AttributeList_AddElement(PATTRIBUTELIST a, DATA_DIR *e)
{
        if(a && e) {
		if(dadddir(a, e)==NULL) {
                }
        }
}

INT4 AttributeElement_GetType(PATTRIBUTEELEMENT a)
{
        if(a)
                return (INT4)zutil_GetTaggedInt(a,ATTRIBUTETYPE_TAG);
	return -1;
}

PATTRIBUTEELEMENT AttributeElement_Create()
{
	PATTRIBUTEELEMENT t;

	if((t=dmake(ASN1_SEQUENCE, ASN1_UNIVERSAL, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;

}

INT4 AttributeElement_GetValueNumeric(PATTRIBUTEELEMENT a)
{
        if(a)
                return (INT4)zutil_GetTaggedInt(a,NUMERIC_TAG);
	return -1;
}

PRPNRPNOP RpnRpnOp_Create()
{
	PRPNRPNOP t;

	if((t=dmake(RPNRPNOP_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;

}

DATA_DIR *RpnRpnOp_GetOperator(PRPNRPNOP r)
{
	if(r)
		return (DATA_DIR *)zutil_GetTaggedObject(r, OPERATOR_TAG);
	return NULL;
}

INT4 Operator_GetType(POPERATOR o)
{
	if((o) && (o->ptr.child))
		return o->ptr.child->fldid;
	return -1;
}

PNAMEPLUSRECORD NamePlusRecord_CreateHTML(PCHR db, DATA_DIR *html)
{
        POBJECTIDENTIFIER Oid;
        PEXTERNAL Ext;
        PNAMEPLUSRECORD NPR;
        PDATABASENAME DB;
 
        DB = (PDATABASENAME)OctetString_CreateTag(0);
        DatabaseName_SetData(DB, (PUCHR)db,
                STRLEN(db));
 
        Oid = ObjectIdentifier_Create();
        ObjectIdentifier_SetData(Oid,(PUCHR)HTML_OID,
                STRLEN(HTML_OID));
 
        Ext=External_Create();
        External_SetDirectReference(Ext, Oid);
        External_SetEncodingSingle(Ext, html);
 
        NPR = NamePlusRecord_Create();
        NamePlusRecord_SetName(NPR, DB);
        NamePlusRecord_SetRetrievalRecord(NPR, Ext);
 
        return NPR;
}

PNAMEPLUSRECORD NamePlusRecord_CreateSUTRS(PCHR db, PSUTRS Sutrs)
{
        POBJECTIDENTIFIER Oid;
        PEXTERNAL Ext;
        PNAMEPLUSRECORD NPR;
        PDATABASENAME DB;
 
        DB = (PDATABASENAME)OctetString_CreateTag(0);
        DatabaseName_SetData(DB, (PUCHR)db,
                STRLEN(db));
 
        Oid = ObjectIdentifier_Create();
        ObjectIdentifier_SetData(Oid,(PUCHR)SUTRS_OID,
                STRLEN(SUTRS_OID));
 
        Ext=External_Create();
        External_SetDirectReference(Ext, Oid);
 
        External_SetEncodingSingle(Ext, Sutrs);
 
        NPR = NamePlusRecord_Create();
        NamePlusRecord_SetName(NPR, DB);
        NamePlusRecord_SetRetrievalRecord(NPR, Ext);
 
        return NPR;
}

PCHR ObjectIdentifier_GetData(POBJECTIDENTIFIER o)
{
	if(o)
		return dgetoid(o);
	return NULL;
}

INT4 ObjectIdentifier_GetLength(POBJECTIDENTIFIER o)
{
	if(o)
		return (INT4)STRLEN(dgetoid(o));
	return -1;
}

PDATABASENAMES DatabaseNames_Create()
{
	PDATABASENAMES t;

	if((t=dmake(DATABASENAMES_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

void DatabaseNames_Add(PDATABASENAMES n, PDATABASENAME d)
{
        if(n && d) {
		if(dadddir(n, d)==NULL) {
                }
        }
}

void SearchRequest_SetPreferredRecordSyntax(PSEARCHREQUEST s,POBJECTIDENTIFIER o)
{
        /*
        cheat until I do it properly.  User should never notice the
        difference though!
        */
        if(s && o) {
                PCHR t;
                t = NEWSTRING(o->count);
                strncpy((PCHR)t, (PCHR)o->ptr.data, o->count);
                t[o->count]='\0';
                if(daddoid(s, PREFERREDRECORDSYNTAX_TAG, ASN1_CONTEXT,t)==NULL){
                }
        }
}

void RpnQuery_SetOperand(PRPNQUERY q, POPERAND o)
{
        if(q && o) {
		if(dadddir(q, o)==NULL) {
                }
        }
}

void Type1Query_SetRpnStructure(PTYPE1QUERY q, PRPNSTRUCTURE r)
{
        if(q && r) {
		if(dadddir(q, r)==NULL) {
			fprintf(stderr, "Type1Query_Set:Failed\n");
                }
        }

}

void Type1Query_SetAttributeSetId(PTYPE1QUERY q,POBJECTIDENTIFIER o)
{
        /*
        cheat until I do it properly.  User should never notice the
        difference though!
        */
        if(q && o) {
                PCHR t;
                t = NEWSTRING(o->count);
                strncpy((PCHR)t, (PCHR)o->ptr.data, o->count);
                t[o->count]='\0';
                if(daddoid(q, OBJECTIDENTIFIER_TAG, ASN1_UNIVERSAL,t)==NULL){
                }
        }
}

DATA_DIR *NamePlusRecord_GetRecord(PNAMEPLUSRECORD n)
{
	if(n)
		return (DATA_DIR *)zutil_GetTaggedObject(n, 
			RECORD_TAG);
	return NULL;
}

void PresentRequest_SetPreferredRecordSyntax(PPRESENTREQUEST s,POBJECTIDENTIFIER o)
{
        /*
        cheat until I do it properly.  User should never notice the
        difference though!
        */
        if(s && o) {
                PCHR t;
                t = NEWSTRING(o->count);
                strncpy((PCHR)t, (PCHR)o->ptr.data, o->count);
                t[o->count]='\0';
                if(daddoid(s, PREFERREDRECORDSYNTAX_TAG, ASN1_CONTEXT,t)==NULL){
                }
        }
}

PEXTERNAL RetrievalRecord_GetExternal(PRETRIEVALRECORD r)
{
	if(r)
		return (DATA_DIR *)zutil_GetTaggedObject(r, 
			ASN1_EXTERNAL);
	return NULL;
}

/*
Attribute types/values are encoded within brackets, comma-delimited.
For example:
S = "term[1,2,3,4,5,6]"
*/
PATTRIBUTELIST AttributeList_CreateKWAQS(PCHR S)
{
	PATTRIBUTELIST a;
	PCHR t,p;
	INT4 i, Right=0, Left=0, Type, Value;
	CHR EvenOdd=0;
	PCHR TempS;

#ifdef DEBUG_PARSE
	printf("AttributeList_Create:Starting with: '%s'\n", S);
#endif
	if((a=AttributeList_Create())==NULL) {
		fprintf(stderr, "AttributeList_Create:no memory\n");
		return NULL;
	}

	if((TempS = NEWSTRING(strlen(S)+1))==NULL) {
		fprintf(stderr, "AttributeList_CreateKWAQS:no memory.\n");
		return NULL;
	}

	strcpy(TempS, S);

	/* sanity check.  Are there bounding brackets? */
	for(i=0;i<strlen(TempS);i++) {
		if(TempS[i]=='[')
			Left++;
		if(TempS[i]==']')
			Right++;
	}
	if(Left!=Right) {
		fprintf(stderr, 
			"AttributeList_CreateKWAQS:Query syntax error.\n");
		fprintf(stderr, "-->Expected even number of brackets.\n");
		fprintf(stderr,"-->Brackets [] reserved for attribute list.\n");
		FREE(TempS);
		return NULL;
	}
	if(Left==0) {
		/* No attributes */
#ifdef DEBUG_PARSE
		printf("AttributeList_Create:No attributes available\n");
#endif
		FREE(TempS);
		return NULL;
	}

	t=strchr(TempS, '[');
	
	/* try to sanity check a little */
	if(TempS[strlen(TempS)-2]==',') {
		fprintf(stderr, "AttributeList_CreateKWAQS:Syntax error\n");
		fprintf(stderr, "-->You must supply valid attribute pair.\n");
		FREE(TempS);
		return NULL;
	}
	if(*(t+1)==',') {
		fprintf(stderr, "AttributeList_CreateKWAQS:Syntax error\n");
		fprintf(stderr, "-->You must supply valid attribute pair.\n");
		FREE(TempS);
		return NULL;
	}
	t+=1;

	if((p = strtok(t,"],"))==NULL) {
#ifdef DEBUG_PARSE
		printf("AttributeList_Create:Didn't find a ] or ,\n");
#endif
		FREE(TempS);
		return NULL;
	}
	Type = atoi(p);
#ifdef DEBUG_PARSE
	printf("AttributeList_Create:First Type: '%d'\n", Type);
#endif
	EvenOdd = 1;

	while(p=strtok(NULL,"],")) {
		if(EvenOdd == 0) {
			Type = atoi(p);	
#ifdef DEBUG_PARSE
	printf("AttributeList_Create:Type: '%d'\n", Type);
#endif
			EvenOdd = 1;
		} else {
			PATTRIBUTEELEMENT e;

			if((e = AttributeElement_Create())==NULL) {
				fprintf(stderr, 
					"AttributeList_Create:no memory\n");
				FREE(TempS);
				return NULL;
			}
			Value = atoi(p);
#ifdef DEBUG_PARSE
			printf("AttributeList_Create:Value: '%d'\n", Value);
#endif
			EvenOdd = 0;
			/* We have a pair */
			AttributeElement_SetType(e, Type);
			AttributeElement_SetValueNumeric(e, Value);
			AttributeList_Add(a,e);
		}
	}
	if(EvenOdd == 1) {
		fprintf(stderr, 
			"AttributeList_CreateKWAQS:Odd number of attributes\n");
		fprintf(stderr, 
			"AttributeList_CreateKWAQS:Truncating attributes\n");
		FREE(TempS);
		return NULL;
	}
	FREE(TempS);
	return a;
}

void RpnRpnOp_SetRpn1(PRPNRPNOP r, DATA_DIR *o)
{
	if(!(r) || !(o))
		return;
	if(dadddir(r, o)==NULL)
		fprintf(stderr, "RpnRpnOp_SetRpn1:Failed\n");
}

void RpnRpnOp_SetRpn2(PRPNRPNOP r, DATA_DIR *o)
{
	if(!(r) || !(o))
		return;
	if(dadddir(r, o)==NULL)
		fprintf(stderr, "RpnRpnOp_SetRpn2:Failed\n");
}

void RpnRpnOp_SetOp(PRPNRPNOP r, POPERATOR o)
{
	if(!(r) || !(o))
		return;
	if(dadddir(r, o)==NULL)
		fprintf(stderr, "RpnRpnOp_SetOp:Failed\n");
}

POPERATOR Operator_Create()
{
	POPERATOR o;

	if((o=dmake(OPERATOR_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return o;
}

PAND And_Create()
{
	PAND a;

	if((a=dmake(AND_TAG, ASN1_CONTEXT, 1))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
	return a;
}

POR Or_Create()
{
	POR a;

	if((a=dmake(OR_TAG, ASN1_CONTEXT, 1))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
	return a;
}

PANDNOT AndNot_Create()
{
	PANDNOT a;

	if((a=dmake(ANDNOT_TAG, ASN1_CONTEXT, 1))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
	return a;
}

void Operator_SetType(POPERATOR o, DATA_DIR *t)
{
	if(!(o) || !(t))
		return;
	if(dadddir(o, t)==NULL)
		fprintf(stderr, "Operator_SetType:Failed\n");
}

PPROXIMITY Proximity_Create()
{
	return NULL;
}

PNONSURROGATEDIAGNOSTIC NonSurrogateDiagnostic_Create()
{
	PNONSURROGATEDIAGNOSTIC d;

	if((d=dmake(NONSURROGATEDIAGNOSTIC_TAG, ASN1_CONTEXT, 4))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
	return d;
}

void DefaultDiagFormat_SetDiagnosticSetId(PDEFAULTDIAGFORMAT d, 
	POBJECTIDENTIFIER o)
{
	if(!(d) || !(o))
		return;
	if(dadddir(d, o)==NULL)
		fprintf(stderr, "DefaultDiagFormat_SetDiagSetId:Failed\n");
}

void DefaultDiagFormat_SetCondition(PDEFAULTDIAGFORMAT d, INT4 c)
{
        if(d) {
                if(daddnum(d, ASN1_INTEGER, ASN1_UNIVERSAL, (PUCHR)&c, 
			sizeof(c))==NULL) {
			fprintf(stderr, 
				"DefaultDiagFormat_SetDiagSetId:Failed\n");
                }
        }
}

void DefaultDiagFormat_SetAddInfo(PDEFAULTDIAGFORMAT d, 
	POBJECTIDENTIFIER o)
{
	if(!(d) || !(o))
		return;
	if(dadddir(d, o)==NULL)
		fprintf(stderr, "DefaultDiagFormat_SetAddInfo:Failed\n");
}

DATA_DIR *SearchRequest_GetDatabaseNames(PSEARCHREQUEST s)
{
	if(s)
		return (DATA_DIR *)zutil_GetTaggedObject(s, 
			DATABASENAMES_TAG);
	return NULL;
}

POCTETSTRING SearchRequest_GetDatabaseName(PSEARCHREQUEST s, INT e)
{
	if(s) {
		INT4 i=1;
		DATA_DIR *d=s->ptr.child;
		while((d) && i<e) {
			d=d->next;
			i++;
		}
		if(e==i)
			return d;
	}
	return NULL;
}

PTARGETINFO ExplainRecord_CreateTargetInfo()
{
	PTARGETINFO t;

	if((t=dmake(TARGETINFO_TAG, ASN1_CONTEXT, 15))==NULL) {
                /* Enter any error logging here */
                return NULL;
        }
 
        return t;
}

