/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           z3950sup.h
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/*
These are routines that I'm generating by hand that will eventually
come from the asn.1 interpretter.
*/
#ifndef _z3950sup_
#define _z3950sup_

#include "berutil.h"
#ifdef USE_MEMH
#include <mem.h>
#endif /* USE_MEMH */
#ifdef USE_ALLOCH
#include <alloc.h>
#endif /* USE_ALLOCH */
#include "gdt.h"
#include "zutil.h"
#include "z3950.h"
#include "list.h"

#define BIB1_OID "1.2.840.10003.3.1"
#define STAS_OID "1.2.840.10003.3.1000.6.1"

#define SUTRS_OID "1.2.840.10003.5.101"
#define USMARC_OID "1.2.840.10003.5.10"
#define HTML_OID "1.2.840.10003.5.1000.34.1"

#define TARGETINFO_TAG 0
#define NONSURROGATEDIAGNOSTIC_TAG 130
#define MULTIPLENONSURDIAGNOSTICS_TAG 205
#define PRESENTREQUEST_TAG 24
#define SUTRS_TAG 27
#define DATABASENAME_TAG 105
#define DATABASENAMES_TAG 18
#define RESPONSERECORDS_TAG 28
#define RECORD_TAG 1
#define RETRIEVALRECORD_TAG 1
#define SINGLE_ASN1_TYPE_TAG 0
#define DIRECTREFERENCE_TAG 6
#define OBJECTIDENTIFIER_TAG 6
#define SIMPLE_TAG 19
#define GENERICELEMENTSETNAME_TAG 0
#define QUERY_TAG 21
#define ATTRIBUTESETID_TAG 6
#define ATTRIBUTELIST_TAG 44
#define ATTRIBUTESPLUSTERM_TAG 102
#define RESULTSETPLUSATTRIBUTES_TAG 214
#define TYPE0_TAG 0
#define TYPE1_TAG 1
#define TYPE2 TAG 2
#define TYPE100_TAG 100
#define TYPE101_TAG 101
#define TYPE102_TAG 102
#define OP_TAG 0
#define RPNRPNOP_TAG 1
#define ATTRIBUTETYPE_TAG 120
#define NUMERIC_TAG 121
#define TERMGENERAL_TAG 45
#define OPERATOR_TAG 46
#define AND_TAG 0
#define OR_TAG 1
#define ANDNOT_TAG 2
#define PROX_TAG 3
#define PREFERREDRECORDSYNTAX_TAG 104

#define DELETERESULTSETREQUEST_TAG 26
#define DELETERESULTSETRESPONSE_TAG 27
#define ACCESSCONTROLREQUEST_TAG 28
#define ACCESSCONTROLRESPONSE_TAG 29
#define RESOURCECONTROLREQUEST_TAG 30 
#define RESOURCECONTROLRESPONSE_TAG 31
#define TRIGGERRESOURCECONTROLREQUEST_TAG 32
#define RESOURCEREPORTREQUEST_TAG 33
#define RESOURCEREPORTRESPONSE_TAG 34
#define SCANREQUEST_TAG 35
#define SCANRESPONSE_TAG 36
#define SORTREQUEST_TAG 43
#define SORTRESPONSE_TAG 44
#define SEGMENTREQUEST_TAG 45
#define EXTENDEDSERVICESREQUEST_TAG 46
#define EXTENDEDSERVICESRESPONSE_TAG 47
#define CLOSE_TAG 48

typedef DATA_DIR TARGETINFO;
typedef DATA_DIR *PTARGETINFO;
typedef DATA_DIR SUTRS;
typedef DATA_DIR *PSUTRS;
typedef DATA_DIR RESPONSERECORDS;
typedef DATA_DIR *PRESPONSERECORDS;
typedef DATA_DIR NAMEPLUSRECORD;
typedef DATA_DIR *PNAMEPLUSRECORD;
typedef DATA_DIR DIAGREC;
typedef DATA_DIR *PDIAGREC;
typedef DATA_DIR DATABASENAMES;
typedef DATA_DIR *PDATABASENAMES;
typedef DATA_DIR DATABASENAME;
typedef DATA_DIR *PDATABASENAME;
typedef DATA_DIR ELEMENTSETNAMES;
typedef DATA_DIR *PELEMENTSETNAMES;
typedef DATA_DIR ATTRIBUTESPLUSTERM;
typedef DATA_DIR *PATTRIBUTESPLUSTERM;
typedef DATA_DIR RPNSTRUCTURE;
typedef DATA_DIR *PRPNSTRUCTURE;
typedef DATA_DIR OPERAND;
typedef DATA_DIR *POPERAND;
typedef DATA_DIR ATTRIBUTELIST;
typedef DATA_DIR *PATTRIBUTELIST;
typedef DATA_DIR QUERY;
typedef DATA_DIR *PQUERY;
typedef DATA_DIR TYPE1QUERY;
typedef DATA_DIR *PTYPE1QUERY;
typedef DATA_DIR RPNQUERY;
typedef DATA_DIR *PRPNQUERY;
typedef DATA_DIR ATTRIBUTESETID;
typedef DATA_DIR *PATTRIBUTESETID;
typedef DATA_DIR RPNRPNOP;
typedef DATA_DIR *PRPNRPNOP;
typedef DATA_DIR ATTRIBUTEELEMENT;
typedef DATA_DIR *PATTRIBUTEELEMENT;
typedef DATA_DIR OPERATOR;
typedef DATA_DIR *POPERATOR;
typedef DATA_DIR RECORDCOMPOSITION;
typedef DATA_DIR *PRECORDCOMPOSITION;
typedef DATA_DIR RETRIEVALRECORD;
typedef DATA_DIR *PRETRIEVALRECORD;
typedef DATA_DIR RECORD;
typedef DATA_DIR *PRECORD;
typedef DATA_DIR PROXIMITYUNITCODE;
typedef DATA_DIR *PPROXIMITYUNITCODE;
typedef DATA_DIR PROXIMITY;
typedef DATA_DIR *PPROXIMITY;
typedef DATA_DIR AND;
typedef DATA_DIR *PAND;
typedef DATA_DIR OR;
typedef DATA_DIR *POR;
typedef DATA_DIR ANDNOT;
typedef DATA_DIR *PANDNOT;
typedef DATA_DIR DEFAULTDIAGFORMAT;
typedef DATA_DIR *PDEFAULTDIAGFORMAT;
typedef DATA_DIR NONSURROGATEDIAGNOSTIC;
typedef DATA_DIR *PNONSURROGATEDIAGNOSTIC;

#define sutrs_Create() (PSUTRS)OctetString_CreateTagUniv(SUTRS_TAG)
#define sutrs_Destroy(a) OctetString_Destroy(a)
#define sutrs_SetData(a,b,c) OctetString_SetData(a,b,c)
#define sutrs_GetData(a) OctetString_GetData(a)
#define sutrs_GetLength(a) OctetString_GetLength(a)
#define sutrs_GetASCIIString(a) OctetString_GetASCIIString(a)
#define sutrs_Printf(a,b) OctetString_Printf(a,b)

#define DatabaseName_Create() (PSUTRS)OctetString_CreateTag(DATABASENAME_TAG)
#define DatabaseName_Destroy(a) OctetString_Destroy(a)
#define DatabaseName_SetData(a,b,c) OctetString_SetData(a,b,c)
#define DatabaseName_GetData(a) OctetString_GetData(a)
#define DatabaseName_GetLength(a) OctetString_GetLength(a)
#define DatabaseName_GetASCIIString(a) OctetString_GetASCIIString(a)
#define DatabaseName_Printf(a,b) OctetString_Printf(a,b)

#define ObjectIdentifier_Create() (POBJECTIDENTIFIER)OctetString_CreateTagUniv(OBJECTIDENTIFIER_TAG)
#define ObjectIdentifier_Destroy(a) OctetString_Destroy(a)
#define ObjectIdentifier_SetData(a,b,c) OctetString_SetData(a,b,c)
/* see z3950sup.c for GetData() and GetLength() */
#define ObjectIdentifier_GetASCIIString(a) OctetString_GetASCIIString(a)
#define ObjectIdentifier_Printf(a,b) OctetString_Printf(a,b)

#define PreferredRecordSyntax_Create() (PPREFERREDRECORDSYNTAX)OctetString_CreateTag(PREFERREDRECORDSYNTAX_TAG)
#define DatabaseName_Destroy(a) OctetString_Destroy(a)
#define PreferredRecordSyntax_SetData(a,b,c) OctetString_SetData(a,b,c)
#define PreferredRecordSyntax_GetData(a) ObjectIdentifier_GetData(a)
#define PreferredRecordSyntax_GetLength(a) ObjectIdentifier_GetLength(a)

#define DirectReference_SetData(a,b,c) OctetString_SetData(a,b,c)
#define DirectReference_GetData(a) ObjectIdentifier_GetData(a)
#define DirectReference_GetLength(a) ObjectIdentifier_GetLength(a)

#define AttributeSetId_Create() (PATTRIBUTESETID)OctetString_CreateTagUniv(OBJECTIDENTIFIER_TAG)
#define DatabaseName_Destroy(a) OctetString_Destroy(a)
#define AttributeSetId_SetData(a,b,c) OctetString_SetData(a,b,c)
#define AttributeSetId_GetData(a) ObjectIdentifier_GetData(a)
#define AttributeSetId_GetLength(a) ObjectIdentifier_GetLength(a)

PINITREQUEST InitRequest_CreateInitAllASCII(PCHR ReferenceId,
        PCHR ProtocolVersion, PCHR Options, INT4 PreferredMessageSize,
        INT4 ExceptionalRecordSize, PCHR ImplementationId,
        PCHR ImplementationName, PCHR ImplementationVersion,
        PCHR UserInformation);
PINITRESPONSE InitResponse_CreateInitAllASCII(PCHR ReferenceId,
        PCHR ProtocolVersion, PCHR Options, INT4 PreferredMessageSize,
        INT4 ExceptionalRecordSize, BOOLEAN Result, PCHR ImplementationId,
        PCHR ImplementationName, PCHR ImplementationVersion,
        PCHR UserInformation);
PSEARCHREQUEST SearchRequest_CreateInitAllASCII(PCHR ReferenceId,
        INT4 SmallSetUpperBound, INT4 LargeSetLowerBound,
        INT4 MediumSetPresentNumber, BOOLEAN ReplaceIndicator,
        PCHR ResultSetName, PCHR DatabaseNames,
        PCHR SmallSetElementSetNames, PCHR MediumSetElementSetNames,
        PCHR PreferredRecordSyntax, PCHR Query, PCHR AttributeSetId);
PSEARCHRESPONSE SearchResponse_CreateInitAllASCII(PCHR ReferenceId,
        INT4 ResultCount, INT4 NumRecordsReturned, INT4 NextResultSetPos,
        BOOLEAN SearchStatus, INT4 ResultSetStatus, INT4 PresentStatus,
        DATA_DIR *Records,  DATA_DIR *AdditionalSearchInfo,
        DATA_DIR *OtherInfo);
PPRESENTREQUEST PresentRequest_CreateInitAllASCII(PCHR ReferenceId,
        PCHR ResultSetId, INT4 ResultSetStartPoint,
        INT4 NumberOfRecordsRequested, PCHR ElementSetName,
        PCHR PreferredRecordSyntax);
PPRESENTRESPONSE PresentResponse_CreateInitAllASCII(PCHR ReferenceId,
        INT4 NumRecordsReturned, INT4 NextResultSetPos,
        INT4 PresentStatus, PRESPONSERECORDS Records, DATA_DIR *OtherInfo);
 
PBITSTRING BitString_CreateTag(INT4 Tag);
void BitString_Destroy(PBITSTRING p);
void BitString_SetBits(PBITSTRING p, PUCHR bits);
CHR BitString_GetBit(PBITSTRING p, INT4 BitNum);
void OctetString_CopyToCString(POCTETSTRING o, PCHR s);
INT ProtocolVersion_GetVersion(PPROTOCOLVERSION p);

/* Records */
PRESPONSERECORDS ResponseRecords_Create(void);
void ResponseRecords_Destroy(PRESPONSERECORDS r);
void ResponseRecords_Add(PRESPONSERECORDS r, PNAMEPLUSRECORD n);
void ResponseRecords_Delete(PRESPONSERECORDS r, INT4 c);
INT4 ResponseRecords_GetCount(PRESPONSERECORDS r);
PDATABASENAME ResponseRecords_GetName(PRESPONSERECORDS r, INT4 c);
PRECORD ResponseRecords_GetRecord(PRESPONSERECORDS r, INT4 c);
void ResponseRecords_AddSUTRS(PRESPONSERECORDS r, PCHR db, PCHR d, INT4 l);

PNAMEPLUSRECORD NamePlusRecord_Create(void);
void NamePlusRecord_Destroy(PNAMEPLUSRECORD n);
void NamePlusRecord_SetName(PNAMEPLUSRECORD n, PDATABASENAME d);
PDATABASENAME NamePlusRecord_GetName(PNAMEPLUSRECORD n);
INT4 NamePlusRecord_GetRecordType(PNAMEPLUSRECORD n);
PRETRIEVALRECORD NamePlusRecord_GetRetrievalRecord(PNAMEPLUSRECORD n);
PDIAGREC NamePlusRecord_GetSurrogateDiagnostic(PNAMEPLUSRECORD n);
void NamePlusRecord_SetRetrievalRecord(PNAMEPLUSRECORD n, PEXTERNAL r);
void NamePlusRecord_SetSurrogateDiagnostic(PNAMEPLUSRECORD n, PDIAGREC r);

PEXTERNAL External_Create(void);
void External_Destroy(PEXTERNAL e);
void External_SetDirectReference(PEXTERNAL e, POBJECTIDENTIFIER o);
void External_SetEncodingSingle(PEXTERNAL e, PANY a);
POBJECTIDENTIFIER External_GetDirectReference(PEXTERNAL e);
PANY External_GetEncodingSingle(PEXTERNAL e);
PANY External_GetEncodingAligned(PEXTERNAL e);
INT4 External_GetEncodingType(PEXTERNAL e);

POCTETSTRING OctetString_CreateTagUniv(INT4 Tag);
void PresentResponse_SetRecords(PPRESENTRESPONSE p, PRESPONSERECORDS r);
void SearchResponse_SetRecords(PSEARCHRESPONSE p, DATA_DIR *r);
INT4 PresentRequest_GetRecordCompositionType(PPRESENTREQUEST p);
PELEMENTSETNAMES PresentRequest_GetSimple(PPRESENTREQUEST p);
POCTETSTRING ElementSetNames_GetGeneric(PELEMENTSETNAMES e);
DATA_DIR *PresentRequest_GetComplex(PPRESENTREQUEST p);
STACK *RpnQuery_GetStack(DATA_DIR *rpn, STACK *Stack);
PQUERY SearchRequest_GetQuery(PSEARCHREQUEST p);
INT4 Query_GetQueryType(PQUERY p);
PQUERY Query_CreateKWAQS(PCHR AttrSet, PCHR S);
PRPNQUERY Query_GetQueryType1(PQUERY p);
PATTRIBUTESETID RpnQuery_GetAttributeSetId(PRPNQUERY r);
DATA_DIR *RpnStructure_CreateKWAQS(PCHR S);
INT4 RpnQuery_GetRpnType(PRPNQUERY r);
POPERAND RpnQuery_GetRpnOp(PRPNQUERY r);
PRPNRPNOP RpnQuery_GetRpnRpnRpnOp(PRPNQUERY r);
INT4 Operand_GetType(POPERAND o);
PATTRIBUTESPLUSTERM Operand_GetAttributesPlusTerm(POPERAND o);
PATTRIBUTELIST AttributesPlusTerm_GetAttributeList(PATTRIBUTESPLUSTERM a);
POCTETSTRING AttributesPlusTerm_GetTermGeneral(PATTRIBUTESPLUSTERM a);
INT4 AttributeList_GetElementCount(PATTRIBUTELIST a);
PATTRIBUTEELEMENT AttributeList_GetElement(PATTRIBUTELIST a, INT4 e);
INT4 AttributeElement_GetType(PATTRIBUTEELEMENT a);
INT4 AttributeElement_GetValueNumeric(PATTRIBUTEELEMENT a);
POPERATOR RpnRpnOp_GetOperator(PRPNRPNOP r);
PRPNRPNOP RpnRpnOp_Create();
INT4 Operator_GetType(POPERATOR o);
PNAMEPLUSRECORD NamePlusRecord_CreateSUTRS(PCHR db, PSUTRS Sutrs);
PNAMEPLUSRECORD NamePlusRecord_CreateHTML(PCHR db, DATA_DIR *html);
PCHR ObjectIdentifier_GetData(POBJECTIDENTIFIER o);
INT4 ObjectIdentifier_GetLength(POBJECTIDENTIFIER o);

PDATABASENAMES DatabaseNames_Create();
void DatabaseNames_Add(PDATABASENAMES n, PDATABASENAME d);
POPERAND Operand_Create();
void Operand_SetType(POPERAND p, DATA_DIR *t);
void AttributeList_Add(PATTRIBUTELIST a, PATTRIBUTEELEMENT e);
void AttributesPlusTerm_SetTermGeneral(PATTRIBUTESPLUSTERM a, POCTETSTRING t);
PATTRIBUTELIST AttributeList_Create();
PATTRIBUTELIST AttributeList_CreateKWAQS(PCHR S);
void AttributeList_SetType(PATTRIBUTELIST a, INT4 t);
void AttributeList_SetValueNumeric(PATTRIBUTELIST a, INT4 t);
PATTRIBUTEELEMENT AttributeElement_Create();
void AttributeElement_SetValueNumeric(PATTRIBUTEELEMENT a, INT4 t);
void AttributeElement_SetType(PATTRIBUTEELEMENT a, INT4 t);
PQUERY Query_CreateInitAllASCII(PCHR AttrSet, PCHR s);
PQUERY Query_Create(void);
PTYPE1QUERY Type1Query_Create();
void Type1Query_SetAttributeSetId(PTYPE1QUERY q, PATTRIBUTESETID a);
void Type1Query_SetRpnStructure(PTYPE1QUERY q, PRPNSTRUCTURE r);
PRPNQUERY RpnQuery_Create();
PATTRIBUTESPLUSTERM AttributesPlusTerm_Create();
void SearchRequest_SetDatabaseNames(PSEARCHREQUEST s, PDATABASENAMES d);
void SearchRequest_SetQuery(PSEARCHREQUEST s, PQUERY q);
void Query_SetType(PQUERY q, DATA_DIR *d);
void RpnQuery_SetAttributeSetId(PRPNQUERY q,POBJECTIDENTIFIER o);
void RpnQuery_SetOperand(PRPNQUERY r, PQUERY q);
PRECORDCOMPOSITION RecordComposition_CreateSimple();
void PresentRequest_SetResultSetName(PPRESENTREQUEST p, PRESULTSETNAME r);
PRESPONSERECORDS PresentResponse_GetResponseRecords(PPRESENTREQUEST p);
INT PresentResponse_GetRecordType(PPRESENTREQUEST p);
PNONSURROGATEDIAGNOSTIC PresentResponse_GetNonSurrogateDiagnostic(PPRESENTREQUEST p);
DATA_DIR *NamePlusRecord_GetRecord(PNAMEPLUSRECORD n);
PEXTERNAL RetrievalRecord_GetExternal(PRETRIEVALRECORD r);
void RecordComposition_SetSimpleGeneric(PRECORDCOMPOSITION r,
	PINTERNATIONALSTRING i);
void PresentRequest_SetRecordComposition(PPRESENTREQUEST p, 
	PRECORDCOMPOSITION e);
void AttributesPlusTerm_SetAttributeList(PATTRIBUTESPLUSTERM a, 
	PATTRIBUTELIST l);
void RpnRpnOp_SetRpn1(PRPNRPNOP r, DATA_DIR *o);
void RpnRpnOp_SetRpn2(PRPNRPNOP r, DATA_DIR *o);
void RpnRpnOp_SetOp(PRPNRPNOP r, POPERATOR o);
PAND And_Create();
POR Or_Create();
PANDNOT AndNot_Create();
PPROXIMITY Proximity_Create();
void Proximity_SetExclusion(PPROXIMITY p, INT4 t);
void Proximity_SetDistance(PPROXIMITY p, INT4 t);
void Proximity_SetOrdered(PPROXIMITY p, INT4 t);
void Proximity_SetRelationType(PPROXIMITY p, INT4 t);
PPROXIMITYUNITCODE ProximityUnitCode_Create();
void ProximityUnitCode_SetKnown(PPROXIMITYUNITCODE p, INT4 t);
void ProximityUnitCode_SetPrivate(PPROXIMITYUNITCODE p, INT4 t);
POPERATOR Operator_Create();
void Operator_SetType(POPERATOR o, DATA_DIR *t);
PDEFAULTDIAGFORMAT DefaultDiagFormat_Create();
void DefaultDiagFormat_SetDiagnosticSetId(PDEFAULTDIAGFORMAT d,
        POBJECTIDENTIFIER o);
void DefaultDiagFormat_SetAddInfo(PDEFAULTDIAGFORMAT d,
        POBJECTIDENTIFIER o);
void DefaultDiagFormat_SetCondition(PDEFAULTDIAGFORMAT d, INT4 c);
PNONSURROGATEDIAGNOSTIC NonSurrogateDiagnostic_Create();
POCTETSTRING SearchRequest_GetDatabaseName(PSEARCHREQUEST s, INT e);
DATA_DIR *SearchRequest_GetDatabaseNames(PSEARCHREQUEST s);

#endif /* _z3950sup_ */

