/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zstats.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include "libz3950.h"
#include "zstats.h"

PZSTATS zstats_Create()
{
	PZSTATS z;
	
	if((z=NEWSTRUCT(ZSTATS))==NULL)
		return NULL;

        z->BytesSent=0;
        z->BytesRec=0;
        z->SessionLength=0;
        z->InitRequestCount=0;
        z->InitResponseCount=0;
        z->SearchRequestCount=0;
        z->SearchResponseCount=0;
        z->PresentRequestCount=0;
        z->PresentResponseCount=0;
        z->DeleteResultSetRequestCount=0;
        z->DeleteResultSetResponseCount=0;
        z->AccessControlRequestCount=0;
        z->AccessControlResponseCount=0;
        z->ResourceControlRequestCount=0;
        z->ResourceControlResponseCount=0;
        z->TriggerResourceControlCount=0;
        z->ResourceReportRequestCount=0;
        z->ResourceReportResponseCount=0;
        z->ScanRequestCount=0;
        z->ScanResponseCount=0;
        z->SortRequestCount=0;
        z->SortResponseCount=0;
        z->SegmentRequest=0;
        z->ExtendedServicesRequestCount=0;
        z->ExtendedServicesResponseCount=0;
        z->CloseRequestCount=0;

	return z;
}

PZSTATS zstats_CreateInit(PCHR Host)
{
	PZSTATS z;

	if((z=zstats_Create())==NULL)
		return NULL;

	zstats_SetHost(z, Host);
	zstats_SetDateTime(z);

	return z;
}

void zstats_Destroy(PZSTATS s)
{
	if(s)
		FREE(s);
}

void zstats_SetHost(PZSTATS s, PCHR Host)
{
	if(!s)
		return;
	
	strncpy(s->Host, Host, sizeof(s->Host));
}

void zstats_SetDateTime(PZSTATS s)
{
	if(!s)
		return;
	
	time(&s->Time);
}

void zstats_AddPDU(PZSTATS s, DATA_DIR *d)
{
	INT flow;	/* if 0, bytes received, 1 bytes sent */
	if(!s)
		return;
 
	switch(d->fldid) {
		case INITREQUEST_TAG:
                        s->InitRequestCount++;
			flow=0;
			break;
		case INITRESPONSE_TAG:
                        s->InitResponseCount++;
			flow=1;
			break;
		case SEARCHREQUEST_TAG:
                        s->SearchRequestCount++;
			flow=0;
			break;
		case SEARCHRESPONSE_TAG:
                        s->SearchResponseCount++;
			flow=1;
			break;
		case PRESENTREQUEST_TAG:
                        s->PresentRequestCount++;
			flow=0;
			break;
		case PRESENTRESPONSE_TAG:
                        s->PresentResponseCount++;
			flow=1;
			break;
		case DELETERESULTSETREQUEST_TAG:
                        s->DeleteResultSetRequestCount++;
			flow=0;
			break;
		case DELETERESULTSETRESPONSE_TAG:
                        s->DeleteResultSetResponseCount++;
			flow=1;
			break;
		case ACCESSCONTROLREQUEST_TAG:
                        s->AccessControlRequestCount++;
			flow=0;
			break;
		case ACCESSCONTROLRESPONSE_TAG:
                        s->AccessControlResponseCount++;
			flow=1;
			break;
		case RESOURCECONTROLREQUEST_TAG:
                        s->ResourceControlRequestCount++;
			flow=0;
			break;
		case RESOURCECONTROLRESPONSE_TAG:
                        s->ResourceControlResponseCount++;
			flow=1;
			break;
		case TRIGGERRESOURCECONTROLREQUEST_TAG:
                        s->TriggerResourceControlCount++;
			flow=0;
			break;
		case RESOURCEREPORTREQUEST_TAG:
                        s->ResourceReportRequestCount++;
			flow=0;
			break;
		case RESOURCEREPORTRESPONSE_TAG:
                        s->ResourceReportResponseCount++;
			flow=1;
			break;
		case SCANREQUEST_TAG:
                        s->ScanRequestCount++;
			flow=0;
			break;
		case SCANRESPONSE_TAG:
                        s->ScanResponseCount++;
			flow=1;
			break;
		case SORTREQUEST_TAG:
                        s->SortRequestCount++;
			flow=0;
			break;
		case SORTRESPONSE_TAG:
                        s->SortResponseCount++;
			flow=1;
			break;
		case SEGMENTREQUEST_TAG:
			s->SegmentRequest++;
			flow=0;
			break;
		case EXTENDEDSERVICESREQUEST_TAG:
                        s->ExtendedServicesRequestCount++;
			flow=0;
			break;
		case EXTENDEDSERVICESRESPONSE_TAG:
		        s->ExtendedServicesResponseCount++;
			flow=1;
			break;
		case CLOSE_TAG:
                        s->CloseRequestCount++;
			flow=0;
			break;
		default:
			break;
	}
	if(flow==0)
		s->BytesRec = rec_len(d);
	else
		s->BytesSent = rec_len(d);

}

void zstats_WriteLog(PZSTATS s, PCHR f)
{
	int fd;
	FILE *ffp;	

	if((ffp=fopen(f,"ab"))==NULL) {
		perror("zstats_WriteLog:fopen");
		return;
	}

	fd = fileno(ffp);
/*
	if(lockf(fd, F_LOCK, 0)!=0) {
		perror("zstats_WriteLog:locking");
		fclose(ffp);
		return;
	}
*/
	fwrite(s->Host, sizeof(s->Host), 1, ffp);
	fwrite(&s->Time, sizeof(time_t), 1, ffp);
	fwrite(&s->SessionLength, sizeof(INT4), 1, ffp);
	fwrite(&s->BytesRec, sizeof(INT4), 1, ffp);
	fwrite(&s->BytesSent, sizeof(INT4), 1, ffp);
	fwrite(&s->InitRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->InitResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->SearchRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->SearchResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->PresentRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->PresentResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->DeleteResultSetRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->DeleteResultSetResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->AccessControlRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->AccessControlResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ResourceControlRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ResourceControlResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->TriggerResourceControlCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ResourceReportRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ResourceReportResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ScanRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ScanResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->SortRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->SortResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->SegmentRequest, sizeof(INT4), 1, ffp);
	fwrite(&s->ExtendedServicesRequestCount, sizeof(INT4), 1, ffp);
	fwrite(&s->ExtendedServicesResponseCount, sizeof(INT4), 1, ffp);
	fwrite(&s->CloseRequestCount, sizeof(INT4), 1, ffp);
/*
	if(lockf(fd, F_ULOCK, 0)!=0) {
		perror("zstats_WriteLog:unlocking");
		fclose(ffp);
		return;
	}
*/
	fclose(ffp);
}

void zstats_Display(PZSTATS s, FILE *fp)
{
	fprintf(fp, "%d\n", s->BytesSent);
	fprintf(fp, "%d\n", s->BytesRec);
	if(s->Host[0]!='\0')
        	fprintf(fp, "%s\n", s->Host);
        fprintf(fp, "%s", (PCHR)ctime(&(s->Time)));
	fprintf(fp, "%d\n", s->SessionLength);
	fprintf(fp, "%d\n", s->InitRequestCount);
	fprintf(fp, "%d\n", s->InitResponseCount);
	fprintf(fp, "%d\n", s->SearchRequestCount);
	fprintf(fp, "%d\n", s->SearchResponseCount);
	fprintf(fp, "%d\n", s->PresentRequestCount);
	fprintf(fp, "%d\n", s->PresentResponseCount);
	fprintf(fp, "%d\n", s->DeleteResultSetRequestCount);
	fprintf(fp, "%d\n", s->DeleteResultSetResponseCount);
	fprintf(fp, "%d\n", s->AccessControlRequestCount);
	fprintf(fp, "%d\n", s->AccessControlResponseCount);
	fprintf(fp, "%d\n", s->ResourceControlRequestCount);
	fprintf(fp, "%d\n", s->ResourceControlResponseCount);
	fprintf(fp, "%d\n", s->TriggerResourceControlCount);
	fprintf(fp, "%d\n", s->ResourceReportRequestCount);
	fprintf(fp, "%d\n", s->ResourceReportResponseCount);
	fprintf(fp, "%d\n", s->ScanRequestCount);
	fprintf(fp, "%d\n", s->ScanResponseCount);
	fprintf(fp, "%d\n", s->SortRequestCount);
	fprintf(fp, "%d\n", s->SortResponseCount);
	fprintf(fp, "%d\n", s->SegmentRequest);
	fprintf(fp, "%d\n", s->ExtendedServicesRequestCount);
	fprintf(fp, "%d\n", s->ExtendedServicesResponseCount);
	fprintf(fp, "%d\n", s->CloseRequestCount);
	
}

PLIST zstats_ReadLog(PCHR f)
{
	FILE *fp;
	PLIST list;
	PZSTATS s;

	list = list_Create();
	if(!list) {
		fprintf(stderr, "zstats_ReadLog:out of memory\n");
		return NULL;
	}

	if(!(s=zstats_Create())) {
		fprintf(stderr, "zstats_ReadLog:out of memory\n");
		return NULL;
	}

	if((fp=fopen(f, "rb"))==NULL) {
		perror("stats_ReadLog");
		return (PLIST)NULL;
	}

	for(;;) {
		PZSTATS t;

		if(fread(s->Host, sizeof(s->Host), 1, fp)<=0)
			break;
		fread(&s->Time, sizeof(time_t), 1, fp);
		fread(&s->SessionLength, sizeof(INT4), 1, fp);
		fread(&s->BytesRec, sizeof(INT4), 1, fp);
		fread(&s->BytesSent, sizeof(INT4), 1, fp);
		fread(&s->InitRequestCount, sizeof(INT4), 1, fp);
		fread(&s->InitResponseCount, sizeof(INT4), 1, fp);
		fread(&s->SearchRequestCount, sizeof(INT4), 1, fp);
		fread(&s->SearchResponseCount, sizeof(INT4), 1, fp);
		fread(&s->PresentRequestCount, sizeof(INT4), 1, fp);
		fread(&s->PresentResponseCount, sizeof(INT4), 1, fp);
		fread(&s->DeleteResultSetRequestCount, sizeof(INT4), 1, fp);
		fread(&s->DeleteResultSetResponseCount, sizeof(INT4), 1, fp);
		fread(&s->AccessControlRequestCount, sizeof(INT4), 1, fp);
		fread(&s->AccessControlResponseCount, sizeof(INT4), 1, fp);
		fread(&s->ResourceControlRequestCount, sizeof(INT4), 1, fp);
		fread(&s->ResourceControlResponseCount, sizeof(INT4), 1, fp);
		fread(&s->TriggerResourceControlCount, sizeof(INT4), 1, fp);
		fread(&s->ResourceReportRequestCount, sizeof(INT4), 1, fp);
		fread(&s->ResourceReportResponseCount, sizeof(INT4), 1, fp);
		fread(&s->ScanRequestCount, sizeof(INT4), 1, fp);
		fread(&s->ScanResponseCount, sizeof(INT4), 1, fp);
		fread(&s->SortRequestCount, sizeof(INT4), 1, fp);
		fread(&s->SortResponseCount, sizeof(INT4), 1, fp);
		fread(&s->SegmentRequest, sizeof(INT4), 1, fp);
		fread(&s->ExtendedServicesRequestCount, sizeof(INT4), 1, fp);
		fread(&s->ExtendedServicesResponseCount, sizeof(INT4), 1, fp);
		fread(&s->CloseRequestCount, sizeof(INT4), 1, fp);

		
		list_InsertAfter(list, list_Last(list), s, LIST_NEAR);
		if(!(t=zstats_Create())) {
			fprintf(stderr, "zstats_ReadLog:out of memory\n");
			return NULL;
		}
		s = t;
	}
	fclose(fp);	
	return list;
}

/*
Adds the integer values from src to those in dest.
Host member is ignored.
*/
void zstats_Accumulate(PZSTATS dest, PZSTATS src)
{
	if(!(dest) || !(src))
		return;

	dest->BytesRec+=src->BytesRec;
	dest->BytesSent+=src->BytesSent;
	dest->Time=src->Time;
	dest->InitRequestCount+=src->InitRequestCount;
	dest->InitResponseCount+=src->InitResponseCount;
	dest->SearchRequestCount+=src->SearchRequestCount;
	dest->SearchResponseCount+=src->SearchResponseCount;
	dest->PresentRequestCount+=src->PresentRequestCount;
	dest->PresentResponseCount+=src->PresentResponseCount;
	dest->DeleteResultSetRequestCount+=src->DeleteResultSetRequestCount;
	dest->DeleteResultSetResponseCount+=src->DeleteResultSetResponseCount;
	dest->AccessControlRequestCount+=src->AccessControlRequestCount;
	dest->AccessControlResponseCount+=src->AccessControlResponseCount;
	dest->ResourceControlRequestCount+=src->ResourceControlRequestCount;
	dest->ResourceControlResponseCount+=src->ResourceControlResponseCount;
	dest->TriggerResourceControlCount+=src->TriggerResourceControlCount;
	dest->ResourceReportRequestCount+=src->ResourceReportRequestCount;
	dest->ResourceReportResponseCount+=src->ResourceReportResponseCount;
	dest->ScanRequestCount+=src->ScanRequestCount;
	dest->ScanResponseCount+=src->ScanResponseCount;
	dest->SortRequestCount+=src->SortRequestCount;
	dest->SortResponseCount+=src->SortResponseCount;
	dest->SegmentRequest+=src->SegmentRequest;
	dest->ExtendedServicesRequestCount+=src->ExtendedServicesRequestCount;
	dest->ExtendedServicesResponseCount+=src->ExtendedServicesResponseCount;
	dest->CloseRequestCount+=src->CloseRequestCount;

}

