/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zutil.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#include <stdio.h>
#include <malloc.h>
#include "netbox.h"
#include "gdt.h"
#include "zutil.h"
#include "strbox.h"
#include "z3950sup.h"

DATA_DIR * far zutil_CreateFromData(PUCHR Buffer, INT4 Length)
{
        DATA_DIR * d;
 
        if((d = dalloc (15)) == NULL)
                return(NULL);
 
        if (!bld_dir (Buffer, d)) {
                return NULL;
        }
 
        return d;
}


INT4 zutil_GetBERFromNet(NETBOXPROFILE *profile, PUCHR b, INT4 maxbuffersize) {
	INT4 len, remainder, tlen;

	if (netbox_WaitForData (profile) == 0)
		return - 1;
	len = netbox_GetBuffer (profile, (PCHR)b, 6);
	if(!len)
		return(-1);
	while (!IsCompleteBER (b, len, &remainder)) {
		if (netbox_WaitForData (profile) == 0)
			return - 1;
		tlen=netbox_GetBuffer(profile,(PCHR)b+len,remainder ?
			remainder:1);
		if (!tlen)
			return (-1);
		len += tlen;
	} /* while */

	return (len);
}

void zutil_Log(DATA_DIR * O, PCHR FileName)
{
	FILE *fp;
	if((fp=fopen(FileName,"a"))==NULL) {
		fprintf(stderr, "zutil_Log:Failed to open file %s\n",FileName);
		return;
	}
	hex_dirf(O,0,fp);
	fclose(fp);
}

void zutil_LogToStream(DATA_DIR * O, FILE *S)
{
	hex_dirf(O,0,S);
}

INT4 zutil_GetTaggedInt(DATA_DIR * D, INT4 Tag)
{
	DATA_DIR * t;

	if((t=zutil_GetTaggedObject(D, Tag))!=NULL) {
		return dgetnum(t);
	}

	return -1;
}

unsigned char zutil_GetTaggedUnsignedChar(DATA_DIR * D, INT4 Tag)
{
	DATA_DIR * t;

	if((t=zutil_GetTaggedObject(D, Tag))!=NULL) {
		return (unsigned char)(*(unsigned char *)t->ptr.data);
	}

	return 0;
}

PUCHR zutil_GetTaggedOctetString(DATA_DIR * D, INT4 Tag, PUCHR Buf, INT4 Size)
{
	DATA_DIR * t;

	Buf[0]='\0';
	if((t=zutil_GetTaggedObject(D, Tag))!=NULL) {
		COPY(Buf,t->ptr.data, Size<t->count?Size:t->count);
		Buf[t->count]='\0';
		return Buf;
	}

	return NULL;
}

/*
Scans through children (but not grandchildren) of D looking for Tag.
Returns pointer to the child or NULL.
*/
DATA_DIR * zutil_GetTaggedObject(DATA_DIR *D, INT4 Tag)
{
	DATA_DIR * sub;

	if((D==NULL) || (D->ptr.child==NULL))
		return NULL;

	for(sub=D->ptr.child; sub; sub=sub->next)
		if(sub->fldid==Tag)
			return sub;

	return NULL;
}

PUCHR far zutil_GetBEREncodedBuffer(DATA_DIR * D, PUCHR Buffer, PINT4 Length,
	INT4 MaxRecordSize)
{
        INT4 TempLen;
        PUCHR TempRec;

	if((!D) || (!Buffer) || (MaxRecordSize <= 0)) {
		fprintf(stderr, "zutil_GetBEREncodedBuffer:Bad input\n");
		return NULL;
	}

        if((TempRec=bld_rec(D,&TempLen))==NULL) {
                fprintf(stderr, "zutil_GetBEREncodedBuffer:bld_rec failed\n");
                return NULL;
        }
        if(TempLen > MaxRecordSize) {
		fprintf(stderr, "MaxBufSize=%d, ActualRecSize=%d\n", 
			MaxRecordSize, TempLen);
                fprintf(stderr, "zutils_GetBEREncodedBuffer:bad length\n");
                FREE(TempRec);
                return NULL;
        }
 
        COPY(Buffer,TempRec,TempLen);
        *Length=TempLen;
        FREE(TempRec);
 
        return Buffer;
}

void zutil_PrintOctetString(POCTETSTRING p)
{
	INT4 i;

	for(i=0;i<p->count;i++)
		printf("%c",p->ptr.data[i]);
	printf("\n");
}

INT4 zutil_GetTag(DATA_DIR * D)
{
	return D->fldid;
}

void zutil_LogPDU(DATA_DIR *d, FILE *fp)
{
	CHR t[80];

	if(d && fp) {
		strbox_TheTime(t);
		fprintf(fp, "%s\n", t);
		hex_dirf(d, 1, fp);
	}
}

void zutil_AddToFile(PCHR File, PCHR s)
{
	FILE *fp;

	if((fp=fopen(File, "a"))) {
		fprintf(fp, "%s", s);
		fclose(fp);
	}
}

/*
strips anything within brackets (including the brackets)

eg. 'dog[1,21,2,3]' --> 'dog'
*/
void zutil_StripAttributes(PCHR S)
{
	INT i;
	CHR Left=0, Right=0;
	PCHR p;

	if(!S)
		return;

#ifdef DEBUG_PARSE
	printf("zutil_StripAttributes:Starting with: '%s'\n", S);
#endif
        /* Is there a pair of bounding brackets? */
        for(i=0;i<strlen(S);i++) {
                if(S[i]=='[')
                        Left++;
                if(S[i]==']')
                        Right++;
        }
        if(Left != 1 || Right !=1)
                return;

	p = strchr(S, '[');
	*p = '\0';
#ifdef DEBUG_PARSE
	printf("zutil_StripAttributes:Ending with: '%s'\n", S);
#endif

}

/*
1 on success, 0 on failure
*/
INT zutil_GetArgs(PCHR S, PCHR Arg1, PCHR Arg2)
{
	PCHR Term, p;
	INT RightP=0, LeftP=0, RightB=0, LeftB=0, Comma=0, i;
	INT PLevel=0, BLevel=0;
	INT4 Len;
	CHR Cont=1;

	if(!(S) || !(Arg1) || !(Arg2)) {
		fprintf(stderr, "zutil_GetArgs:NULL strings passed\n");
		return 0;
	}

#ifdef DEBUG_PARSE
	printf("zutil_GetArgs:Starting with: '%s'\n", S);
#endif
 	Len = strlen(S);
	if(Len<=0) {
		fprintf(stderr, "zutil_GetArgs:Zero length string\n");
		return 0;
	}

	if((Term=NEWSTRING(Len+1))==NULL) {
		fprintf(stderr, "zutil_GetArgs:Out of memory\n");
		return 0;
	}

	/* Is it a supported boolean operation? */
	if(zutil_IsBooleanOp(S)==-1) {
		fprintf(stderr,"zutil_GetArgs:Unsupported boolean operation\n");
		return 0;
	}

	/* Is the number of parens and brackets even? */
	for(i=0;i<Len;i++) {
		if(S[i]=='[')
			LeftB++;
		if(S[i]==']')
			RightB++;
		if(S[i]=='(')
			LeftP++;
		if(S[i]==')')
			RightP++;
		if(S[i]==',')
			Comma++;
	}
	if(LeftP!=RightP) {
		fprintf(stderr,"zutil_GetArgs:Syntax error in query.\n");
		fprintf(stderr,"-->Expected even paren count.\n");
		fprintf(stderr,"-->Parens () reserved for boolean terms .\n");
		return 0;
	}
	if(RightB!=LeftB) {
		fprintf(stderr,"zutil_GetArgs:Syntax error in query.\n");
		fprintf(stderr,"-->Expected even bracket count.\n");
		fprintf(stderr,"-->Brackets [] reserved for attributes.\n");
		return 0;
	}

	/* Strip off the boolean operator */
	if(S[Len-1]!=')') {
		fprintf(stderr,"zutil_GetArgs:Syntax error in query.\n");
		fprintf(stderr,"-->Expected paren as last character.\n");
		return 0;
	}
	/* kill the operator and leading paren */
	p = strchr(S, '(');
	strncpy(Term, p+1, Len);
	/* kill the last paren */
	Term[strlen(Term)-1]='\0';
	Len = strlen(Term);	
#ifdef DEBUG_PARSE
	printf("zutil_GetArgs:After stripping operator: '%s'\n", Term);
#endif
	/* Find the comma separating the 2 terms */
	for(i=0;i<Len && Cont;i++) {
		if((i==0) && 
			(Term[i]=='(' ||
			Term[i]==')' ||
			Term[i]=='[' ||
			Term[i]==']' ||
			Term[i]==',')) {
			fprintf(stderr, "zutil_GetArg:Syntax Error\n");
			fprintf(stderr, "-->Reserved character cannot be used in a search term: '%c'\n",Term[i]);
			return 0;
		} /* if */
		if(Term[i]==',') {
			if(PLevel==0 && BLevel==0) {
				/* have term 1*/
				strncpy(Arg1, Term, i);
				Arg1[i]='\0';
#ifdef DEBUG_PARSE
	printf("zutil_GetArgs:I think arg 1 is: '%s'\n", Arg1);
#endif
				strncpy(Arg2, Term+i+1, Len-i+1);
				Arg2[Len-i+1]='\0';
#ifdef DEBUG_PARSE
	printf("zutil_GetArgs:I think arg 2 is: '%s'\n", Arg2);
#endif
				Cont=0;
			}
		}
		if(Term[i] == '(') {
			PLevel++;
		}
		if(Term[i] == ')') {
			if(PLevel==0) {
				fprintf(stderr, "zutil_GetArg:Syntax Error\n");
				fprintf(stderr, "-->Unexpected ')'\n");
				return 0;
			}
			PLevel--;
		}	
		if(Term[i] == '[') {
			BLevel++;
		}
		if(Term[i] == ']') {
			if(BLevel==0) {
				fprintf(stderr, "zutil_GetArg:Syntax Error\n");
				fprintf(stderr, "-->Unexpected ']'\n");
				return 0;
			}
			BLevel--;
		}	
	}
	return 1;
}

INT zutil_IsBooleanOp(PCHR S)
{
        if(!S)
                return 0;
 
        if(!strncasecmp(S, "AND(", 4))
                return AND_TAG;
        if(!strncasecmp(S, "OR(", 3))
                return OR_TAG;
        if(!strncasecmp(S, "ANDNOT(", 7))
                return ANDNOT_TAG;
        if(!strncasecmp(S, "PROX(", 5))
                return PROX_TAG;

	return -1;
}

