/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zstatsview.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "libz3950.h"
#include "zstatsview.h"

INT2 DisplayType = DISPLAY_PLAIN;
static INT GFROMSCRIPT=0;

INT main (INT argc, PCHR *argv) {
	PZSTATSVIEW stats;
	time_t RequestedTime;
	INT i;
	CHR Temp[16];

	if((stats = NEWSTRUCT(ZSTATSVIEW))==NULL) {
		printf("zstatsview: Out of memory!\n");
		exit(0);
	}
	
	if(!(stats->LifeStats=zstats_Create())) {
		fprintf(stderr, "%s\n", Messages[MSG_NOMEM][DisplayType]);
		exit(1);
	}

	if(!(stats->TodaysStats=zstats_Create())) {
		fprintf(stderr, "%s\n", Messages[MSG_NOMEM][DisplayType]);
		exit(1);
	}

	time(&stats->TodaysTime);

	if(argc < 2) {
		zstatsview_DisplayUsage();
		exit(1);
	}

	/* make this an optional command line arg */
	strcpy(stats->urlPrefix, "http://patents.cnidr.org/cgi-bin/aidspatstats?");

	if(argc==2) {
		stats->YY=stats->MM=stats->DD=-1;
	} else {
		for(i=0;i<strlen(argv[2]);i++) {
			if(!isdigit(argv[2][i])) {
				zstatsview_DisplayUsage();
				exit(1);
			}
		}
		/* argv[2] is 'yymmdd', 'yymm' or 'dd' */
		/* the system stores months starting from 0*/
		switch(strlen(argv[2])) {
			case 6:
				strncpy(Temp, argv[2], 2);
				stats->YY=atoi(Temp);
				strncpy(Temp, argv[2]+2, 2);
				stats->MM=(atoi(Temp))-1;
				strncpy(Temp, argv[2]+4, 2);
				stats->DD=atoi(Temp);
				break;
			case 4:
				strncpy(Temp, argv[2], 2);
				stats->YY=atoi(Temp);
				strncpy(Temp, argv[2]+2, 2);
				stats->MM=(atoi(Temp))-1;
				stats->DD=-1;
				break;
			case 2:
				strncpy(Temp, argv[2], 2);
				stats->YY=atoi(Temp);
				stats->MM=-1;
				stats->DD=-1;
				break;
			default:
				zstatsview_DisplayUsage();
				exit(1);
				break;
		}
			
	}
	if(argc==4) {
		GFROMSCRIPT=1;
	}
	stats->List = zstats_ReadLog(argv[1]);

	if(!stats->List) {
		fprintf(stderr, "%s\n", Messages[MSG_NOSTATSFILE][DisplayType]);
		exit(1);
	}

	zstatsview_Generate(stats);

	time(&RequestedTime);
		
	
	zstatsview_Display(stats);

	exit(0);
}

void zstatsview_Generate(PZSTATSVIEW stats)
{
	time_t FirstDay=0, LastDay=0, Today;
	INT4 count=0,i,j;
	INT Found=0;
	PZSTATS z;
	INT4 TempCount, Count;
	POSITION *pos;

	time(&Today);
	time(&stats->TodaysStats->Time);	
	stats->DayCount=0;

	pos = list_First(stats->List);

	for(i=1;i<=list_Length(stats->List);i++) {
		PZSTATS z;
		time_t TempTime;

		Found = 0;
		z = list_Retrieve(stats->List,pos);
		if(!z)
			break;
		TempCount = z->BytesSent + z->BytesRec;
		TempTime = z->Time;
		zstats_Accumulate(stats->LifeStats, z);
		if(zstatsview_IsSameDay(TempTime,stats->TodaysTime))
			zstats_Accumulate(stats->TodaysStats, z);
		if(stats->DayCount==0) {
			stats->FirstDay=TempTime;
			stats->Days[0] = z;
			stats->DayCount++;
		} else {
			stats->FirstDay = stats->FirstDay > TempTime ? TempTime:stats->FirstDay;
			stats->LastDay = stats->LastDay > TempTime ? stats->LastDay:TempTime;
			for(j=0;j<stats->DayCount;j++)	{
				if(zstatsview_IsSameDay(stats->Days[j]->Time,
					TempTime)==1){
					zstats_Accumulate(stats->Days[j], z);
					j=stats->DayCount;
					break;
				} else {
					/* new day */
					stats->Days[stats->DayCount]=z;
					stats->DayCount++;
				}
			}
		}

		pos = list_Next(stats->List, pos);
	}	
	stats->HighDay.Count = 0;
	stats->HighDay.Time = 0;
	stats->LowDay.Count = 0;
	stats->LowDay.Time = 0;
	
	for(i=0;i<stats->DayCount;i++) {
		Count = stats->Days[i]->BytesSent+stats->Days[i]->BytesRec;
		if(Count > stats->HighDay.Count) {
			stats->HighDay.Count = Count;
			stats->HighDay.Time = stats->Days[i]->Time;
		}
		if(i==0) {
			stats->LowDay.Count = Count;
			stats->LowDay.Time = stats->Days[i]->Time;
		}
		if(Count < stats->LowDay.Count) {
			stats->LowDay.Count = Count;
			stats->LowDay.Time = stats->Days[i]->Time;
		}
	}
	return;
}

/*
0 if false
1 if true
*/
int zstatsview_IsSameDay(time_t t1, time_t t2)
{
	struct tm *time1, *time2;
	int yday1,year1,yday2,year2,mday1,mday2;

	time1 = gmtime(&t1);
	yday1 = time1->tm_yday;
	year1 = time1->tm_year;
	mday1 = time1->tm_mday;
	
	time2 = gmtime(&t2);
	year2 = time2->tm_year;
	mday2 = time2->tm_mday;
	yday2 = time2->tm_yday;
	
	if((yday1 == yday2) && (year1 == year2) && (mday1==mday2)) {
		return 1;
	}
	return 0;
		
}

void zstatsview_DisplayDaily(PZSTATSVIEW stats)
{
	struct tm *GMTime;
	CHR buf[256];
	PZSTATS TempStats;
	INT yy,mm,dd;
	INT4 Found = 0, i;

	/* Did they ask for today's stats? */
	GMTime = gmtime(&stats->TodaysTime);
	yy = GMTime->tm_year;
	mm = GMTime->tm_mon;
	dd = GMTime->tm_mday;
		
	if(stats->DisplayType==DISPLAY_HTML)
		printf("<title>CNIDR AIDSPAT Daily Statistics</title>");

	if((yy == stats->YY)&& (mm == stats->MM)&& (dd == stats->DD)) {
		TempStats = stats->TodaysStats;
	} else {
		/* No, then find the day they asked for */
		for(i=0;i<stats->DayCount;i++) {
			GMTime = gmtime(&stats->Days[i]->Time);
			yy = GMTime->tm_year;		
			mm = GMTime->tm_mon;		
			dd = GMTime->tm_mday;		
			if((GMTime->tm_year == stats->YY)&& 
				(GMTime->tm_mon == stats->MM)&&
				(GMTime->tm_mday == stats->DD)) {
				TempStats = stats->Days[i];
				Found=1;
				break;
			}
		}
		if(!Found) {
			printf("Statistics for requested day unavailable\n");
			exit(1);
		}
	}
	
	GMTime = gmtime(&TempStats->Time);

	/* Display the results */	
        if(stats->DisplayType == DISPLAY_HTML) {
		strftime(buf, sizeof(buf), "%A, %B %e, %Y", GMTime);
		printf("<H1><i>Statistics for %s</i></H1>\n",buf);
		printf("<HR>\n");
		printf("<pre>\n");
	} else {
		printf("Statistics for %s\n", buf);
		printf("%s\n", BAR);
	}
	printf("Total bytes sent/received:\t%d\n\n", TempStats->BytesSent+ 
		TempStats->BytesRec);

	strcpy(TempStats->Host, "NONE");
	TempStats->SessionLength = -1;
	zstatsview_DisplayStats(TempStats);

        if(stats->DisplayType == DISPLAY_HTML) {
		printf("</pre>\n");
	}
}
void zstatsview_DisplayMonthly(PZSTATSVIEW stats)
{
	struct tm *GMTime;
	INT TempDays[ZSTATSVIEW_MAXNUMDAYS], used=0;
	INT4 TempDayCount=0, i;
	INT4 Found,yy,mm,dd, j;
	CHR useday[5],usemon[5];
	CHR buf[256];
	PZSTATS Monthly;

	Monthly = zstats_Create();

	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<title>CNIDR AIDSPAT Monthly Statistics</title>");
		printf("<h1><i>Days Available</i></h1>");
		printf("<ul>\n");
	} else {
		printf("Days Available\n");
		printf("%s\n",BAR);
	}

	for(i=0;i<stats->DayCount;i++) {
		GMTime = gmtime(&stats->Days[i]->Time);
		yy=GMTime->tm_year;
		mm=GMTime->tm_mon;
		dd=GMTime->tm_mday;
		if((yy == stats->YY) &&
			(mm == stats->MM)) {
			Found=1;
			strftime(buf, sizeof(buf), "%A, %B %e, %Y", GMTime);
			if(stats->DisplayType==DISPLAY_HTML) {
				/* have we already listed this day? */
				for(j=0;j<TempDayCount;j++) {
					if(TempDays[j]==dd) {
						used=1;
						break;
					}
				}
				if((i==0)||(!used)) 
					TempDays[TempDayCount++]=dd;
				if(!used) {
					sprintf(useday,"%i", GMTime->tm_mday);
					if(GMTime->tm_mday <=9) 
						sprintf(useday,"0%i",dd);
					/* add 1 to the month! */
					sprintf(usemon, "%i",mm+1);
					if(GMTime->tm_mon+1<=9)
						sprintf(usemon,"0%i",mm);
					printf("<li><a href=\"%s%i",
						stats->urlPrefix, yy);
					printf("%s", usemon);
					printf("%s", useday);
					printf("+NOHEADER");
					printf("\">%s</a><p>\n", buf);
				}
				used=0;
			}
		}
	}

	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("</ul>\n");
	}
	for(i=0;i<stats->DayCount;i++) {
		GMTime = gmtime(&stats->Days[i]->Time);
		if(i==0) {
			strftime(buf, sizeof(buf), "%B, %Y", GMTime);
			printf("<HR><H1><i>Statistics for %s</i></H1>\n",buf);
			printf("<HR>\n");
		} 
		GMTime = gmtime(&stats->Days[i]->Time);
		yy=GMTime->tm_year;
		mm=GMTime->tm_mon;
		if((GMTime->tm_year == stats->YY) &&
			(GMTime->tm_mon == stats->MM)) {
			zstats_Accumulate(Monthly,stats->Days[i]);
		}

	}
	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<pre>");
		strcpy(Monthly->Host, "NONE");
		Monthly->SessionLength=-1;
		zstatsview_DisplayStats(Monthly);
		printf("</pre>");
	} else {
		strcpy(Monthly->Host, "NONE");
		Monthly->SessionLength=-1;
		zstatsview_DisplayStats(Monthly);
	}
	if(!Found) {
		if(stats->DisplayType==DISPLAY_HTML) 
			printf("<strong>Statistics for requested week unavailable</strong>\n");
		else
			printf("Statistics for requested week unavailable\n");
		exit(1);
	}
}

void zstatsview_DisplayYearly(PZSTATSVIEW stats)
{
	struct tm *GMTime;
	PZSTATS TempDays[ZSTATSVIEW_MAXNUMDAYS];
	INT4 TempDayCount=0, i,j;
	INT4 Found,yy,mm;
	CHR useday[5],usemon[5];
	CHR buf[256];
	PZSTATS Yearly;
	INT aumnth[ZSTATSVIEW_MAXNUMDAYS];
	INT4 aumnthCount=0, used=0;
 
	Yearly = zstats_Create();

	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<title>CNIDR AIDSPAT Yearly Statistics</title>");
		printf("<h1><i>Months Available</i></h1>");
		printf("<ul>\n");
	} else {
		printf("Months Available\n");
		printf("%s\n",BAR);
	}

	/* Display links for months available for selected year */
	for(i=0;i<stats->DayCount;i++) {
		GMTime = gmtime(&stats->Days[i]->Time);
		yy=GMTime->tm_year;
		mm=GMTime->tm_mon;
		if(yy == stats->YY) { 
			TempDays[TempDayCount++] = stats->Days[i];
			Found=1;
			strftime(buf, sizeof(buf), "%B", GMTime);
			if(stats->DisplayType==DISPLAY_HTML) {
				/* have we already listed this month? */
				for(j=0;j<aumnthCount;j++) {
					if(aumnth[j]==mm) {
						used=1;
						break;
					}
				}
				if(i==0) {
					aumnth[aumnthCount++]=mm;
				}
				if(!used) {
					/* add 1 to the month! */
					sprintf(usemon, "%i",mm+1);
					if(mm+1<=9) {
						sprintf(usemon,"0%i",mm);
					}
					printf("<li><a href=\"%s%i",
						stats->urlPrefix, yy);
					printf("%s", usemon);
					printf("+NOHEADER");
					printf("\">%s</a><p>\n", buf);
				} else {
					aumnth[aumnthCount++]=mm;
				}
				used=0;
			}
		}
	}

	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("</ul>\n");
	}
	/* Now display the yearly total */
	for(i=0;i<stats->DayCount;i++) {
		GMTime = gmtime(&stats->Days[i]->Time);
		if(i==0) {
			strftime(buf, sizeof(buf), "%Y", GMTime);
			printf("<HR><H1><i>Statistics for %s</i></H1>\n",buf);
			printf("<HR>\n");
		}
		GMTime = gmtime(&stats->Days[i]->Time);
		yy=GMTime->tm_year;
		mm=GMTime->tm_mon;
		if(GMTime->tm_year == stats->YY)
			zstats_Accumulate(Yearly,stats->Days[i]);

	}
	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<pre>");
		strcpy(Yearly->Host, "NONE");
		Yearly->SessionLength=-1;
		zstatsview_DisplayStats(Yearly);
		printf("</pre>");
	} else {
		strcpy(Yearly->Host, "NONE");
		Yearly->SessionLength=-1;
		zstatsview_DisplayStats(Yearly);
	}

	if(!Found) {
		if(stats->DisplayType==DISPLAY_HTML) 
			printf("<strong>Statistics for requested year unavailable</strong>\n");
		else
			printf("Statistics for requested year unavailable\n");
		exit(1);
	}
}

void zstatsview_DisplayUsage()
{
        if(httpbox_IsHTTPGateway()) {
		printf("Content-type: text/html\n\n");
	}

        if(httpbox_IsHTTPGateway()) {
		printf("<h1><a href=\"http://cnidr.org/\">CNIDR</a> zstatsview, Version %i.%i\n</h1>", HIVERSION,LOVERSION);
		printf("<pre>\n");
	} else {
	printf("CNIDR zstatsview, Version %i.%i\n\n", HIVERSION,LOVERSION);
	}
	printf("\tusage: zstatsview <statsfile> [option]\n\n");
	printf("Statsfile must be a statistics file generated by the CNIDR zserver.\n");
	printf("Option takes the form yymmdd.  yy will display yearly, yymm monthly, and yymmdd daily.\n");
	printf("Examples:\n\t'zstatsview 941201' displays December 1, 1994 statistics.\n");
	printf("\t'zstatsview 9412' displays December, 1994 statistics.\n");
	printf("\t'zstatsview 94' displays 1994 statistics.\n\n");
	printf("No option specified defaults to current day's statistics.\n\n");
	
        if(httpbox_IsHTTPGateway()) {
		printf("</pre>\n");
		printf("<i>I detect that you're trying to access this service via CGI script.  URLs should be of the form: <b>http://host.name/cgi-bin/scriptname?941001</b> for Oct 1, 1994.</i>\n");
	}
}

/*
To suppress displaying...	Set value to...
	Host			"NONE"
	SessionLength		-1
*/
void zstatsview_DisplayStats(PZSTATS s)
{
	if(strcasecmp(s->Host, "NONE"))
		printf("Host:               %s\n", s->Host);
	if(s->SessionLength != -1)
		printf("Session Length:     %d\n", s->SessionLength);
        printf("Bytes Received:             %d\n", s->BytesRec);
        printf("Bytes Sent:                 %d\n", s->BytesSent);
        printf("Init Request:               %d\n", s->InitRequestCount);
        printf("Init Response:              %d\n", s->InitResponseCount);
        printf("Search Request:             %d\n", s->SearchRequestCount);
        printf("Search Response:            %d\n", s->SearchResponseCount);
        printf("Present Requests:           %d\n", s->PresentRequestCount);
        printf("Present Response:           %d\n", s->PresentResponseCount);
        printf("DeleteResultSetRequest:     %d\n", 
		s->DeleteResultSetRequestCount);
        printf("DeleteResultSetResponse:    %d\n",
		s->DeleteResultSetResponseCount);
        printf("Access Control Request:     %d\n", s->AccessControlRequestCount);
        printf("Access Control Response:    %d\n",s->AccessControlResponseCount);
        printf("Resource Control Request:   %d\n", 
		s->ResourceControlRequestCount);
        printf("Resource Control Response:  %d\n", 
		s->ResourceControlResponseCount);
        printf("Trigger Resource Control:   %d\n", 
		s->TriggerResourceControlCount);
        printf("Resource Report Request:    %d\n", 
		s->ResourceReportRequestCount);
        printf("Resource Report Response:   %d\n", 
		s->ResourceReportResponseCount);
        printf("Scan Request:               %d\n", s->ScanRequestCount);
        printf("Scan Response:              %d\n", s->ScanResponseCount);
        printf("Sort Request:               %d\n", s->SortRequestCount);
        printf("Sort Response:              %d\n", s->SortResponseCount);
        printf("Segment Request:            %d\n", s->SegmentRequest);
        printf("Extended Services Request:  %d\n", 
		s->ExtendedServicesRequestCount);
        printf("Extended Services Response: %d\n", 
		s->ExtendedServicesResponseCount);
        printf("Close Request:              %d\n", s->CloseRequestCount);

}


void zstatsview_Display(PZSTATSVIEW stats)
{
	struct tm *GMTime;
	CHR buf[256];	
	if((stats->DD == 0)&&(stats->YY == 0)&&(stats->MM == -1)) {
		/* secret access ;-) */
		stats->DisplayType=DISPLAY_HTML;
		zstatsview_DisplayLife(stats);
		return;
	}
        if(httpbox_IsHTTPGateway() && (GFROMSCRIPT==0)) {
		printf("Content-type: text/html\n\n");
		stats->DisplayType=DISPLAY_HTML;
	}
	if(GFROMSCRIPT==1)
		stats->DisplayType=DISPLAY_HTML;
	if(GFROMSCRIPT==0) {
		if(stats->DisplayType == DISPLAY_HTML) {
			printf("<h1><a href=\"http://cnidr.org/\">CNIDR</a> zserver Z39.50 Usage Statistics</h1><HR>\n");
		} else {
			printf("CNIDR zserver Z39.50 Usage Statistics\n");
			printf("%s\n", BAR);
		}
	}

	if((stats->DD == -1)&&(stats->YY == -1)&&(stats->MM == -1)) {
		/* default to daily */
		GMTime = gmtime(&stats->TodaysTime);
		stats->DD = GMTime->tm_mday;
		stats->MM = GMTime->tm_mon;
		stats->YY = GMTime->tm_year; /* years since 1900 */

		zstatsview_DisplayDaily(stats);
		return;
	}

	if((stats->MM == -1) && (stats->DD == -1)) {
		/* year */
		zstatsview_DisplayYearly(stats);
		return;
	}
	if(stats->DD == -1) {
		/* month */
		zstatsview_DisplayMonthly(stats);
		return;
	}
	zstatsview_DisplayDaily(stats);
}

/*
Display total numbers plus links to all available years
*/
void zstatsview_DisplayLife(PZSTATSVIEW stats)
{
	struct tm *GMTime;
	PZSTATS TempDays[ZSTATSVIEW_MAXNUMDAYS];
	INT4 TempDayCount=0, i,j;
	INT4 Found,yy,mm;
	CHR useday[5],usemon[5];
	CHR buf[256];
	PZSTATS Lifetime;
	INT aumnth[ZSTATSVIEW_MAXNUMDAYS];
	INT auyear[ZSTATSVIEW_MAXNUMDAYS];
	INT4 aumnthCount=0, used=0,auyearCount=0;


	/* first, display links to years available */
	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<h1><i>Years Available</i></h1>");
		printf("<ul>\n");
	} else {
		printf("Years Available\n");
		printf("%s\n",BAR);
	}

	/* 
	Step through each day, check the year against those already
	checked, and create a link as appropriate
	*/
	for(i=0;i<stats->DayCount;i++) {
		GMTime = gmtime(&stats->Days[i]->Time);
		yy=GMTime->tm_year;
		strftime(buf, sizeof(buf), "%Y", GMTime);
		/* Only make links if displaying HTML */
		if(stats->DisplayType==DISPLAY_HTML) {
			/* have we already listed this year? */
			for(j=0;j<auyearCount;j++) {
				if(auyear[j]==yy) {
					used=1;
					break;
				}
			}
			if(auyearCount==0) {
				auyear[auyearCount++]=yy;
			}
			if(!used) {
				printf("<li><a href=\"%s%i+NOHEADER",stats->urlPrefix, yy);
				printf("\">%s</a><p>\n", buf);
			} else {
				auyear[auyearCount++]=yy;
			}
			used=0;
		}
	}


	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("</ul>\n");
	}

	/* Now, display lifetime numbers */
	printf("<HR><h1><i>System Lifetime Statistics</i></h1><HR>\n"); 
	Lifetime = zstats_Create();
	for(i=0;i<stats->DayCount;i++) {
		zstats_Accumulate(Lifetime,stats->Days[i]);
	}
	strcpy(Lifetime->Host, "NONE");
	Lifetime->SessionLength=-1;
	if(stats->DisplayType==DISPLAY_HTML) {	
		printf("<pre>");
		zstatsview_DisplayStats(Lifetime);
		printf("</pre>");
	} else {
		zstatsview_DisplayStats(Lifetime);
	}

}
