/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zstatsview.h
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#ifndef _zstatsview_
#define _zstatsview_

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "libz3950.h"

#define HIVERSION 1
#define LOVERSION 0
#define ZSTATSVIEW_MAXNUMDAYS 10000
 
#define DISPLAY_PLAIN 0
#define DISPLAY_HTML 1
 
#define MSG_USAGE 0
#define MSG_NOSTATSFILE 1
#define MSG_NOMEM 2

#define BAR "-------------------------------------------------------------"
 
PCHR Messages[]={
        "Usage: zstatsview <file> [options]",
        "<strong>You must call this application from a CGI shell script specifyi
ng a stats file on which to operate</strong>",
        "Error reading specified log file",
        "<strong>Error reading specified log file</strong>",
        "Out of memory!",
        "<strong>Out of memory!</strong>"
};

typedef struct tagDAYSTATS {
        time_t Time;
        INT4 Count;
} DAYSTATS;

typedef struct tagZSTATSVIEW { 
	CHR buf[256];
	PLIST List;
	PZSTATS LifeStats, TodaysStats, Days[ZSTATSVIEW_MAXNUMDAYS];
	DAYSTATS HighDay, LowDay;
	INT2 Options;
	time_t FirstDay, LastDay, TodaysTime;
	INT DisplayType;
	INT4 DayCount;
	INT2 YY,MM,DD;
	struct tm *GMTime;
	CHR urlPrefix[256];		/* used for building URLs */
} ZSTATSVIEW;
typedef ZSTATSVIEW *PZSTATSVIEW;

void zstatsview_DisplayUsage();
void zstatsview_Display(PZSTATSVIEW stats);
void zstatsview_DisplayDaily(PZSTATSVIEW stats);
void zstatsview_DisplayWeekly(PZSTATSVIEW stats);
void zstatsview_DisplayMonthly(PZSTATSVIEW stats);
void zstatsview_DisplayYearly(PZSTATSVIEW stats);
void zstatsview_DisplayLife(PZSTATSVIEW stats);
void zstatsview_Generate(PZSTATSVIEW stats);
int zstatsview_IsSameDay(time_t t1, time_t t2);
void zstatsview_Test();
void zstatsview_DisplayStats(PZSTATS s);

#endif

