/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zclient.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/*#define ZCLIENT_DEBUG*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "gdt.h"
#include "libcnidr.h"
#include "libz3950.h"
#include "zclient.h"

INT check_attrs (INT i);
INT ztoi (PCHR p,INT type);
PCHR zclient_ConstructQuery();
PCHR build_term (INT i);
INT4 MaxTerms;

FILE *GTraceFP;
void zutil_StripAllBrackets(PCHR s);
void xputs (PCHR s);

INT4 main(INT argc, PCHR *argv)
{
        PZCLIENT Client;
	/*freopen("/tmp/kag.out", "w", stderr);*/
        
	if(!(Client = zclient_CreateInit(argc, argv))) {
                fprintf(stderr, "zserver: Unable to create zserver object!\n");
                exit(1);
        }
 
	zclient_Initialize(Client);
#ifdef ZCLIENT_DEBUG
	fprintf(stderr, "main:Initialized\n");
#endif
	zclient_Search(Client);
#ifdef ZCLIENT_DEBUG
	fprintf(stderr, "main:Search complete\n");
#endif
	zclient_Present(Client);
#ifdef ZCLIENT_DEBUG
	fprintf(stderr, "main:Present complete\n");
#endif
	zclient_Destroy(Client);

	exit(0);
}

void zclient_Initialize(PZCLIENT zclient)
{
	INT4 MaxLength,Length;
	PNETBOXPROFILE Socket;
	PSESSION Session;
	PINITREQUEST InitRequest;
	PINITRESPONSE InitResponse;

	if(!zclient || zclient->CurrentState!=ZCLIENT_STATE_BEGIN)
		return;

	Socket = zclient_GetSocket(zclient);
	Session = zclient_GetDefaults(zclient);

	/* Open a connection to the remote server */
	if(netbox_Open(Socket)!=1) {
		printf("Couldn't open a connection to %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}

	/* Call routine to construct an entire init request pdu. */
	InitRequest=InitRequest_CreateInitAllASCII((PCHR)NULL, 
		"yy","yy", Session->iMaxRecSize, 
		Session->iPreferredMsgSize, 
		Session->iImpID,
		Session->iImpName, 
		Session->iImpVersion, (PCHR)NULL);
	if(!InitRequest) {
		fprintf(stderr, "zclient_Init:failed to allocate Request\n");
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}
#ifdef ZCLIENT_DEBUG
	hex_dirf(InitRequest, 0, stderr);
#endif
	/* 
	Get a BER-encoded octet buffer representation of the Init Request
	from the Init Request PDU object.
	 */
	MaxLength = zclient->Defaults->iMaxRecSize;
	zutil_GetBEREncodedBuffer(InitRequest,zclient->Buffer,
		&Length,MaxLength);
	if(Length <= 0) {
		fprintf(stderr, "zclient_Init:Unable to get BER buffer\n");
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}	

	/* 
	Destroy the Init Request PDU object since we only needed the
	octet buffer it encoded for us.
	*/
	InitRequest_Destroy(InitRequest);

	/* Send the BER-encoded Init Request over the wire */
	if((netbox_SendBuffer(Socket, (PCHR)zclient->Buffer, Length))!=Length) {
		printf("Couldn't send data to %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}
	/* Read a BER-encoded buffer from the wire */
	if((Length=zutil_GetBERFromNet(Socket, zclient->Buffer,MaxLength))<=0) {
		printf("Couldn't read data from %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}

	/* "Cast" the BER-encoded buffer into a real PDU object */
	InitResponse = (PINITRESPONSE)zutil_CreateFromData(zclient->Buffer, 
		Length);

	/* Is it what we think it is? */
	if(InitResponse_GetTag(InitResponse)!=INITRESPONSE_TAG) {
		printf("Server didn't return initialize response!\n");
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	} else {
		/* Let's grab some of the objects within the response object */

#ifdef ZCLIENT_DEBUG
	hex_dirf(InitResponse, 0, stderr);
#endif
		Session->iPreferredMsgSize = 
			InitResponse_GetPreferredMessageSize(InitResponse);
		Session->iMaxRecSize = 
			InitResponse_GetExceptionalRecordSize(InitResponse);

		if(!InitResponse_GetResult(InitResponse)) {
			printf("Server access denied to %s[%i]\n",
				netbox_GetHostName(Socket),
				netbox_GetPort(Socket));
			zclient->CurrentState = ZCLIENT_STATE_FATAL;
		} else
			zclient->CurrentState = ZCLIENT_STATE_SEARCH;
	}

	/* Destroy the object */
	InitResponse_Destroy(InitResponse);

}

void zclient_Search(PZCLIENT zclient)
{
	INT4 MaxLength,Length;
	PNETBOXPROFILE Socket;
	PSESSION Session;
	DATA_DIR *pdu;

	if(!zclient || zclient->CurrentState!=ZCLIENT_STATE_SEARCH)
		return;

	Socket = zclient_GetSocket(zclient);
	Session = zclient_GetDefaults(zclient);

	/* Call routine to construct an entire search request pdu. */
	pdu=(PSEARCHREQUEST)SearchRequest_CreateInitAllASCII((PCHR)NULL,
		Session->sSmallSetUpperBound,
		Session->sLargeSetLowerBound,
		Session->sMedSetPresentNum,
		Session->sReplaceIndicator,
		Session->sResultSetName,Session->sDBNames,
		(PCHR)NULL, (PCHR)NULL, Session->pPreferredRecSyntax,
		Session->sQuery, BIB1_OID);
	if(!pdu) {
		zclient->CurrentState = ZCLIENT_STATE_SEARCH;
		return;
	}
#ifdef ZCLIENT_DEBUG
	hex_dirf(pdu, 0, stderr);
#endif
	/* 
	Get a BER-encoded octet buffer representation of the Search Request
	from the Search Request PDU object.
	 */
	MaxLength = zclient->Defaults->iMaxRecSize;
	zutil_GetBEREncodedBuffer(pdu,zclient->Buffer,
		&Length,MaxLength);
	if(Length <= 0) {
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}	

	/* 
	Destroy the Search Request PDU object since we only needed the
	octet buffer it encoded for us.
	*/
	SearchRequest_Destroy(pdu);

	/* Send the BER-encoded Search Request over the wire */
	if((netbox_SendBuffer(Socket, (PCHR)zclient->Buffer, Length))!=Length) {
		printf("Couldn't send data to %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		return;
	}
	/* Read a BER-encoded buffer from the wire */
	if((Length=zutil_GetBERFromNet(Socket, zclient->Buffer,MaxLength))<=0) {
		printf("Couldn't read data from %s[%i].  Expected search response\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		return;
	}

	/* "Cast" the BER-encoded buffer into a real PDU object */
	pdu = (PSEARCHRESPONSE)zutil_CreateFromData(zclient->Buffer, 
		Length);
	/* Is it what we think it is? */
	if(zutil_GetTag(pdu)!=SEARCHRESPONSE_TAG) {
		printf("Server didn't return search response!\n");
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	} else {
		/* Let's grab some of the objects within the response object */

#ifdef ZCLIENT_DEBUG
	hex_dirf(pdu, 0, stderr);
#endif
		zclient->HitCount = SearchResponse_GetResultCount(pdu);	
		zclient_DisplaySearchResults(zclient, pdu);
	}

	/* Destroy the object */
	SearchResponse_Destroy(pdu);
}

PZCLIENT zclient_Create()
{
        PZCLIENT s;
 
        if((s=NEWSTRUCT(ZCLIENT))==NULL)
                return NULL;
 
        s->Socket = NULL;
        s->Defaults = NULL;
        s->CurrentState = ZCLIENT_STATE_BEGIN;
	s->Buffer = NULL;
	s->DisplayBuffer = NULL;
 
        return s;
}
 
void zclient_Destroy(PZCLIENT z)
{
        if(z) {
                if(z->Defaults)
                        FREE(z->Defaults);
                if(z->Socket)
                        netbox_Destroy(z->Socket);
		if(z->Buffer)
			FREE(z->Buffer);
        	FREE(z);
	}
}

/* sorry, quick hack */
INT GDoCommandLine=0;

PZCLIENT zclient_CreateInit(INT argc, PCHR *argv)
{
        INT Result, i;
        PZCLIENT s;
	PNETBOXPROFILE Socket;
	PSESSION Session;
 
        if((s=zclient_Create())==NULL)
                return NULL;
 
        if((s->Socket = NEWSTRUCT(NETBOXPROFILE))==NULL) {
                zclient_Destroy(s);
                return NULL;
        }
 
        if((s->Defaults = NEWSTRUCT(SESSION))==NULL) {
                zclient_Destroy(s);
                return NULL;
        }
 
        zclient_SetDefaults(s);

	/* 
	If there is an ini file specified, we DO NOT want to start
	as a CGI, even if environment variables say to do so.
	*/
	for(i=1;i<argc;i++) {
		if((argv[i][0]=='-') && (argv[i][1]=='i')) 
			GDoCommandLine=1;
	}
	
	/* Should the client act as a CGI? */ 
	if(httpbox_IsHTTPGateway()) {
		printf("Content-type: text/html\n\n");
	}
	if((httpbox_IsHTTPGateway())&& (!GDoCommandLine)) {
		PCGI cgi;

		cgi = cgi_Create();
		cgi_GetInput(cgi);

		/* This function is useful for debugging.  It displays
		the contents of the HTML form submitted to zclient
		*/
/*
		cgi_DisplayContents(cgi);
		exit(0);
*/
		/* load what you need into the session table */
		zclient_LoadFromCGI(s,cgi);
		
		if(s->CurrentState == ZCLIENT_STATE_FATAL) {
			zclient_Destroy(s);
			return NULL;
		}
		cgi_Destroy(cgi);
        } else {
		ReadProfileSettings(s->Defaults,(PCHR)PROFILEFN,
			(PCHR)PROFILEGROUP);
        	if((Result=GetCmdLineSettings(s->Defaults,argc,(PCHR*)argv,
                	(PCHR)PROFILEFN,(PCHR)PROFILEGROUP))!=0)
                	printf("Failed to read command line option #%i\n",
				Result);
	}
	/* Allocate a buffer for net reading and writing */
	if((s->Buffer=NEWUSTRING(s->Defaults->iMaxRecSize))==NULL) {
		fprintf(stderr, "Error allocating buffer\n");
		return 0;
	}

	Session = zclient_GetDefaults(s);
	Socket = zclient_GetSocket(s);
	
	/* Print a banner */
/*
	if(!strcmp(Session->DisplayRecSyntax, HTML_OID))
		printf("%s\n", ZCLIENT_BANNER_HTML);
	else {
		printf("%s\n", ZCLIENT_BANNER_PLAIN);
	}
*/
	/* Input values ok? */
	if(Session->Host[0]=='\0')
		zclient_Error(s, ZCLIENT_ERR_HOST);
	if(Session->sQuery[0]=='\0')
		zclient_Error(s, ZCLIENT_ERR_TERM);
	if(Session->sDBNames[0]=='\0')
		STRCPY(Session->sDBNames, "xxdefault");

	netbox_SetHostName(Socket,Session->Host);
	netbox_SetPort(Socket, Session->Port);
	netbox_SetTimeOutSec(Socket, Session->TimeOut);

        return s;
}

void zclient_Error(PZCLIENT z, INT4 Error)
{
	if(!strcmp(z->Defaults->DisplayRecSyntax, HTML_OID))
		printf("%s\n", ZCLIENT_ERRORS[Error].html);
	else {
		printf("%s\n", ZCLIENT_ERRORS[Error].plain);
	}

	/* What is the severity of the error? */
	switch(Error) {
		case ZCLIENT_ERR_HOST:
			z->CurrentState = ZCLIENT_STATE_FATAL;
			break;
		case ZCLIENT_ERR_TERM:
			z->CurrentState = ZCLIENT_STATE_FATAL;
			break;
		default:
			z->CurrentState = ZCLIENT_STATE_OK;
	}
	
	if(z->CurrentState == ZCLIENT_STATE_FATAL)
		exit(1);
}
 
void zclient_SetDefaults(PZCLIENT zserver)
{
        if(zserver && zserver->Defaults) {
                PSESSION d=zserver->Defaults;
 
                d->MaxSessions = 10;
                d->Port = 210;
                d->TimeOut = 3600L;
                STRCPY(d->Trace,"OFF");
                STRCPY(d->TraceLog, "zclient_trace.log");
                STRCPY(d->AccessLog, "zclient_access.log");
                STRCPY(d->QueryFormat, "RPN");
                STRCPY(d->DisplayRecSyntax, "1.2.840.10003.5.1000.34.1");
                STRCPY(d->DiagSetId, "1.2.840.10003.3.1");
                STRCPY(d->iImpID, "34");
                STRCPY(d->iImpName, "CNIDR zclient/gateway");
                STRCPY(d->iImpVersion, "2.0");
                d->iProtocolVersion = 2;
                STRCPY(d->iOptions, "yy");
                d->iPreferredMsgSize = 2000000;
                d->iMaxRecSize = 2000000;
                d->iResult = 0L;
                d->sResultCount = 0L;
                d->sSmallSetUpperBound = 0l;
                d->sLargeSetLowerBound = 2l;
                d->sMedSetPresentNum = 0l;
                d->sReplaceIndicator = 0;
                STRCPY(d->sResultSetName, "Default");
                STRCPY(d->sAttributeSet, "1.2.840.10003.3.1");
                STRCPY(d->pResultSetId, "Default");
                d->pResultSetStartPoint = 1l;
                d->pNumRecsReq = 1l;
                STRCPY(d->pElementSetNames, "B");
                STRCPY(d->pPreferredRecSyntax, "1.2.840.10003.5.101");
                d->pNumRecsReturned = 0L;
                d->pNextResultSetPos = 0L;
        }
}
 
PSESSION zclient_GetDefaults(PZCLIENT z)
{
	if(z)
		return z->Defaults;
	return NULL;
}

PNETBOXPROFILE zclient_GetSocket(PZCLIENT z)
{
	if(z)
		return z->Socket;
	return NULL;
}

#define MAXTERMLEN 1024
#define MAXTERMCOUNT 100
PCHR DecodedTerms[MAXTERMCOUNT];
CHR GOperator[MAXTERMLEN];

typedef struct tagTERM {
	CHR Term[MAXTERMLEN];
	INT	Use;
	INT	Structure;
	INT	Truncation;
	INT	Position;
INT	Relation;
} TERM;
TERM Terms[MAXTERMCOUNT];


#define T_USE 0
#define T_STRUCTURE 1
#define T_POSITION 2
#define T_RELATION 3
#define T_TRUNCATION 4

void zclient_LoadFromCGI(PZCLIENT z, PCGI cgi)
{
	PSESSION s;
	CHR Term[MAXTERMLEN];
	CHR Use[MAXTERMLEN];
	CHR Structure[MAXTERMLEN];
	CHR Position[MAXTERMLEN];
	CHR Relation[MAXTERMLEN];
	CHR Truncation[MAXTERMLEN];
	CHR TermSpecified=0;
	PCHR p,t;
	INT4 i,j,k, Term1Filled=0, Term2Filled=0;
	
	if(!z || !cgi)
		return;

	s = z->Defaults;
	
	MaxTerms = cgi->EntryCount;
	
	/* Grab the easy stuff */
	STRCPY(s->Host, cgi_GetValueByName(cgi, "Host"));
	s->Port = atoi(cgi_GetValueByName(cgi, "Port"));
	if((t=cgi_GetValueByName(cgi, "startrec"))!=NULL)
		s->pResultSetStartPoint = atoi(t);
	s->pNumRecsReq = atoi(cgi_GetValueByName(cgi, "maxrecords"));

	if((t=cgi_GetValueByName(cgi, "ESN"))!=NULL)
		STRCPY(s->pElementSetNames, t);
	else
		STRCPY(s->pElementSetNames, "B");
/*
		s->pElementSetNames[0]='';
*/
	if((t=cgi_GetValueByName(cgi, "dbname"))!=NULL)
		STRCPY(s->sDBNames, t);
	else
		s->sDBNames[0]='\0';
	if((t=cgi_GetValueByName(cgi, "operator"))!=NULL)
		STRCPY(GOperator, t);
	else
		GOperator[0]='\0';
	if((t=cgi_GetValueByName(cgi, "rtype"))!=NULL) {
		if(!strcasecmp(t, "SUTRS"))
			STRCPY (s->pPreferredRecSyntax, SUTRS_OID);
		if(!strcasecmp(t, "HTML"))
			STRCPY(s->pPreferredRecSyntax,HTML_OID);
		if(!strcasecmp(t, "USMARC"))
			STRCPY(s->pPreferredRecSyntax,USMARC_OID);
	} else
		s->pPreferredRecSyntax[0]='\0';
	
	/* Get the DisplayRecordSyntax */
	if((t=cgi_GetValueByName(cgi, "DisplayRecordSyntax"))!=NULL) {
		if(!strcasecmp(t, "SUTRS"))
			STRCPY (s->DisplayRecSyntax, SUTRS_OID);
		if(!strcasecmp(t, "HTML"))
			STRCPY(s->DisplayRecSyntax,HTML_OID);
		if(!strcasecmp(t, "USMARC"))
			STRCPY(s->DisplayRecSyntax,USMARC_OID);
	} else
		s->DisplayRecSyntax[0]='\0';

	if((t=cgi_GetValueByName(cgi, "attrset"))!=NULL) {
		if(!strcasecmp(t, "STAS"))
			STRCPY (s->sAttributeSet, "1.2.840.10003.3.1000.6.1");
		if(!strcasecmp(t, "BIB1"))
			STRCPY (s->sAttributeSet, "1.2.840.10003.3.1");
	} else
		s->sAttributeSet[0]='\0';

	/* Grab the hard stuff */
	for(i=0;i<MaxTerms;i++) {
		sprintf(Term, "term_term_%i", i);
		sprintf(Use, "term_use_%i", i);
		sprintf(Structure, "term_struct_%i", i);
		sprintf(Position, "term_pos_%i", i);
		sprintf(Relation, "term_rel_%i", i);
		for(j=0;j<MAXTERMCOUNT;j++) {
			if(p=cgi_GetValueByName(cgi, Term)) {
				for(k=0;k<strlen(p);k++) {
					if(!(isspace(p[k]))) {
						strcpy(Terms[i].Term, p);
						TermSpecified = 1;
						if(i==1) {
							/* good, regular term*/
							Term1Filled=1;
						} else {
							Term2Filled=1;
						}
						break;
					}
				}
			}
			if(p=cgi_GetValueByName(cgi, Use))
				Terms[i].Use = ztoi(p,T_USE);
			if(p=cgi_GetValueByName(cgi, Structure))
				Terms[i].Structure = ztoi(p,T_STRUCTURE);
			if(p=cgi_GetValueByName(cgi, Position))
				Terms[i].Position = ztoi(p,T_POSITION);
			if(p=cgi_GetValueByName(cgi, Relation))
				Terms[i].Relation = ztoi(p,T_RELATION);
			if(p=cgi_GetValueByName(cgi, Truncation))
				Terms[i].Relation = ztoi(p,T_TRUNCATION);
		} /* for */
	} /* for */

	if(!TermSpecified) {
		printf("<i><img src=\"/pto/etc/warning.gif\">You must supply a query term!</i>");
		z->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}
	if(Term2Filled && !Term1Filled) {
		printf("<i><img src=\"/pto/etc/warning.gif\">Searching on anti-text alone prohibited!</i>");
		z->CurrentState = ZCLIENT_STATE_FATAL;
		return;

	}

        for (i = 1; (i <= MAXTERMCOUNT) && (strlen (Terms[i].Term) > 0); i++)
               DecodedTerms[i] = build_term (i);

	MaxTerms = i-1;
 
	if(p=zclient_ConstructQuery())
		strcpy(s->sQuery, p);
	else s->sQuery[0]='\0';

	z->CurrentState = ZCLIENT_STATE_BEGIN;
}

PCHR build_term (INT i) {
	CHR buf[MAXTERMLEN], tmp[MAXTERMLEN];
	PCHR p;

	if (!strlen (Terms[i].Term))
		return (NULL);
 
	if (check_attrs (i) == 0)
		return (strdup (Terms[i].Term));
	sprintf (buf, "%s[", Terms[i].Term);
	if (Terms[i].Use > 0) {
		sprintf (tmp, "%d,%d,", 1, Terms[i].Use);
		strcat (buf, tmp);
	}
	if (Terms[i].Relation > 0) {
		sprintf (tmp, "%d,%d,", 2, Terms[i].Relation);
		strcat (buf, tmp);
	}
	if (Terms[i].Structure > 0) {
		sprintf (tmp, "%d,%d,", 4, Terms[i].Structure);
		strcat (buf, tmp);
	}
	if (Terms[i].Position > 0) {
		sprintf (tmp, "%d,%d,", 3, Terms[i].Position);
		strcat (buf, tmp);
	}
	if (Terms[i].Truncation > 0) {
		sprintf (tmp, "%d,%d,", 5, Terms[i].Truncation);
		strcat (buf, tmp);
	}
	p = (char*)strrchr (buf, ',');
	if (p)
		*p = ']';
	return (strdup (buf));
}

INT check_attrs (INT i) 
{

    int     count = 0;
 
    if (Terms[i].Use > 0)
        ++count;
    if (Terms[i].Position > 0)
        ++count;
    if (Terms[i].Relation > 0)
        ++count;
    if (Terms[i].Truncation > 0)
        ++count;
    if (Terms[i].Position > 0)
        ++count;
    if (Terms[i].Structure > 0)
        ++count;
 
    return (count);
}

static INT i=0;

PCHR zclient_ConstructQuery()
{
	CHR buf[500];
	PCHR p;

	if (MaxTerms == 1)
		return(strdup(DecodedTerms[1]));

	if (i == MaxTerms - 2) {
		sprintf (buf, "%s(%s,%s)", GOperator, DecodedTerms[i + 1],
			DecodedTerms[i+2]);
	} else {
		++i;
		p = zclient_ConstructQuery();
		--i;
		sprintf (buf, "%s(%s,%s)", GOperator, DecodedTerms[i + 1], p);
		free (p);
	}
	return (strdup (buf));
}

struct _nm{
	char name[50];
	int value;
};

struct _nm use_tab[]={
	{"FullText",2145},
	{"Title",4},
	{"Abstract",62},
	{"Claims",2562},
	{"InventorName",2506},
	{"AssigneeName",2512},
	{"PatentNumber",2033},
	{"Subject",21},
	{"Author",1003}};
int use_tab_count=9;
struct _nm rel_tab[]={
	{"<",1},
	{"<=",2},
	{"=",3},
	{">=",4},
	{">",5},
	{"<>",6},
	{"PHONETIC",100},
	{"STEM",101},
	{"RELEVANCE",102},
	{"ANTI",10} /* KAG - I've added this to BIB-1 of v.3 */
};
int rel_tab_count=10;
struct _nm pos_tab[]={
	{"first in field",1},
	{"first in subfield",2},
	{"any position",3}};
int pos_tab_count=3;
 
struct _nm struct_tab[]={
	{"phrase",1},
	{"word",2},
	{"key",3},
	{"year",4},
	{"date",5},
	{"word list",6}};
int struct_tab_count=6;

INT ztoi (PCHR p,INT type) {
	INT i;

	if (!strlen (p))
		return (-1);
	
	switch(type) {
		case T_USE:
			for(i=0; i<use_tab_count; i++){
				if(!strcasecmp(p,use_tab[i].name)){
					return(use_tab[i].value);
				}
			}
			return(1); /* default=title */
		case T_RELATION:
			for(i=0; i<rel_tab_count; i++){
				if(!strcasecmp(p,rel_tab[i].name))
					return(rel_tab[i].value);
				}
			return(3); /* default=equal */
		case T_STRUCTURE:
			for(i=0; i<struct_tab_count; i++){
				if(!strcasecmp(p,struct_tab[i].name))
					return(struct_tab[i].value);
			}
			return(2); /* default=word */
		case T_POSITION:
			return 3;	/* any position in field */
		case T_TRUNCATION:
			return 100;	/* do not truncate */
		default:
			return 0; /* ? */
	}
}

INT4 GStartRec;
void zclient_Present(PZCLIENT zclient)
{
	INT4 MaxLength,Length;
	PNETBOXPROFILE Socket;
	PSESSION Session;
	DATA_DIR *pdu;
	INT4 NumRecs;

	if(!(zclient) || (zclient->CurrentState!=ZCLIENT_STATE_PRESENT))
		return;
	
	Socket = zclient_GetSocket(zclient);
	Session = zclient_GetDefaults(zclient);
	
	/* 
	Figure out the starting record number and number of records to 
	ask for.  If user asked for out of range, do I fudge it for them?
	Nah.
	*/
	GStartRec = Session->pResultSetStartPoint;
	NumRecs = (GStartRec+Session->pNumRecsReq-1) > zclient->HitCount? 
		zclient->HitCount-GStartRec+1:Session->pNumRecsReq,

	/* Call routine to construct an entire present request pdu. */
	pdu=(PSEARCHREQUEST)PresentRequest_CreateInitAllASCII((PCHR)NULL,
		Session->pResultSetId, GStartRec, NumRecs,
		Session->pElementSetNames, Session->pPreferredRecSyntax); 
#ifdef ZCLIENT_DEBUG
	hex_dirf(pdu, 0, stderr);
#endif
	/* 
	Get a BER-encoded octet buffer representation of the Present Request
	from the Present Request PDU object.
	 */
	MaxLength = zclient->Defaults->iMaxRecSize;
	zutil_GetBEREncodedBuffer(pdu,zclient->Buffer,
		&Length,MaxLength);
	if(Length <= 0) {
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	}	

	/* 
	Destroy the Present Request PDU object since we only needed the
	octet buffer it encoded for us.
	*/
	PresentRequest_Destroy(pdu);

	/* Send the BER-encoded Present Request over the wire */
	if((netbox_SendBuffer(Socket, (PCHR)zclient->Buffer, Length))!=Length) {
		printf("Couldn't send data to %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		return;
	}
	/* Read a BER-encoded buffer from the wire */
	if((Length=zutil_GetBERFromNet(Socket, zclient->Buffer,MaxLength))<=0) {
		printf("Couldn't read data from %s[%i]\n",
			netbox_GetHostName(Socket), 
			netbox_GetPort(Socket));
		return;
	}

	/* "Cast" the BER-encoded buffer into a real PDU object */
	pdu = (PSEARCHRESPONSE)zutil_CreateFromData(zclient->Buffer, 
		Length);
#ifdef ZCLIENT_DEBUG
/*
	hex_dirf(pdu, 0, stderr);
*/
#endif
	/* Is it what we think it is? */
	if(zutil_GetTag(pdu)!=PRESENTRESPONSE_TAG) {
		printf("Server didn't return present response!\n");
		zclient->CurrentState = ZCLIENT_STATE_FATAL;
		return;
	} else {
		/* Let's grab some of the objects within the response object */
		zclient->RecordCount = 
			PresentResponse_GetNumberOfRecordsReturned(pdu);	
		if(zclient->RecordCount > 0) {
			DATA_DIR *r;
			zclient->CurrentState = ZCLIENT_STATE_OK;
		
			switch(PresentResponse_GetRecordType(pdu)) {
				case RESPONSERECORDS_TAG:
					r=(PRESPONSERECORDS)PresentResponse_GetResponseRecords(pdu);
					break;
				case NONSURROGATEDIAGNOSTIC_TAG:
					r=(PNONSURROGATEDIAGNOSTIC)PresentResponse_GetNonSurrogateDiagnostic(pdu);
					break;
				default:
					printf("Unknown record type\n");
			}	
			if(r!=NULL)
				zclient_DisplayRecords(zclient, r);
		} else
			zclient->CurrentState = ZCLIENT_STATE_PRESENT;


	}

	/* Destroy the object */
	PresentResponse_Destroy(pdu);

	zclient->CurrentState = ZCLIENT_STATE_OK;
}

/*
Attempt to display all response records.
*/
void zclient_DisplayRecords(PZCLIENT zclient, DATA_DIR *r)
{
	INT4 i,RecSyntaxLen;
	DATA_DIR *rec,*retRec,*record;
	PDATABASENAME dbname;
	PEXTERNAL ext;
	PSUTRS sutrs;
	POBJECTIDENTIFIER oid;
	CHR RecSyntax[256], DispRecSyntax[256];
	FILE *fin,*fout;
	PCHR TempFile;
	INT fino,vtype;
	
	/* 
	Determine the record syntax that the user wants records display
		in.  Conversion may be necessary.
	Loop through all records available in r.
	Convert if necessary, otherwise print the record in native format.
	*/

	strcpy(DispRecSyntax, zclient->Defaults->DisplayRecSyntax);

	/* What type of records do we have? */
	switch(zutil_GetTag(r)) {
		case RESPONSERECORDS_TAG:
			break;
		case NONSURROGATEDIAGNOSTIC_TAG:
			zclient_DisplayDiagRec(r);
			return;
		case MULTIPLENONSURDIAGNOSTICS_TAG:
			printf("Unsupported record type:Multiple diags\n");
	}

	/* 
	The code I have to display MARC records reads from a file,
	so go ahead and open one just in case.  
	*/
	if((TempFile = tempnam("/tmp","zclient"))==NULL) {
		perror("zclient_DisplayRecords");
		printf("Unable to get a temp file name\n");
	}
	if((fout=fopen(TempFile, "w"))==NULL) {
		perror("zclient_DisplayRecords");
		printf("Unable to open temp file\n");
		return;
	}

	/* Loop through all available records */
	for(i=1;i<=ResponseRecords_GetCount(r);i++) {
		/* Grab the database name */
		dbname=ResponseRecords_GetName(r,i);
		if(!dbname) {
			/* The database name is optional */
			/*printf("Database:\n");*/
		} else {
			/* Want to print the database name? */
			/*DatabaseName_Printf(dbname, "Database: %s\n");*/
		}

		record=ResponseRecords_GetRecord(r,i);
		if(!record) {
			printf("No record in NamePlusRecord!\n");
			continue;
		}
		retRec = NamePlusRecord_GetRetrievalRecord(record);
		if(!retRec) {
			/* might be a diagnostic record */
			if(record->ptr.child->fldid==2) {
				/* surrogate diagnostic */
				zclient_DisplayDiagRec(record->ptr.child);
			} else printf("No Retrieval Record available\n");
			continue;
		}
		ext = RetrievalRecord_GetExternal(retRec);
		if(!ext) {
			printf("No External available\n");
			continue;
		}
		/* 
		We now have a handle to the external record.
		Depending on the OID, we will check the contents of the
		single-ASN1-type, octet-aligned or arbitrary choice
		of the external encoding
		*/
		oid = External_GetDirectReference(ext);
		if(!oid) {
			/* don't know what type of record we have! */
			RecSyntax[0] = '\0';	
		} else {
			RecSyntaxLen= DirectReference_GetLength(oid);
			MEMCPY(RecSyntax,DirectReference_GetData(oid),
				RecSyntaxLen);
			RecSyntax[RecSyntaxLen]='\0';
		}
		if(!strcmp(RecSyntax, USMARC_OID))
			rec = External_GetEncodingAligned(ext);
		if(!strcmp(RecSyntax, SUTRS_OID) ||
			!strcmp(RecSyntax, HTML_OID))
			rec = External_GetEncodingSingle(ext);

		if(!rec) {
			printf("No Record in External!\n");
			continue;
		}
		/* 
		At this point, rec->ptr.child is an ASN1 type of
		ANY.  You should check the OID (Direct Reference)
		for record type.
		Go ahead and nip it in the bud if we don't support
		the record syntax received.
		*/
		if(strcmp(RecSyntax, USMARC_OID) &&
			strcmp(RecSyntax, SUTRS_OID) &&
			strcmp(RecSyntax, HTML_OID)) {
			printf("Unsupported record syntax of %s\n",
				RecSyntax);
			continue;
		}

		/* Is it a USMARC record? */
		if(!strcmp(RecSyntax, USMARC_OID)) {
			/* 
			Write MARC records to a file for display later. 
			*/
			fwrite(rec->ptr.data, 1, rec->count,fout);
			continue;
                }
		/* Load the record into a buffer for future processing */
		if(!strcmp(RecSyntax, SUTRS_OID) || !strcmp(RecSyntax,
			HTML_OID)) {
			sutrs = rec->ptr.child;
			if(!sutrs) {
				printf("No record under External encoding\n");
				continue;
			}
			MEMCPY(zclient->Buffer, sutrs->ptr.data, sutrs->count);
			zclient->Buffer[sutrs->count]='\0';
		}
		/* Is it a SUTRS record? */
		if(!strcmp(RecSyntax, SUTRS_OID)) {
			if(!strcmp(RecSyntax, DispRecSyntax)) {
				/* User wants native format */
				printf("%s\n", zclient->Buffer);
				printf("-----------------------------------\n");
			} else {
				/* Conversion necessary */
				if(!strcmp(DispRecSyntax, HTML_OID)) {
					printf("<pre>\n");
					printf("%s\n", zclient->Buffer);
					printf("</pre><hr>\n");
				}
				if(!strcmp(DispRecSyntax, USMARC_OID))
					printf("Unable to convert to USMARC\n");
			}
		}
		/* Is it an HTML record? */
		if(!strcmp(RecSyntax, HTML_OID)) {
			if(!strcmp(RecSyntax, DispRecSyntax)) {
				/* User wants native format */
				printf("%s\n", zclient->Buffer);
			} else {
				/* Conversion necessary */
				if(!strcmp(DispRecSyntax, SUTRS_OID))
					printf("Unable to convert to SUTRS\n");
				if(!strcmp(DispRecSyntax, USMARC_OID))
					printf("Unable to convert to USMARC\n");
			}
		}
	}
	fclose(fout);
	/* 
	If record syntax was USMARC, you should now have a file
	sitting somewhere with all the MARC records.  Call dispmarc for each.
	*/
	if(!strcmp(RecSyntax, USMARC_OID)) {
		INT4 RecCount;

		if((fin=fopen(TempFile, "r"))==NULL) {
			perror("zclient_DisplayRecords");
			printf("File error displaying USMARC record\n");
			return;
		}
		fino=fileno(fin);
		vtype=0;
		if(!strcmp(DispRecSyntax, HTML_OID))
			printf("<pre>\n");
		RecCount = ResponseRecords_GetCount(r);
		for(i=0;i<RecCount;i++) {

			if(!strcmp(DispRecSyntax, HTML_OID)) {
				printf("<b><i>Result set record %d of %d</b></i>",
					i+GStartRec,zclient->HitCount);
				dbname=ResponseRecords_GetName(r,i);
				if(dbname!=NULL)
					DatabaseName_Printf(dbname, 
						"<b><i>, from the %s database.</b></i><p>\n");
				else
					printf("<p>\n");
			} 
			if(DispMARC (fino,0,i,vtype,(long)-1,(long)80,(void *)NULL)==NULL) {
				printf("Failed to decode MARC record.\n");
			}
			if(!strcmp(DispRecSyntax, HTML_OID))
				printf("<hr>\n");
			if(!strcmp(DispRecSyntax, SUTRS_OID))
				printf("-----------------------------------\n");
		}
		if(!strcmp(DispRecSyntax, HTML_OID))
			printf("</pre>\n");
		fclose(fin);
		unlink(TempFile);
	}
}
	
void zclient_DisplaySearchResults(PZCLIENT z, DATA_DIR *d)
{
	DATA_DIR *t;
	PCHR YourQuery;

	if(!z)
		return;
	YourQuery = NEWSTRING(strlen(z->Defaults->sQuery)+1);
	
	strcpy(YourQuery, z->Defaults->sQuery);
	zutil_StripAllBrackets(YourQuery);

	if(!strcmp(z->Defaults->DisplayRecSyntax, HTML_OID)) {
		printf("<title>CNIDR Search [%s]</title>\n", 
			YourQuery);
		/* 
		I don't want following line displayed if the query
		was really just a fetch.  Have to hack it for now.
		*/
		if(GDoCommandLine==0) {
			if(z->HitCount==1)
				printf("<i><b>%d</b> record matched your query <b>[%s]</b><p></i>\n", z->HitCount, YourQuery);
			else
				printf("<i><b>%d</b> records matched your query <b>[%s]</b><p></i>\n", z->HitCount, YourQuery);
		}

		printf("<i>Generated by <a href=\"http://cnidr.org/\">CNIDR</a> http->Z39.50 gateway (zclient, version %s)</i><p><hr>",VERSION);
	} else {
		printf("Query matched %d items\n", 
			z->HitCount);
	}
	if(YourQuery)
		FREE(YourQuery);

	if((t = SearchResponse_GetRecords(d))!=NULL) {
		if(zutil_GetTag(t)==RESPONSERECORDS_TAG) {
			z->CurrentState = ZCLIENT_STATE_PRESENT;
			return;
		}
		if(zutil_GetTag(t)==NONSURROGATEDIAGNOSTIC_TAG) {
			zclient_DisplayDiagRec(t);
			z->CurrentState = ZCLIENT_STATE_SEARCH;
			return;
		}
	} else {
		if(z->HitCount>0)
			z->CurrentState = ZCLIENT_STATE_PRESENT;
		else
			z->CurrentState = ZCLIENT_STATE_SEARCH;
	}
}
void zclient_DisplayDiagRec(DATA_DIR *r)
{
	DATA_DIR *d;
	INT ErrorCode;
	if(!r)
		return;

	
	ErrorCode = zutil_GetTaggedInt(r, ASN1_INTEGER);
	printf("Error[%d]: ", ErrorCode);

	/* I need to finish this.  Just doing the most frequent errors */
	switch(ErrorCode) {
		case 109:
			printf("Database unavailable: ");
			break;
		default:
			break;
	}
	if((d = zutil_GetTaggedObject(r, ASN1_VISIBLESTRING))!=NULL) {
		OctetString_Printf(d, "%s\n");
	} else {
		/* version 3 style? */
/* I don't know the tag value for interational string !
		if((d=zutil_GetTaggedObject(r,ASN1_INTERNATIONALSTRING))!=NULL){
			OctetString_Printf(d, "%s\n");
		} else printf("\n");
*/
printf("\n");
	} 

}
void zutil_StripAllBrackets(PCHR s)
{
	INT4 i,j=0,Inside=0;
	PCHR T;
	
	if(!(s) || (s[0]=='\0'))
		return;

	if((T=NEWSTRING(strlen(s)+1))==NULL)
		return;

	for(i=0;i<strlen(s);i++) {	
		switch(s[i]) {
			case '[':
				Inside = 1;
				break;
			case ']':
				if(Inside==1)
					Inside = 0;
				break;
			default:
				if(Inside==0)
					T[j++]=s[i];
				break;
		}
	}
	T[j]='\0';
	strcpy(s,T);
}

void xputs (PCHR s)
{
	fwrite(s,strlen(s),1,stdout);
}
