/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zclient.h
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#ifndef _zclient_
#define _zclient_

#include "gdt.h"
#include "libcnidr.h"
#include "libz3950.h"

#define VERSION "1.02"

#define ZCLIENT_STATE_BEGIN 0
#define ZCLIENT_STATE_OK 1
#define ZCLIENT_STATE_PRESENT 2
#define ZCLIENT_STATE_SEARCH 3
#define ZCLIENT_STATE_FATAL 4

#define PROFILEFN "zclient.ini"
#define PROFILEGROUP "Default"

/* 
If you inherit this code, please leave this banner message as is!
(we don't ask for much!)
*/
#define ZCLIENT_BANNER_HTML "<i><a href=\"http://cnidr.org/\">CNIDR</a> <b>zclient</b>, Version 2.0</i><p>"
#define ZCLIENT_BANNER_PLAIN "CNIDR zclient, Version 2.0"

typedef struct tagERRORMSG {
	CHR plain[256];
	CHR html[256];
} ERRORMSG;

/*
If you add new error messages, MAKE SURE that 
the number of entries in ZCLIENT_ERRORS matches the possible
error code values (prefixed with ZCLIENT_ERR).
*/
#define ZCLIENT_ERR_HOST 0
#define ZCLIENT_ERR_TERM 1

ERRORMSG ZCLIENT_ERRORS[]={
	{"You must supply a valid host name",
	"<strong>You must supply a valid host name</strong>"},
	{"You must supply a query term",
	"<strong>You must supply a query term</strong>"}
};

PINITREQUEST InitRequest_CreateInitAllASCII(PCHR ReferenceId, 
	PCHR ProtocolVersion, PCHR Options, INT4 PreferredMessageSize, 
	INT4 ExceptionalRecordSize, PCHR ImplementationId, 
	PCHR ImplementationName, PCHR ImplementationVersion, 
	PCHR UserInformation);

typedef struct tagZCLIENT {
        SESSION *Defaults;
        NETBOXPROFILE *Socket;
        INT CurrentState;
        LIST *RecordList;
	PUCHR Buffer;
	PCHR DisplayBuffer;
	INT4 HitCount;
	INT4 RecordCount;
} ZCLIENT;
typedef ZCLIENT *PZCLIENT;
 
PZCLIENT zclient_Create();
void zclient_Destroy(PZCLIENT z);
PZCLIENT zclient_CreateInit(INT argc, PCHR *argv);
void zclient_SetDefaults(PZCLIENT zserver);
void zclient_InitializeTraceLog(PSESSION S);
PSESSION zclient_GetDefaults(PZCLIENT z);
PNETBOXPROFILE zclient_GetSocket(PZCLIENT z);
void zclient_LoadFromCGI(PZCLIENT z, PCGI cgi);
void zclient_Initialize(PZCLIENT zclient);
void zclient_Search(PZCLIENT zclient);
void zclient_Present(PZCLIENT zclient);
void zclient_DisplayRecords(PZCLIENT zclient, DATA_DIR *r);
void zclient_Error(PZCLIENT z, INT4 Error);
void zclient_DisplaySearchResults(PZCLIENT z, DATA_DIR *d);
void zclient_DisplayPresentResults(PZCLIENT z);
void zclient_DisplayDiagRec(DATA_DIR *r);

#endif
