/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zserver.c
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

/* Want some debugging messages printed to stderr? Uncomment this */
#define ZSERVER_DEBUG

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "libcnidr.h"
#include "libz3950.h"
#include "sapi.h"
#include "zserver.h"

CHR GTempString[256];
CHR GTempString2[256];

INT main (INT argc, PCHR *argv) {
	PZSERVER Server;
	PSESSION Session;
	PNETBOXPROFILE Socket;
	FILE *tfp;

	zserver_Status("main", ZSERVER_START, "");

	if(!(Server = zserver_CreateInit(argc, argv))) {
		zserver_Error("main",ZSERVER_NOSERVEROBJ,"");
		exit(1);
	}

	Session = zserver_GetDefaults(Server);
	Socket = zserver_GetSocket(Server);
	tfp = Server->Defaults->TFP;

	/* Start the server listening for connections */
	zserver_Status("main", ZSERVER_NETCHECK, "");

	netbox_CurrentStatus(Socket);
	
	zserver_Status("main", ZSERVER_STARTING, "");

	if (netbox_StartServer(Socket) < 0) {
		if(Socket->ServerOwner == NB_COMMANDLINE) {
			sprintf(GTempString, "%i", Socket->Port);
			zserver_Error("main", ZSERVER_BADPORT1, GTempString);
			zserver_Error("main", ZSERVER_BADPORT2, "");
		}
		zserver_Destroy(Server);
		exit(1);
	}

	if(!strcasecmp(Session->ServerType, "STANDALONE")) {
		zserver_Status("main", ZSERVER_STANDALONE, "");
		zserver_RunForkingServer(Server);
	} else {
		zserver_Status("main", ZSERVER_INETD, "");
		zserver_RunSingleServer(Server);
	}

	zserver_Destroy(Server);

	exit(0);
} /* main() */

void zserver_RunSingleServer(PZSERVER Server)
{
    	PNETBOXPROFILE ChildSocket, Socket;
	PSESSION Session;
	FILE *tfp;

	Socket = zserver_GetSocket(Server);
	Session = zserver_GetDefaults(Server);
	tfp = Session->TFP;
	
	sprintf(GTempString, "%i", Session->Port);
	zserver_Status("zserver_RunSingleServer", ZSERVER_WAITING, GTempString);
	if (!(ChildSocket = netbox_AcceptClient(Socket))) {
		zserver_Error("zserver_RunSingleServer", ZSERVER_NOACCEPT,"");
		exit(0);
	}
	zserver_Status("zserver_RunSingleServer", ZSERVER_CONNECT, 
		ChildSocket->PeerName);
	zserver_UpdateAccessLog(Session->AccessLog, ChildSocket->PeerName);

	netbox_Close(Socket);
	netbox_Destroy(Socket);	
	zserver_SetSocket(Server, ChildSocket);
	zserver_ServeClient(Server);
}

void zserver_RunForkingServer(PZSERVER Server)
{
    	PNETBOXPROFILE ChildSocket, Socket;
	INT child_proc, zpid;
    	FILE *tfp;
	INT ChildCount=0;
	PINT Children;
	INT i,Status,j;
	INT id;
	PSESSION Session;
	INT MaxChildren;
	
	Socket = zserver_GetSocket(Server);
	Session = zserver_GetDefaults(Server);

 	MaxChildren = Session->MaxSessions;
	Children = (PINT)calloc(sizeof(INT),MaxChildren);

	tfp = Session->TFP; 
	
	for(;;) {
		sprintf(GTempString, "%i", Session->Port);
		zserver_Status("zserver_RunForkingServer", ZSERVER_WAITING, 
			GTempString);
		if (!(ChildSocket = netbox_AcceptClient(Socket))) {
			exit(0);
		}
		zserver_Status("zserver_RunForkingServer", ZSERVER_CONNECT, 
			ChildSocket->PeerName);
		zserver_UpdateAccessLog(Session->AccessLog,
			ChildSocket->PeerName);
		/* Any children finished yet? */
		for(i=0;i<ChildCount;i++) {
			id = waitpid(Children[i], &Status,
				WNOHANG|WUNTRACED);
			switch(id) {
				case 0:
					break;
				case -1:
					perror("zserver_RunForkingServer");
					break;
				default:
					ChildCount--;
					Children[i]=-5;
			}
		}
		/* Squeeze children list */
		j = 0;
		for(i=0;i<MaxChildren;i++)
			if(Children[i] != -5)
				Children[j++]=Children[i];
						
		if(ChildCount == MaxChildren) {
			/* Send init response set to denied */
			printf("Session limit of %i exceeded\n",
				Session->MaxSessions);
			/* SEND INIT RESPONSE HERE */
			netbox_Close(ChildSocket);
		} else {

		if((child_proc=fork())==0) {
			/* In child */
			zpid = getpid();
			Server->Socket = ChildSocket;
			zserver_ServeClient(Server);
			netbox_Close(ChildSocket);
			exit(0);
		} else {
			/* In Parent */
			if(child_proc == -1) {
				/* Error creating child */
				zserver_LogChildError(tfp);
			} else {
				/* 
				Keep track of number of children for
				load balancing purposes
				*/
				Children[ChildCount++]=child_proc;
				/* Close the child socket */
				netbox_Close(ChildSocket);
			}
		} /* else */
		} /* else */
	} /* for */
} /* zserver_RunForkingServer() */

void zserver_ServeClient(PZSERVER Server)
{
	BOOLEAN Continue = TRUE;
	PUCHR Buf; 		/* communications buffer */
	DATA_DIR *pdu, *rpdu;
	INT4 Length;
	PNETBOXPROFILE Socket;
	PSESSION Session;
    	FILE *tfp;

	Session = Server->Defaults;
	Socket = Server->Socket;

	zserver_InitializeTraceLog(zserver_GetDefaults(Server));

	if(!(strcasecmp(Session->Stats, "ON")))
		Server->Stats=zstats_CreateInit(Socket->PeerName);

	Server->pid = getpid();

	tfp = Session->TFP; 
	if((Buf=NEWUSTRING(Session->iMaxRecSize))==NULL)
		return;
	
	/* Start the main server loop */
	while (Continue == TRUE) {

        	/* Read a BER-encoded buffer from the wire */
        	Length=zutil_GetBERFromNet(Socket,Buf,Session->iMaxRecSize);
		if(Length<=0)
			break;

        	/* "Cast" the BER-encoded buffer into a real PDU object */
        	pdu = zutil_CreateFromData(Buf, Length);
		zserver_LogPDU(Server, pdu);
		
		/* Update statistics */
		zstats_AddPDU(zserver_GetStats(Server), pdu);

		switch(zutil_GetTag(pdu)) {
			case INITREQUEST_TAG:
				zserver_Status("main", ZSERVER_INITREC, "");
				rpdu = zserver_CreateResponsePDU(Server,pdu);
			
				/* Update statistics */
				zstats_AddPDU(zserver_GetStats(Server), rpdu);

				zserver_LogPDU(Server, rpdu);
				zutil_GetBEREncodedBuffer(rpdu, Buf,&Length,
					Session->iMaxRecSize);
				netbox_SendBuffer(Socket, (PCHR)Buf, Length);
				zserver_DestroyPDU(Server, rpdu);
				break;
			case SEARCHREQUEST_TAG:
				zserver_Status("main", ZSERVER_SRCHREC, "");
				rpdu = zserver_CreateResponsePDU(Server,pdu);
				
				/* Update statistics */
				zstats_AddPDU(zserver_GetStats(Server), rpdu);

				zserver_LogPDU(Server, rpdu);
				zutil_GetBEREncodedBuffer(rpdu, Buf,&Length,
					Session->iMaxRecSize);
				netbox_SendBuffer(Socket, (PCHR)Buf, Length);
				zserver_DestroyPDU(Server, rpdu);
				break;
			case PRESENTREQUEST_TAG: {
				DATA_DIR *Records;

				zserver_Status("main", ZSERVER_PRESENTREC, "");
				Records = NULL;

				if(Server->CurrentState==ZSERVER_STATE_SEARCH){
					Records=zserver_CreateDiagnosticRecord(
						109, "Result set unavailable");
					rpdu=SearchResponse_CreateInitAllASCII(
						Session->iReferenceId, 1, 0, 0, 						FALSE,
						3, 0, Records, NULL, NULL);
				zserver_LogPDU(Server, rpdu);
				zutil_GetBEREncodedBuffer(rpdu, Buf,&Length,
					Session->iMaxRecSize);
				netbox_SendBuffer(Socket, (PCHR)Buf, Length);
				break;	

				}
				rpdu = zserver_CreateResponsePDU(Server,pdu);
				
				/* Update statistics */
/*
				zstats_AddPDU(zserver_GetStats(Server), rpdu);
*/

				zserver_LogPDU(Server, rpdu);
				zutil_GetBEREncodedBuffer(rpdu, Buf,&Length,
					Session->iMaxRecSize);
				netbox_SendBuffer(Socket, (PCHR)Buf, Length);
				zserver_DestroyPDU(Server, rpdu);
				break;
			}
			default:
				fprintf(stderr, "Unsupported PDU: %i\n", 
					zutil_GetTag(pdu));
				Continue = FALSE;
		} /* switch() */
/*
		zserver_DestroyPDU(Server, pdu);
*/
	} /* while (Continue == TRUE) */

	FREE(Buf);

	if(Socket->ServerOwner == NB_COMMANDLINE)
		printf("Closing session.\n");
/*
	zstats_WriteLog(Server->Stats, Server->Defaults->StatsLog);
*/
	
	return;

} /* zserver_ServeClient() */

void zserver_UpdateAccessLog(PCHR File, PCHR S)
{
	FILE *fp;
	CHR DateTime[80];

	DateTime[0]='\0';	
	strbox_TheTime(DateTime);

	if((fp = fopen(File, "a"))!=NULL) {
		fprintf(fp, "%s %s\n", DateTime, S);
		fclose(fp);
	}
}

void zserver_LogChildError(FILE *fp)
{
	switch(errno) {	
		case EAGAIN:
			break;
		case ENOMEM:
			break;
		default:
			break;
	}
}

void zserver_InitializeTraceLog(PSESSION S)
{
	/* Does the sysadmin want Tracing on? */
	if(!strcasecmp(S->Trace, "ON")) {
		if (S->TraceLog[0] == '\0')
			S->TFP = stdout;
		else {
			/* Open TraceLog file */
			/* Cat the process ID onto the filename */
			PCHR T;

			T = NEWSTRING(STRLEN(S->TraceLog)+20);
			pid = getpid();
			sprintf(T, "%s.%i", (PCHR)S->TraceLog, pid);
			STRCPY(S->TraceLog, T);	
			FREE(T);
			if((S->TFP=fopen(S->TraceLog,"a"))==NULL) {
				perror("zserver");
			} /* if */
          	} /* else */
	} /* if */
} 

PZSERVER zserver_Create()
{
	PZSERVER s;

	if((s=NEWSTRUCT(ZSERVER))==NULL)
		return NULL;

	s->Socket = NULL;
	s->Defaults = NULL;
	s->CurrentState = ZSERVER_STATE_OK;

	return s;
}

void zserver_Destroy(PZSERVER z)
{
	if(z) {
		if(z->Defaults)
			FREE(z->Defaults);
		if(z->Socket)
			netbox_Destroy(z->Socket);
		if(z->Stats)
			FREE(z->Stats);
		if(z->ResultSet!=NULL)
			ResultSet_Destroy(z->ResultSet);
	}
}

PZSERVER zserver_CreateInit(INT argc, PCHR *argv)
{
	INT4 SAPIDBCount, i, ZSDBCount;
	INT Result;
	PZSERVER s;

	if((s=zserver_Create())==NULL)
		return NULL;

	if((s->Socket = NEWSTRUCT(NETBOXPROFILE))==NULL) {
		zserver_Destroy(s);
		return NULL;
	}

	if((s->Defaults = NEWSTRUCT(SESSION))==NULL) {
		zserver_Destroy(s);
		return NULL;
	}

	zserver_SetDefaults(s);
	
	ReadProfileSettings(s->Defaults,(PCHR)PROFILEFN,(PCHR)PROFILEGROUP);
	if((Result=GetCmdLineSettings(s->Defaults,argc,(PCHR*)argv,
		(PCHR)PROFILEFN,(PCHR)PROFILEGROUP))!=0)
		printf("Failed to read command line option #%i\n",Result);

	sprintf((PCHR)GIniFile,"%s/%s",(PCHR)s->Defaults->ServerPath,
		(PCHR)PROFILEFN);

	s->Socket->Port = s->Defaults->Port; 
	s->Socket->TimeOutSec = s->Defaults->TimeOut;

	s->RecordList = (LIST *)NULL;

	/* Initialize the CNIDR Search API */
	s->sapi = sapi_CreateInitFile(s->Defaults->SAPI,"Default");
	if(s->sapi == NULL) {
		zserver_Error("zserver_CreateInit", ZSERVER_BADSAPI1, "");
		zserver_Error("zserver_CreateInit", ZSERVER_BADSAPI2, "");
		zserver_Destroy(s);
		return NULL;
	}
	s->ResultSet = NULL;
	/* 
	Make sure all the databases we claim to serve are valid via
	the Search API.
	*/
	/* How many databases does the SAPI have? */
	if((SAPIDBCount = sapi_GetDatabaseCount(s->sapi))<=0) {
		zserver_Error("zserver_CreateInit", ZSERVER_NOSAPIDBS1, "");
		zserver_Error("zserver_CreateInit", ZSERVER_NOSAPIDBS2, "");
		zserver_Destroy(s);
		return NULL;
	}
	sprintf(GTempString, "%d", SAPIDBCount);
	zserver_Status("zserver_CreateInit", ZSERVER_SAPIDBS, GTempString);

	/* How many databases does zserver.ini want to serve? */
	GetPrivateProfileString(PROFILEGROUP,"DBList","",
		GTempString,MAXPROFILESTR,(PCHR)GIniFile);
	if(GTempString[0]=='\0') {
		/* None??? */
		zserver_Error("zserver_CreateInit", ZSERVER_NOZSDBS1, "0");
		zserver_Error("zserver_CreateInit", ZSERVER_NOZSDBS2, "");
		zserver_Destroy(s);
		return NULL;
	}
	ZSDBCount = strbox_itemCount(GTempString);
	/* Attempt to mount each database */
	s->DatabaseList = list_Create();
	for(i=1;i<=ZSDBCount;i++) {
		PSAPI_DBASE DB;
		strbox_item(GTempString,i,GTempString2);
		DB = sapi_GetDatabase(s->sapi, GTempString2);
		if(DB==NULL) {
			zserver_Error("zserver_CreateInit", ZSERVER_NOMOUNT, 
				GTempString2);
			zserver_Destroy(s);
			return NULL;
		}
		zserver_Status("zserver_CreateInit",ZSERVER_MOUNT,GTempString2);
		list_InsertAfter(s->DatabaseList, list_Last(s->DatabaseList),
			DB, LIST_NEAR);
	}
	sprintf(GTempString2, "%d", list_Length(s->DatabaseList));
	zserver_Status("zserver_CreateInit", ZSERVER_ZSDBS, GTempString2);
	
	return s;
}

void zserver_SetDefaults(PZSERVER zserver)
{
	if(zserver && zserver->Defaults) {
		PSESSION d=zserver->Defaults;

        	STRCPY(d->ServerType, "StandAlone");
        	d->MaxSessions = 10;
        	d->Port = 210;
        	d->TimeOut = 3600L;
        	STRCPY(d->Trace,"OFF");
        	STRCPY(d->TraceLog, "zserver_trace.log");
        	STRCPY(d->Stats,"OFF");
        	STRCPY(d->StatsLog, "zserver_stats.log");
        	STRCPY(d->AccessLog, "zserver_access.log");
        	STRCPY(d->SAPI, "sapi.ini");
        	STRCPY(d->QueryFormat, "RPN");
        	STRCPY(d->DiagSetId, "1.2.840.10003.3.1");
        	STRCPY(d->iImpID, "34");
        	STRCPY(d->iImpName, "CNIDR zserver");
        	STRCPY(d->iImpVersion, "1.0");
        	d->iProtocolVersion = 2;
        	STRCPY(d->iOptions, "yy");
        	d->iPreferredMsgSize = 2000000;
        	d->iMaxRecSize = 2000000;
        	d->iResult = 0L;
        	d->sResultCount = 0L;
        	d->sSmallSetUpperBound = 0l;
        	d->sLargeSetLowerBound = 1l;
        	d->sMedSetPresentNum = 0l;
        	d->sReplaceIndicator = 255; 
        	STRCPY(d->sResultSetName, "Default");
        	STRCPY(d->sAttributeSet, "1.2.840.10003.3.1");
        	STRCPY(d->pResultSetId, "Default");
        	d->pResultSetStartPoint = 1l;
        	d->pNumRecsReq = 1l;
        	STRCPY(d->pElementSetNames, "B");
        	STRCPY(d->pPreferredRecSyntax, "1.2.840.10003.5.101");
        	d->pNumRecsReturned = 0L;
        	d->pNextResultSetPos = 0L;
	}
}

/*
Negotiates based on pdu from client
*/
DATA_DIR *zserver_CreateResponsePDU(PZSERVER zserver, DATA_DIR *pdu)
{
	POCTETSTRING OStr;
	PBITSTRING Bits;
	DATA_DIR *rpdu;	
	INT4 Temp;
	PSESSION Session;

	Session = zserver_GetDefaults(zserver);

	if((zserver) && (zserver->Defaults) && (pdu)) {
		PSESSION d=zserver->Defaults;

		switch(zutil_GetTag(pdu)) {
			case INITREQUEST_TAG:
				zserver_SetState(zserver, ZSERVER_STATE_OK);	

				/* Do we like the protocol version? */
				Bits = InitRequest_GetProtocolVersion(pdu);
				Temp = ProtocolVersion_GetVersion(Bits);
				d->iProtocolVersion = Temp;
				if((Temp > 3) || (Temp < 2)) {
					/* Nope, we don't support it */
					zserver_SetState(zserver, 
						ZSERVER_STATE_BADPROTVER);	
				}	
				/* Do we like the options?  */
				Bits = InitRequest_GetOptions(pdu);

				/* Grab the Reference ID */
				OStr = InitRequest_GetReferenceId(pdu);
				OctetString_CopyToCString(OStr,d->iReferenceId);
				
				/* Negotiate the buffer sizes */
				Temp=InitRequest_GetPreferredMessageSize(pdu);
        			d->iPreferredMsgSize = 
					d->iPreferredMsgSize < Temp?
					d->iPreferredMsgSize:Temp;
				Temp=InitRequest_GetExceptionalRecordSize(pdu);
        			d->iMaxRecSize = 
					d->iMaxRecSize > Temp?
					d->iMaxRecSize:Temp;
				/* 
				Could do cute thing with cgi for 
				authentication checking (but not now!)
				*/

				/* This adds 1404 bytes */
				rpdu = InitResponse_CreateInitAllASCII(
					d->iReferenceId, 
					"yy",d->iOptions,d->iPreferredMsgSize,
					d->iMaxRecSize, TRUE,
					d->iImpID, d->iImpName, d->iImpVersion,
					NULL);
				if(!rpdu) {
					zserver_SetState(zserver, 
						ZSERVER_STATE_SYSTEMERROR);	
				}
				return rpdu;
			case SEARCHREQUEST_TAG: {
				STACK *Stack;
				INT4 HitCount;
				PQUERY Query;
				PRPNQUERY Rpn;
				DATA_DIR *Records;
				POCTETSTRING DBN;
				DATA_DIR *DBList;
				/* 
				See SearchAPI for details on these data types
				*/
				PSAPI_DBASE sapiDB;
				PSAPI_QUERY sapiQuery;

				Records=NULL;

				/* we only support one result set */
				if(zserver->ResultSet!=NULL) {
					ResultSet_Destroy(zserver->ResultSet);
					zserver->ResultSet=NULL;
				}
				
				DBList = SearchRequest_GetDatabaseNames(pdu);
				DBN = SearchRequest_GetDatabaseName(DBList, 1);
				OctetString_CopyToCString(DBN, 
					Session->sDBNames);

				/* Is the requested database in our list? */
				sapiDB = zserver_GetDatabase(zserver,
					Session->sDBNames);
				if(sapiDB == NULL) {
					Records=zserver_CreateDiagnosticRecord(
						109,"Database not available");
					rpdu=SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, FALSE,
						3, 0, Records, NULL, NULL);
					return rpdu;
				}

				/* Do we support the query? */
				Stack = NEWSTRUCT(STACK);
				Query = SearchRequest_GetQuery(pdu);
				if(!Query) {
					Records=zserver_CreateDiagnosticRecord(
						108, "Malformed query");
					rpdu=SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, FALSE,
						3, 0, Records, NULL, NULL);
					return rpdu;
				
				}
				Rpn = Query_GetQueryType1(Query);
				if(!Rpn) {
					Records=zserver_CreateDiagnosticRecord(
						107,"Query type not supported");
					rpdu=SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, FALSE,
						3, 0, Records, NULL, NULL);
					return rpdu;

				}
				/* Build a stack */
				RpnQuery_GetStack(Rpn->ptr.child->next, Stack);	

fprintf(stderr, "Starting search[%i]...\n", zserver->pid);
				if((sapiQuery=query_CreateStack(Stack))==NULL) {
					/* Search API couldn't build query */
					Records=zserver_CreateDiagnosticRecord(
						3,"Search failed");
					rpdu=SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, FALSE,
						3, 0, Records, NULL, NULL);
					return rpdu;
					
				}
				zserver->ResultSet = 
					sapi_Search(zserver->sapi, sapiDB, 
						sapiQuery);
				if(zserver->ResultSet==NULL) {
					/* Search failed big time */
					Records=zserver_CreateDiagnosticRecord(
						3,"Search failed");
					rpdu=SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, FALSE,
						3, 0, Records, NULL, NULL);
					return rpdu;
				}
				HitCount = ResultSet_GetHitCount(
					zserver->ResultSet);
fprintf(stderr, "Completed search[%i]...\n", zserver->pid);
				/* check for errors */
				switch(HitCount) {
					case -7:
						Records=
						zserver_CreateDiagnosticRecord(
							1,MSG_NOINDEX);
						rpdu=
					SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, 
						FALSE, 3, 0, Records, NULL, 
						NULL);
						return rpdu;
						break;
					case -6:
							Records=
						zserver_CreateDiagnosticRecord(
							3,MSG_BADQUERYSTACK);
							rpdu=
					SearchResponse_CreateInitAllASCII( 
						d->iReferenceId, 1, 0, 0, 
						FALSE, 3, 0, Records, NULL, 
						NULL);
						return rpdu;
					case -8:
							Records=
						zserver_CreateDiagnosticRecord(
							3,MSG_BADQUERYSTACKSIZE);
						rpdu=
					SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, 
						FALSE, 3, 0, Records, NULL, 
						NULL);
						return rpdu;
					case -5:
							Records=
						zserver_CreateDiagnosticRecord(
							3,MSG_BADBUILDATTR);
							rpdu=
					SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, 
						FALSE, 3, 0, Records, NULL, 
						NULL);
						return rpdu;
					default:
						if(HitCount < 0) {
							Records=
						zserver_CreateDiagnosticRecord(
							 3,MSG_UNKNOWN);
							rpdu=
					SearchResponse_CreateInitAllASCII(
						d->iReferenceId, 1, 0, 0, 
						FALSE, 3, 0, Records, NULL, 
						NULL);
							return rpdu;
						} /* if */
						break;
				}
				rpdu = SearchResponse_CreateInitAllASCII(
					d->iReferenceId, HitCount, 0, 0, TRUE,
					0, 0, Records, NULL, NULL);
				if(!rpdu) {
					zserver_SetState(zserver, 
						ZSERVER_STATE_SYSTEMERROR);	
				}
				/* Clean up stack */
				return rpdu;
			}
			case PRESENTREQUEST_TAG: {
				PRESPONSERECORDS Records;

				/* Are there records to clean up? */
				if(zserver->RecordList!=NULL)
					zserver_DestroyRecordList(
						zserver->RecordList);
				zserver->RecordList = list_Create();

				/* Is the range requested valid? */

				/* Is the result set name OK? */
	
				/* Create the response records */
				Records = zserver_CreateResponseRecords(
					zserver, pdu);
				
				/* Construct the Present Response */
				rpdu = PresentResponse_CreateInitAllASCII(
					d->iReferenceId, 
					list_Length(zserver->RecordList), 
					0, 0, Records, 
					NULL);
				if(!rpdu) {
					zserver_SetState(zserver, 
						ZSERVER_STATE_SYSTEMERROR);	
				}
				return rpdu;
				break;
			}
			default:
				fprintf(stderr, "Unknown tag\n");
				break;
		} /* switch()*/
	} 
	return NULL;
}

void zserver_SetState(PZSERVER zserver, INT State)
{
	zserver->CurrentState = State;
}

INT zserver_GetState(PZSERVER zserver)
{
	return zserver->CurrentState;
}

void zserver_DestroyPDU(PZSERVER z, DATA_DIR *d) 
{

	dfree(d);
}

void zserver_LogPDU(PZSERVER z, DATA_DIR *d)
{
	if((z->Defaults) && (z->Defaults->TFP)) {
		FILE *tfp=z->Defaults->TFP;
		fprintf(tfp, "%s\n", z->Socket->PeerName);
		zutil_LogPDU(d,tfp);
	}
}

PNETBOXPROFILE zserver_GetSocket(PZSERVER z)
{
	if(z)
		return z->Socket;
	return NULL;
}

PSESSION zserver_GetDefaults(PZSERVER z)
{
	if(z)
		return z->Defaults;
	return NULL;
}

PZSTATS zserver_GetStats(PZSERVER z)
{
	if(z)
		return z->Stats;
	return NULL;
}

void zserver_SetSocket(PZSERVER z, PNETBOXPROFILE s)
{
	if(z)
		z->Socket = s;
}

void zserver_DestroyRecordList(LIST *l)
{
	if(l) {
		PCHR *c;
		while(!list_Empty(l)) {
			c = (PCHR *)list_Retrieve(l, list_Last(l)); 
			if(c)
				FREE(c);
			list_Delete(l,list_Last(l));
		}
		FREE(l);
	}
}

#define TEST_RECORD "This is a test record\n"

PRESPONSERECORDS zserver_CreateResponseRecords(PZSERVER zserver, DATA_DIR *pdu)
{
	PRESPONSERECORDS Records;
	PNAMEPLUSRECORD NPR;
	INT4 i;
	INT4 RecordLength, Start, Count;
	PCHR Record, ESName;
	PELEMENTSETNAMES ESN;  /* ElementSetNames */
	POCTETSTRING Str;
	PPREFERREDRECORDSYNTAX PRS;
	PCHR Prs=(PCHR)NULL;	/* PreferredRecordSyntax */
	CHR Ars[64];		/* ActualRecordSyntax */
	INT4 PrsLen;
	PSESSION Session;

	if(!zserver || !pdu)
		return NULL;

	if(zserver->ResultSet==NULL)
		return NULL;

	Session = zserver->Defaults;

	Records=ResponseRecords_Create();

	Start = PresentRequest_GetResultSetStartPoint(pdu);
	Count = PresentRequest_GetNumberOfRecordsRequested(pdu);

	/* Sanity check requested record values */
	
	ESN = PresentRequest_GetSimple(pdu);
	if(ESN) {
		Str=ElementSetNames_GetGeneric(ESN);
		ESName = NEWSTRING(OctetString_GetLength(Str)+1);
		strncpy(ESName, (PCHR)OctetString_GetData(Str),
			OctetString_GetLength(Str));
		ESName[OctetString_GetLength(Str)]= '\0';
	} else {
		ESName = NEWSTRING(64);
		STRCPY(ESName, "F");
	}	
	if((PRS=PresentRequest_GetPreferredRecordSyntax(pdu))!=NULL) {
		Prs=PreferredRecordSyntax_GetData(PRS);
		PrsLen= PreferredRecordSyntax_GetLength(PRS);
		Prs[PrsLen]='\0';
	} 
	if(!Prs) {
		Prs=NEWSTRING(STRLEN(SUTRS_OID)+1);
		STRCPY(Prs, SUTRS_OID);
	}

	for(i=Start;i<Start+Count;i++) {
		PSUTRS sutrs;
		strcpy(Ars, Prs);
		Record = ResultSet_GetRecord(zserver->ResultSet,
			i, Session->iMaxRecSize, &RecordLength,
			ESName,Ars);
		if(!Record)
			break;
		/* Can the buffer size handle this record plus the others? */

		sutrs=sutrs_Create();
		sutrs_SetData(sutrs, (PUCHR)Record, RecordLength);
		list_InsertAfter(zserver->RecordList, 
			list_Last(zserver->RecordList),
			sutrs_GetData(sutrs), 
			LIST_NEAR);
		if(!strcasecmp(Ars, HTML_OID)) {
			NPR=NamePlusRecord_CreateHTML(
			Session->sDBNames,sutrs);
			ResponseRecords_Add(Records,NPR);
		} else {
			NPR=NamePlusRecord_CreateSUTRS(
				Session->sDBNames, sutrs);
			ResponseRecords_Add(Records,NPR);
		}
	} /* for */
	if(ESName)
		FREE(ESName);
	if(Prs)
		FREE(Prs);

	return Records;
}

PNONSURROGATEDIAGNOSTIC zserver_CreateDiagnosticRecord(INT4 Condition,
	PCHR S)
{
	PNONSURROGATEDIAGNOSTIC r;
	POCTETSTRING AddInfo;

	if(!S)
		return NULL;

	r = NonSurrogateDiagnostic_Create();
	if(!r)
		return NULL;

	DefaultDiagFormat_SetCondition(r, Condition);
	AddInfo = OctetString_CreateTag(ASN1_VISIBLESTRING);
	if(!AddInfo)
		return NULL;
	OctetString_SetData(AddInfo, (PUCHR)S, strlen(S));
	DefaultDiagFormat_SetAddInfo(r, AddInfo);
	return r;	
}

/*
Validate the database name.
The supported databases are loaded at runtime
from the zserver.ini file.  If the database name
requested by the client is not in this list
send them a diagnostic message.
We are only supporting a single database name
at this time!!
*/
BOOLEAN zserver_GetDatabaseList(PZSERVER z, PCHR d)
{
	CHR TempString[MAXPROFILESTR];
	CHR TempName[MAXPROFILESTR];
	INT i;
	
	if((!d)||(d[0]=='\0'))
		return FALSE;

	GetPrivateProfileString(PROFILEGROUP,"DBList","",
		TempString,MAXPROFILESTR,(PCHR)GIniFile);
	if(!TempString || TempString[0]=='\0')
		return FALSE;
	for(i=1;i<=strbox_itemCount(TempString);i++) {
		strbox_item(TempString,i,TempName);
		if(!strcasecmp(TempName,d)) 
			return TRUE;
	}
	return FALSE;
	
}

void zserver_Error(PCHR Function, INT Code, PCHR Msg)
{
	if((Msg==NULL)||(Msg[0]=='\0'))
		fprintf(stderr, "zserver:%s():%s", Function, 
			ZSERVER_ERRMSG[Code]);
	else
		fprintf(stderr, "zserver:%s():%s[%s]\n", Function, 
			ZSERVER_ERRMSG[Code],Msg);
}

/*
Print helpful messages when ZSERVER_DEBUG is defined
Function=Function making the call
Code=Code corrisponding to a message defined in zserver.h
Msg=Extra message information
*/
void zserver_Status(PCHR Function, INT Code, PCHR Msg)
{
#ifdef ZSERVER_DEBUG
	if((Msg==NULL)||(Msg[0]=='\0'))
		fprintf(stderr, "zserver:%s():%s", Function, 
			ZSERVER_STATMSG[Code]);
	else
		fprintf(stderr, "zserver:%s():%s[%s]\n", Function, 
			ZSERVER_STATMSG[Code],Msg);
#endif
}

PSAPI_DBASE zserver_GetDatabase(PZSERVER z, PCHR DBName)
{
	INT4 i,Count;
	POSITION *Pos;
	PSAPI_DBASE DBase;

	if((Count=list_Length(z->DatabaseList))<=0)
		return NULL;

	Pos = list_First(z->DatabaseList);

	for(i=1;i<=Count;i++) {
		DBase = (PSAPI_DBASE)list_Retrieve(z->DatabaseList, Pos);
                if(DBase == NULL) {
                        continue;
                }
                if(!strcasecmp(DBase->Name, DBName))
			return DBase;
                Pos = list_Next(z->DatabaseList, Pos);
        }
	return NULL;
}
