/************************************************************************
Copyright Notice

Copyright (c) MCNC, Clearinghouse for Networked Information Discovery and
Retrieval, 1994. 

Permission to use, copy, modify, distribute, and sell this software and
its documentation, in whole or in part, for any purpose is hereby granted
without fee, provided that

1. The above copyright notice and this permission notice appear in all
copies of the software and related documentation. Notices of copyright
and/or attribution which appear at the beginning of any file included in
this distribution must remain intact. 

2. Users of this software agree to make their best efforts (a) to return
to MCNC any improvements or extensions that they make, so that these may
be included in future releases; and (b) to inform MCNC/CNIDR of noteworthy
uses of this software. 

3. The names of MCNC and Clearinghouse for Networked Information Discovery
and Retrieval may not be used in any advertising or publicity relating to
the software without the specific, prior written permission of MCNC/CNIDR. 

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 

IN NO EVENT SHALL MCNC/CNIDR BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR
IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
************************************************************************/

/*@@@
File:           zserver.h
Version:        1.02
Author:         Kevin Gamiel, kgamiel@cnidr.org
@@@*/

#ifndef ZSERVER_H
#define ZSERVER_H

#include "platform.h"
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "sapi.h"
#include "libz3950.h"

CHAR GIniFile[MAXPROFILESTR];
int pid;                        /* Process id */
 
#ifndef TRUE
#define TRUE 1
#endif
#ifndef NOTOK 
#define NOTOK 0
#endif
#ifndef FALSE
#define FALSE 0
#endif


#define ZSERVER_NOSERVEROBJ 0
#define ZSERVER_BADPORT1 1
#define ZSERVER_BADPORT2 2
#define ZSERVER_NOACCEPT 3
#define ZSERVER_BADSAPI1 4
#define ZSERVER_BADSAPI2 5
#define ZSERVER_NOSAPIDBS1 6
#define ZSERVER_NOSAPIDBS2 7
#define ZSERVER_NOZSDBS1 8
#define ZSERVER_NOZSDBS2 9
#define ZSERVER_NOMOUNT 10

PCHR ZSERVER_ERRMSG[]={
	"Unable to create zserver object!\n",
	"Unable to start server on port",
	"Is the port in use by inetd or another application?\n",
	"Unable to accept client connections",
	"Unable to initialize Search API\n",
	"Valid 'SAPI=' directive in zserver.ini?\n",
	"No databases are available via the CNIDR Search API\n",
	"Make sure you Search API configuration file is correct\n",
	"# Databases available via ZServer:",
	"Valid 'DBList=' directive in zserver.ini?\n",
	"Failed to mount SAPI database:"
};

#define ZSERVER_NETCHECK 0
#define ZSERVER_START 1
#define ZSERVER_STARTING 2
#define ZSERVER_STANDALONE 3
#define ZSERVER_INETD 4
#define ZSERVER_CONNECT 5
#define ZSERVER_INITREC 6
#define ZSERVER_SRCHREC 7
#define ZSERVER_PRESENTREC 8
#define ZSERVER_WAITING 9
#define ZSERVER_SAPIDBS 10
#define ZSERVER_ZSDBS 11
#define ZSERVER_MOUNT 12

PCHR ZSERVER_STATMSG[]={
        "Checking network status...\n",
	"ZServer starting (compiled in ZSERVER_DEBUG mode)...\n",
	"Starting Server...\n",
	"Entering STANDALONE mode...\n",
	"Entering INETD mode...\n",
	"Connection from ",
	"Received Initialize Request\n",
	"Received Search Request\n",
	"Received Present Request\n",
	"Waiting for client connection ",
	"# Databases available via CNIDR Search API:",
	"# Databases available via ZServer:",
	"Successfully mounted database:"
};

#define NO_PDU 0
#define INIT_OK 1
#define ACCESS_OK 2
#define SEARCH_OK 4
#define PRESENT_OK 8
#define MAXSESSIONSTRING 1024
#define PROFILEFN "zserver.ini"
#define PROFILEGROUP "Default"

#define ZSERVER_STATE_OK 0
#define ZSERVER_STATE_BADPROTVER 1
#define ZSERVER_STATE_SYSTEMERROR 2
#define ZSERVER_STATE_SEARCH 3

#define MSG_NOINDEX "System error.  Unable to open database index"
#define MSG_BADQUERYSTACK "Unsupported query type.  Unexpected item on query stack"
#define MSG_BADQUERYSTACKSIZE "Unsupported query type.  Unexpected item count on query stack"
#define MSG_BADBUILDATTR "Temporary system error.  Attribute build failed"
#define MSG_UNKNOWN "Temporary system error.  Reason unknown"

int state = NO_PDU;

typedef struct tagZSERVER {
	SESSION *Defaults;
	NETBOXPROFILE *Socket;
	INT CurrentState;
	LIST *RecordList;
	PZSTATS Stats;
	pid_t pid;
	LIST *DatabaseList;
	PSAPI sapi;
	PSAPI_RESULTSET ResultSet;
} ZSERVER;
typedef ZSERVER *PZSERVER;

PZSERVER zserver_Create();
void zserver_Destroy(PZSERVER z);
PZSERVER zserver_CreateInit(INT argc, PCHR *argv);
void zserver_SetDefaults(PZSERVER zserver);
void zserver_InitializeTraceLog(PSESSION S);
void zserver_RunForkingServer(PZSERVER Server);
void zserver_RunSingleServer(PZSERVER Server);
void zserver_ServeClient(PZSERVER Server);
void zserver_LogChildError(FILE *fp);
void zserver_UpdateAccessLog(char *File, char *S);
void stripWhiteSpace(char *S);
DATA_DIR *zserver_CreateResponsePDU(PZSERVER zserver, DATA_DIR *pdu);
void zserver_DestroyPDU(PZSERVER z, DATA_DIR *d);
void zserver_LogPDU(PZSERVER z, DATA_DIR *d);
PNETBOXPROFILE zserver_GetSocket(PZSERVER z);
PSESSION zserver_GetDefaults(PZSERVER z);
PZSTATS zserver_GetStats(PZSERVER z);
void zserver_SetState(PZSERVER zserver, INT State);
void zserver_SetSocket(PZSERVER z, PNETBOXPROFILE s);
void zserver_DestroyRecordList(LIST *l);
PRESPONSERECORDS zserver_CreateResponseRecords(PZSERVER zserver,
	DATA_DIR *pdu);
PNONSURROGATEDIAGNOSTIC zserver_CreateDiagnosticRecord(INT4 Condition,
	PCHR S);
BOOLEAN zserver_IsValidDatabase(PZSERVER z, PCHR d);
void zserver_Status(PCHR Function, INT Code, PCHR Msg);
void zserver_Error(PCHR Function, INT Code, PCHR Msg);
PSAPI_DBASE zserver_GetDatabase(PZSERVER z, PCHR DBName);

#endif /* ZSERVER_H */
